@@ -38,16 +38,16 @@ static bool peekProc(VOID *base, VOID *dest, SIZE_T len) {
 }
 
 static void about(HWND h) {
-	::MessageBox(h, L"Reads audio position information from COD4 (v 1.7.568)", L"Mumble COD4 Plugin", MB_OK);
+	::MessageBox(h, L"Reads audio position information from COD4 (v 1.7.568). IP:Port context without team discriminator.", L"Mumble COD4 Plugin", MB_OK);
 }
 
-
-static int fetch(float *pos, float *front, float *top) {
+static int fetch(float *avatar_pos, float *avatar_front, float *avatar_top, float *camera_pos, float *camera_front, float *camera_top, std::string &context, std::wstring &identity) {
 	float viewHor, viewVer;
 	char state;
+	char ccontext[128];
 
 	for (int i=0;i<3;i++)
-		pos[i]=front[i]=top[i]=0.0f;
+		avatar_pos[i]=avatar_front[i]=avatar_top[i]=0.0f;
 
 	bool ok;
 
@@ -79,15 +79,23 @@ static int fetch(float *pos, float *front, float *top) {
 	if (state != 4)
 		return true; // This results in all vectors beeing zero which tells mumble to ignore them.
 
-	ok = peekProc((BYTE *) 0x0072AFD0, pos+2, 4) &&	//Z
-	     peekProc((BYTE *) 0x0072AFE0, pos, 4) &&	//X
-	     peekProc((BYTE *) 0x0072AFF0, pos+1, 4) && //Y
+	ok = peekProc((BYTE *) 0x0072AFD0, avatar_pos+2, 4) &&	//Z
+	     peekProc((BYTE *) 0x0072AFE0, avatar_pos, 4) &&	//X
+	     peekProc((BYTE *) 0x0072AFF0, avatar_pos+1, 4) && //Y
 	     peekProc((BYTE *) 0x0072AF3C, &viewHor, 4) && //Hor
-	     peekProc((BYTE *) 0x0072AF38, &viewVer, 4); //Ver
+	     peekProc((BYTE *) 0x0072AF38, &viewVer, 4) && //Ver
+	     peekProc((BYTE *) 0x00956D88, ccontext, 128);
 
 	if (! ok)
 		return false;
 
+	/*
+	    Get context string; in this plugin this will be an
+	    ip:port (char 256 bytes) string
+	*/
+	ccontext[127] = 0;
+	context = std::string(ccontext);
+
 	// Scale Coordinates
 	/*
 	   Z-Value is increasing when heading north
@@ -99,8 +107,10 @@ static int fetch(float *pos, float *front, float *top) {
 	   40 units = 1 meter (not confirmed)
 	*/
 	for (int i=0;i<3;i++)
-		pos[i]/=40.0f; // Scale to meters
-	pos[0]*=(-1.0f); // Convert right to left handed
+		avatar_pos[i]/=40.0f; // Scale to meters
+	avatar_pos[0]*=(-1.0f); // Convert right to left handed
+
+	avatar_top[2] = -1; // Head movement is in front vector
 
 	// Calculate view unit vector
 	/*
@@ -118,9 +128,15 @@ static int fetch(float *pos, float *front, float *top) {
 	viewVer *= static_cast<float>(M_PI / 180.0f);
 	viewHor *= static_cast<float>(M_PI / 180.0f);
 
-	front[0] = -sin(viewHor) * cos(viewVer);
-	front[1] = -sin(viewVer);
-	front[2] = cos(viewHor) * cos(viewVer);
+	avatar_front[0] = -sin(viewHor) * cos(viewVer);
+	avatar_front[1] = -sin(viewVer);
+	avatar_front[2] = cos(viewHor) * cos(viewVer);
+
+	for (int i=0;i<3;i++) {
+		camera_pos[i] = avatar_pos[i];
+		camera_front[i] = avatar_front[i];
+		camera_top[i] = avatar_top[i];
+	}
 
 	return ok;
 }
@@ -135,8 +151,11 @@ static int trylock() {
 	if (!h)
 		return false;
 
-	float pos[3], front[3], top[3];
-	if (fetch(pos, front, top))
+	float apos[3], afront[3], atop[3], cpos[3], cfront[3], ctop[3];
+	std::string context;
+	std::wstring identity;
+
+	if (fetch(apos, afront, atop, cpos, cfront, ctop, context, identity))
 		return true;
 
 	CloseHandle(h);
@@ -151,14 +170,22 @@ static void unlock() {
 	}
 }
 
+static const std::wstring longdesc() {
+	return std::wstring(L"Supports Call of Duty 4 v1.7.568 only. Only supports context.");
+}
+
+static std::wstring description(L"Call of Duty 4 v1.7.568");
+static std::wstring shortname(L"Call of Duty 4");
+
 static MumblePlugin cod4plug = {
 	MUMBLE_PLUGIN_MAGIC,
-	L"Call of Duty 4 v1.7.568",
-	L"Call of Duty 4",
+	description,
+	shortname,
 	about,
 	NULL,
 	trylock,
 	unlock,
+	longdesc,
 	fetch
 };
 
