@@ -34,7 +34,8 @@
 #include "murmur_pch.h"
 
 class Channel;
-class Player;
+class User;
+class ServerUser;
 
 class ChanACL : public QObject {
 	private:
@@ -48,33 +49,47 @@ class ChanACL : public QObject {
 			Enter = 0x4,
 			Speak = 0x8,
 			MuteDeafen = 0x10,
-			MoveKick = 0x20,
+			Move = 0x20,
 			MakeChannel = 0x40,
 			LinkChannel = 0x80,
-			AltSpeak = 0x100,
+			Whisper = 0x100,
+			TextMessage = 0x200,
+			MakeTempChannel = 0x400,
+
+			// Root channel only
+			Kick = 0x10000,
+			Ban = 0x20000,
+			Register = 0x40000,
+			SelfRegister = 0x80000,
+
 			Cached = 0x8000000,
-			All = 0xffff
+			All = 0xf07ff
 		};
 
 		Q_DECLARE_FLAGS(Permissions, Perm)
 
 		typedef QHash<Channel *, Permissions> ChanCache;
-		typedef QHash<Player *, ChanCache * > ACLCache;
+		typedef QHash<User *, ChanCache * > ACLCache;
 
 		Channel *c;
 		bool bApplyHere;
 		bool bApplySubs;
 
-		int iPlayerId;
+		bool bInherited;
+
+		int iUserId;
 		QString qsGroup;
 		Permissions pAllow;
 		Permissions pDeny;
 
 		ChanACL(Channel *c);
-		static bool hasPermission(Player *p, Channel *c, QFlags<Perm> perm, ACLCache &cache);
-		static QString shortName(Perm p);
-		static QString permName(Perm p);
+#ifdef MURMUR
+		static bool hasPermission(ServerUser *p, Channel *c, QFlags<Perm> perm, ACLCache &cache);
+#else
 		static QString whatsThis(Perm p);
+#endif
+		static QString permName(QFlags<Perm> p);
+		static QString permName(Perm p);
 };
 
 Q_DECLARE_OPERATORS_FOR_FLAGS(ChanACL::Permissions)
