@@ -29,21 +29,21 @@
 */
 
 #include "Channel.h"
-#include "Player.h"
+#include "User.h"
 #include "Group.h"
 #include "ACL.h"
 
+#ifdef MUMBLE
 QHash<int, Channel *> Channel::c_qhChannels;
 QReadWriteLock Channel::c_qrwlChannels;
+#endif
 
-uint qHash(const Channel::qpPlayerLink & pl) {
-	return qHash(pl.first) + qHash(pl.second);
-}
-
-Channel::Channel(int id, QString name, QObject *p) : QObject(p) {
+Channel::Channel(int id, const QString &name, QObject *p) : QObject(p) {
 	iId = id;
+	iPosition = 0;
 	qsName = name;
 	bInheritACL = true;
+	bTemporary = false;
 	cParent = qobject_cast<Channel *>(p);
 	if (cParent)
 		cParent->addChannel(this);
@@ -67,18 +67,19 @@ Channel::~Channel() {
 	Q_ASSERT(children().count() == 0);
 }
 
+#ifdef MUMBLE
 Channel *Channel::get(int id) {
 	QReadLocker lock(&c_qrwlChannels);
 	return c_qhChannels.value(id);
 }
 
-Channel *Channel::add(int id, QString name, QObject *po) {
+Channel *Channel::add(int id, const QString &name) {
 	QWriteLocker lock(&c_qrwlChannels);
 
 	if (c_qhChannels.contains(id))
 		return NULL;
 
-	Channel *c = new Channel(id,name, po);
+	Channel *c = new Channel(id, name, NULL);
 	c_qhChannels.insert(id, c);
 	return c;
 }
@@ -87,8 +88,16 @@ void Channel::remove(Channel *c) {
 	QWriteLocker lock(&c_qrwlChannels);
 	c_qhChannels.remove(c->iId);
 }
+#endif
 
-bool Channel::isLinked(Channel *l) {
+bool Channel::lessThan(const Channel *first, const Channel *second) {
+	if (first->iPosition != second->iPosition)
+		return first->iPosition < second->iPosition;
+	else
+		return first->qsName < second->qsName;
+}
+
+bool Channel::isLinked(Channel *l) const {
 	return qhLinks.contains(l);
 }
 
@@ -107,48 +116,12 @@ void Channel::unlink(Channel *l) {
 		qhLinks.remove(l);
 		l->qsPermLinks.remove(this);
 		l->qhLinks.remove(this);
-
-		foreach(qpPlayerLink pl, qsPlayerLinks) {
-			if (pl.second == l)
-				qsPlayerLinks.remove(pl);
-		}
-		foreach(qpPlayerLink pl, l->qsPlayerLinks) {
-			if (pl.second == l)
-				qsPlayerLinks.remove(pl);
-		}
 	} else {
 		foreach(Channel *c, qhLinks.keys())
 			unlink(c);
 	}
 }
 
-void Channel::playerLink(Channel *l, Player *p) {
-	qpPlayerLink pl(p, l);
-	if (qsPlayerLinks.contains(pl))
-		return;
-
-	qsPlayerLinks.insert(pl);
-	l->qsPlayerLinks.insert(pl);
-	qhLinks[l]++;
-	l->qhLinks[this]++;
-}
-
-void Channel::playerUnlink(Channel *l, Player *p) {
-	qpPlayerLink pl(p, l);
-	if (! qsPlayerLinks.contains(pl))
-		return;
-
-	qsPlayerLinks.remove(pl);
-	l->qsPlayerLinks.remove(qpPlayerLink(p, this));
-	qhLinks[l]--;
-	l->qhLinks[this]--;
-
-	if (qhLinks[l] == 0)
-		qhLinks.remove(l);
-	if (l->qhLinks[this] == 0)
-		l->qhLinks.remove(this);
-}
-
 QSet<Channel *> Channel::allLinks() {
 	QSet<Channel *> seen;
 	seen.insert(this);
@@ -171,6 +144,24 @@ QSet<Channel *> Channel::allLinks() {
 	return seen;
 }
 
+QSet<Channel *> Channel::allChildren() {
+	QSet<Channel *> seen;
+	if (! qlChannels.isEmpty()) {
+		QStack<Channel *> stack;
+		stack.push(this);
+
+		while (! stack.isEmpty()) {
+			Channel *c = stack.pop();
+			foreach(Channel *chld, c->qlChannels) {
+				seen.insert(chld);
+				if (! chld->qlChannels.isEmpty())
+					stack.append(chld);
+			}
+		}
+	}
+	return seen;
+}
+
 void Channel::addChannel(Channel *c) {
 	c->cParent = this;
 	c->setParent(this);
@@ -183,23 +174,20 @@ void Channel::removeChannel(Channel *c) {
 	qlChannels.removeAll(c);
 }
 
-void Channel::addPlayer(Player *p) {
+void Channel::addUser(User *p) {
 	if (p->cChannel)
-		p->cChannel->removePlayer(p);
-	ClientPlayer *cp = dynamic_cast<ClientPlayer *>(p);
-	if (cp)
-		cp->setParent(this);
+		p->cChannel->removeUser(p);
 	p->cChannel = this;
-	qlPlayers << p;
+	qlUsers << p;
 }
 
-void Channel::removePlayer(Player *p) {
-	foreach(qpPlayerLink pl, qsPlayerLinks) {
-		if (pl.first == p) {
-			qsPlayerLinks.remove(pl);
-			pl.second->qsPlayerLinks.remove(qpPlayerLink(p, this));
-		}
-	}
+void Channel::removeUser(User *p) {
+	qlUsers.removeAll(p);
+}
 
-	qlPlayers.removeAll(p);
+Channel::operator const QString() const {
+	return QString::fromLatin1("%1[%2:%3%4]").arg(qsName,
+	        QString::number(iId),
+	        QString::number(cParent ? cParent->iId : -1),
+	        bTemporary ? QLatin1String("*") : QLatin1String(""));
 }
