@@ -30,7 +30,6 @@
 
 #include "Message.h"
 #include "Connection.h"
-#include "PacketDataStream.h"
 
 #ifdef Q_OS_UNIX
 #include <sys/socket.h>
@@ -39,9 +38,6 @@
 #include <netinet/tcp.h>
 #endif
 
-int Connection::iReceiveLevel = 0;
-QSet<Connection *> Connection::qsReceivers;
-
 #ifdef Q_OS_WIN
 HANDLE Connection::hQoS = NULL;
 #endif
@@ -52,11 +48,8 @@ Connection::Connection(QObject *p, QSslSocket *qtsSock) : QObject(p) {
 	iPacketLength = -1;
 	bDisconnectedEmitted = false;
 
-	dUDPPingAvg = dUDPPingVar = 0.0L;
-	dTCPPingAvg = dTCPPingVar = 0.0L;
-	uiUDPPackets = uiTCPPackets = 0;
-
 	connect(qtsSocket, SIGNAL(error(QAbstractSocket::SocketError)), this, SLOT(socketError(QAbstractSocket::SocketError)));
+	connect(qtsSocket, SIGNAL(encrypted()), this, SIGNAL(encrypted()));
 	connect(qtsSocket, SIGNAL(readyRead()), this, SLOT(socketRead()));
 	connect(qtsSocket, SIGNAL(disconnected()), this, SLOT(socketDisconnected()));
 	connect(qtsSocket, SIGNAL(sslErrors(const QList<QSslError> &)), this, SLOT(socketSslErrors(const QList<QSslError> &)));
@@ -67,7 +60,6 @@ Connection::Connection(QObject *p, QSslSocket *qtsSock) : QObject(p) {
 }
 
 Connection::~Connection() {
-	qsReceivers.remove(this);
 #ifdef Q_OS_WIN
 	if (dwFlow && hQoS) {
 		if (! QOSRemoveSocketFromFlow(hQoS, 0, dwFlow, 0))
@@ -87,7 +79,7 @@ void Connection::setToS() {
 #elif defined(Q_OS_UNIX)
 	int val = 0xa0;
 	if (setsockopt(qtsSocket->socketDescriptor(), IPPROTO_IP, IP_TOS, &val, sizeof(val))) {
-		int val = 0x60;
+		val = 0x60;
 		if (setsockopt(qtsSocket->socketDescriptor(), IPPROTO_IP, IP_TOS, &val, sizeof(val)))
 			qWarning("Connection: Failed to set TOS for TCP Socket");
 	}
@@ -99,74 +91,18 @@ int Connection::activityTime() const {
 }
 
 void Connection::socketRead() {
-	// QSslSocket will, during writes, emit readyRead. Meaning we'd reenter from the message handlers.
-	// At the same time, DBus connections don't like getting multiple concurrent requests.
-	// So, this is a big workaround to serialize user requests.
-
-#if (QT_VERSION < 0x040400)
-	int iPrevLevel = iReceiveLevel;
-
-	iReceiveLevel = 2;
-
-	if (iPrevLevel == 2) {
-		// Recursive entry. Put on list and ignore.
-		qsReceivers.insert(this);
-		return;
-	} else if (iPrevLevel == 1) {
-		// We're iterating from the topmost one.
-		qsReceivers.remove(this);
-	}
-
-	int iAvailable = qtsSocket->bytesAvailable();
-
-
-	if (iPacketLength == -1) {
-		if (iAvailable < 3) {
-			iReceiveLevel = iPrevLevel;
-			return;
-		}
-
-		unsigned char a_ucBuffer[3];
-
-		qtsSocket->read(reinterpret_cast<char *>(a_ucBuffer), 3);
-		iPacketLength = ((a_ucBuffer[0] << 16) & 0xff0000) + ((a_ucBuffer[1] << 8) & 0xff00) + a_ucBuffer[2];
-		iAvailable -= 3;
-	}
-
-	if ((iPacketLength != -1) && (iAvailable >= iPacketLength)) {
-		QByteArray qbaBuffer = qtsSocket->read(iPacketLength);
-		iPacketLength = -1;
-		qtLastPacket.restart();
-		iAvailable -= iPacketLength;
-		if (iAvailable >= 3)
-			qsReceivers.insert(this);
-
-		emit message(qbaBuffer);
-	}
-
-	// At this point, the current *this might be destroyed.
-
-	if (iPrevLevel == 0) {
-		iReceiveLevel = 1;
-		QSet<Connection *>::const_iterator i = qsReceivers.constBegin();
-		while (i != qsReceivers.constEnd()) {
-			(*i)->socketRead();
-			i = qsReceivers.constBegin();
-		}
-	}
-	iReceiveLevel = iPrevLevel;
-#else
 	while (true) {
 		qint64 iAvailable = qtsSocket->bytesAvailable();
 		if (iPacketLength == -1) {
-			if (iAvailable < 3)
+			if (iAvailable < 4)
 				return;
 
-			unsigned char a_ucBuffer[3];
+			unsigned char a_ucBuffer[4];
 
-			qtsSocket->read(reinterpret_cast<char *>(a_ucBuffer), 3);
-			iPacketLength = ((a_ucBuffer[0] << 16) & 0xff0000) + ((a_ucBuffer[1] << 8) & 0xff00) + a_ucBuffer[2];
-			iAvailable -= 3;
+			qtsSocket->read(reinterpret_cast<char *>(a_ucBuffer), 4);
+			uiType = a_ucBuffer[0];
+			iPacketLength = ((a_ucBuffer[1] << 16) & 0xff0000) + ((a_ucBuffer[2] << 8) & 0xff00) + a_ucBuffer[3];
+			iAvailable -= 4;
 		}
 
 		if ((iPacketLength == -1) || (iAvailable < iPacketLength))
@@ -177,9 +113,8 @@ void Connection::socketRead() {
 		qtLastPacket.restart();
 		iAvailable -= iPacketLength;
 
-		emit message(qbaBuffer);
+		emit message(uiType, qbaBuffer);
 	}
-#endif
 }
 
 void Connection::socketError(QAbstractSocket::SocketError) {
@@ -204,49 +139,31 @@ void Connection::socketDisconnected() {
 	}
 }
 
-void Connection::sendMessage(const Message *mMsg) {
-	QByteArray qbaBuffer;
-	mMsg->messageToNetwork(qbaBuffer);
+void Connection::messageToNetwork(const ::google::protobuf::Message &msg, unsigned int msgType, QByteArray &cache) {
+	int len = msg.ByteSize();
+	if (len > 0x7fffff)
+		return;
+	cache.resize(len + 4);
+	unsigned char *uc = reinterpret_cast<unsigned char *>(cache.data());
+	uc[0] = static_cast<unsigned char>(msgType);
+	uc[1] = static_cast<unsigned char>((len >> 16) & 0xFF);
+	uc[2] = static_cast<unsigned char>((len >> 8) & 0xFF);
+	uc[3] = static_cast<unsigned char>(len & 0xFF);
 
-	sendMessage(qbaBuffer);
+	msg.SerializeToArray(uc + 4, len);
+}
+
+void Connection::sendMessage(const ::google::protobuf::Message &msg, unsigned int msgType, QByteArray &cache) {
+	if (cache.isEmpty()) {
+		messageToNetwork(msg, msgType, cache);
+	}
+
+	sendMessage(cache);
 }
 
 void Connection::sendMessage(const QByteArray &qbaMsg) {
-	STACKVAR(unsigned char, a_ucBuffer, 3 + qbaMsg.size());
-
-	if (qtsSocket->state() != QAbstractSocket::ConnectedState)
-		return;
-
-	if (! qtsSocket->isEncrypted())
-		return;
-
-	if (qbaMsg.size() > 0xfffff) {
-		qFatal("Connection: Oversized message (%d bytes)", qbaMsg.size());
-	}
-
-	a_ucBuffer[0]=static_cast<unsigned char>((qbaMsg.size() >> 16) & 0xff);
-	a_ucBuffer[1]=static_cast<unsigned char>((qbaMsg.size() >> 8) & 0xff);
-	a_ucBuffer[2]=static_cast<unsigned char>(qbaMsg.size() & 0xff);
-	memcpy(a_ucBuffer + 3, qbaMsg.constData(), qbaMsg.size());
-
-#if (QT_VERSION < 0x040400)
-	int iPrevLevel = iReceiveLevel;
-	iReceiveLevel = 2;
-#endif
-
-	qtsSocket->write(reinterpret_cast<const char *>(a_ucBuffer), 3 + qbaMsg.size());
-
-#if (QT_VERSION < 0x040400)
-	if (iPrevLevel == 0) {
-		iReceiveLevel = 1;
-		QSet<Connection *>::const_iterator i = qsReceivers.constBegin();
-		while (i != qsReceivers.constEnd()) {
-			(*i)->socketRead();
-			i = qsReceivers.constBegin();
-		}
-	}
-	iReceiveLevel = iPrevLevel;
-#endif
+	if (! qbaMsg.isEmpty())
+		qtsSocket->write(qbaMsg);
 }
 
 void Connection::forceFlush() {
@@ -273,14 +190,6 @@ void Connection::disconnectSocket(bool force) {
 		qtsSocket->disconnectFromHost();
 }
 
-void Connection::updatePing(double &avg, double &var, quint32 &samples, quint64 usec) {
-	samples++;
-	double x = static_cast<double>(usec) / 1000.0;
-	double delta = x - avg;
-	avg += delta / static_cast<double>(samples);
-	var += delta * (x - avg);
-};
-
 QHostAddress Connection::peerAddress() const {
 	return qtsSocket->peerAddress();
 }
@@ -290,7 +199,11 @@ quint16 Connection::peerPort() const {
 }
 
 QList<QSslCertificate> Connection::peerCertificateChain() const {
-	return qtsSocket->peerCertificateChain();
+	const QSslCertificate cert = qtsSocket->peerCertificate();
+	if (cert.isNull())
+		return QList<QSslCertificate>();
+	else
+		return qtsSocket->peerCertificateChain() << cert;
 }
 
 QSslCipher Connection::sessionCipher() const {
