@@ -34,8 +34,10 @@
 #define MUMXTEXT(X) #X
 #define MUMTEXT(X) MUMXTEXT(X)
 
-#define MUMBLE_VERSION 1.1.8
-#if 0
+#ifndef MUMBLE_VERSION
+#define MUMBLE_VERSION 1.2.0~200909021545-b58726
+#endif
+#ifndef MUMBLE_VERSION
 #define MUMBLE_RELEASE "Compiled " __DATE__ " " __TIME__
 #else
 #define MUMBLE_RELEASE MUMTEXT(MUMBLE_VERSION)
