@@ -81,6 +81,7 @@ AudioInputDialog::AudioInputDialog(Settings &st) : ConfigWidget(st) {
 
 	qcbDevice->view()->setTextElideMode(Qt::ElideRight);
 
+	on_qcbPushClick_clicked(g.s.bPushClick);
 	on_Tick_timeout();
 }
 
@@ -104,12 +105,15 @@ void AudioInputDialog::load(const Settings &r) {
 	if (i >= 0)
 		loadComboBox(qcbSystem, i);
 
+	qlePushClickPathOn->setText(r.qsPushClickOn);
+	qlePushClickPathOff->setText(r.qsPushClickOff);
+
 	loadComboBox(qcbTransmit, r.atTransmit);
 	loadSlider(qsTransmitHold, r.iVoiceHold);
-	loadSlider(qsTransmitMin, lroundf(r.fVADmin * 32767.0f));
-	loadSlider(qsTransmitMax, lroundf(r.fVADmax * 32767.0f));
-	loadSlider(qsFrames, r.iFramesPerPacket);
-	loadSlider(qsDoublePush, lround(static_cast<double>(r.uiDoublePush) / 1000.f));
+	loadSlider(qsTransmitMin, iroundf(r.fVADmin * 32767.0f));
+	loadSlider(qsTransmitMax, iroundf(r.fVADmax * 32767.0f));
+	loadSlider(qsFrames, (r.iFramesPerPacket == 1) ? 1 : (r.iFramesPerPacket/2 + 1));
+	loadSlider(qsDoublePush, iroundf(static_cast<float>(r.uiDoublePush) / 1000.f));
 
 	if (r.vsVAD == Settings::Amplitude)
 		qrbAmplitude->setChecked(true);
@@ -118,28 +122,42 @@ void AudioInputDialog::load(const Settings &r) {
 
 	loadCheckBox(qcbPushClick, r.bPushClick);
 	loadSlider(qsQuality, r.iQuality);
-	loadSlider(qsNoise, - r.iNoiseSuppress);
+	if (r.iNoiseSuppress != 0)
+		loadSlider(qsNoise, - r.iNoiseSuppress);
+	else
+		loadSlider(qsNoise, 14);
+
 	loadSlider(qsAmp, 20000 - r.iMinLoudness);
 	loadSlider(qsIdle, r.iIdleTime);
-	loadCheckBox(qcbEcho, r.bEcho);
+
+	int echo = 0;
+	if (r.bEcho)
+		echo = r.bEchoMulti ? 2 : 1;
+
+	loadComboBox(qcbEcho, echo);
 }
 
 void AudioInputDialog::save() const {
 	s.iQuality = qsQuality->value();
-	s.iNoiseSuppress = - qsNoise->value();
+	s.iNoiseSuppress = (qsNoise->value() == 14) ? 0 : - qsNoise->value();
 	s.iMinLoudness = 18000 - qsAmp->value() + 2000;
 	s.iVoiceHold = qsTransmitHold->value();
 	s.fVADmin = static_cast<float>(qsTransmitMin->value()) / 32767.0f;
 	s.fVADmax = static_cast<float>(qsTransmitMax->value()) / 32767.0f;
 	s.vsVAD = qrbSNR->isChecked() ? Settings::SignalToNoise : Settings::Amplitude;
 	s.iFramesPerPacket = qsFrames->value();
+	s.iFramesPerPacket = (s.iFramesPerPacket == 1) ? 1 : ((s.iFramesPerPacket-1) * 2);
 	s.uiDoublePush = qsDoublePush->value() * 1000;
-	s.bPushClick = qcbPushClick->isChecked();
 	s.atTransmit = static_cast<Settings::AudioTransmit>(qcbTransmit->currentIndex());
 	s.iIdleTime = qsIdle->value();
 
+	s.bPushClick = qcbPushClick->isChecked();
+	s.qsPushClickOn = qlePushClickPathOn->text();
+	s.qsPushClickOff = qlePushClickPathOff->text();
+
 	s.qsAudioInput = qcbSystem->currentText();
-	s.bEcho = qcbEcho->isChecked();
+	s.bEcho = qcbEcho->currentIndex() > 0;
+	s.bEchoMulti = qcbEcho->currentIndex() == 2;
 
 	if (AudioInputRegistrar::qmNew) {
 		AudioInputRegistrar *air = AudioInputRegistrar::qmNew->value(qcbSystem->currentText());
@@ -164,7 +182,7 @@ bool AudioInputDialog::expert(bool b) {
 }
 
 void AudioInputDialog::on_qsFrames_valueChanged(int v) {
-	qlFrames->setText(tr("%1 ms").arg(v*20));
+	qlFrames->setText(tr("%1 ms").arg((v==1) ? 10 : (v-1)*20));
 	updateBitrate();
 }
 
@@ -176,18 +194,26 @@ void AudioInputDialog::on_qsDoublePush_valueChanged(int v) {
 }
 
 void AudioInputDialog::on_qsTransmitHold_valueChanged(int v) {
-	float val = static_cast<float>(v * 20);
+	float val = static_cast<float>(v * 10);
 	val = val / 1000.0f;
 	qlTransmitHold->setText(tr("%1 s").arg(val, 0, 'f', 2));
 }
 
 void AudioInputDialog::on_qsQuality_valueChanged(int v) {
-	qlQuality->setText(QString::number(v));
+	qlQuality->setText(tr("%1 kb/s").arg(static_cast<float>(v) / 1000.0f, 0, 'f', 1));
 	updateBitrate();
 }
 
 void AudioInputDialog::on_qsNoise_valueChanged(int v) {
-	qlNoise->setText(tr("-%1 dB").arg(v));
+	QPalette pal;
+
+	if (v < 15) {
+		qlNoise->setText(tr("Off"));
+		pal.setColor(qlNoise->foregroundRole(), Qt::red);
+	} else {
+		qlNoise->setText(tr("-%1 dB").arg(v));
+	}
+	qlNoise->setPalette(pal);
 }
 
 void AudioInputDialog::on_qsAmp_valueChanged(int v) {
@@ -210,27 +236,22 @@ void AudioInputDialog::updateBitrate() {
 	int p = qsFrames->value();
 
 	int audiorate, overhead, posrate;
-	float f = static_cast<float>(q);
-	void *es;
 
-	es = speex_encoder_init(&speex_wb_mode);
-	speex_encoder_ctl(es,SPEEX_SET_VBR_QUALITY, &f);
-	speex_encoder_ctl(es,SPEEX_GET_BITRATE,&audiorate);
-	speex_encoder_destroy(es);
+	audiorate = q;
 
-	// 50 packets, in bits, IP + UDP + Crypt + type/id (Message header) + flags + seq
-	overhead = 50 * 8 * (20 + 8 + 4 + 3 + 1 + 2);
+	// 50 packets, in bits, IP + UDP + Crypt + type + seq
+	overhead = 100 * 8 * (20 + 8 + 4 + 1 + 2);
 
 	// TCP is 12 more bytes than UDP
 	if (NetworkConfig::TcpModeEnabled())
-		overhead += 50 * 8 * 12;
+		overhead += 100 * 8 * 12;
 
 	if (g.s.bTransmitPosition)
 		posrate = 12;
 	else
 		posrate = 0;
 
-	posrate = posrate * 50 * 8;
+	posrate = posrate * 100 * 8;
 
 	overhead = overhead / p;
 	posrate = posrate / p;
@@ -239,14 +260,67 @@ void AudioInputDialog::updateBitrate() {
 
 	QPalette pal;
 
-	if ((total / 8 > g.iMaxBandwidth) && g.uiSession) {
+	if (g.uiSession && (total > g.iMaxBandwidth)) {
 		pal.setColor(qlBitrate->foregroundRole(), Qt::red);
 	}
 
 	qlBitrate->setPalette(pal);
 
-	QString v = tr("%1kbit/s (Audio %2, Position %4, Overhead %3)").arg(total / 1000.0, 0, 'f', 1).arg(audiorate / 1000.0, 0, 'f', 1).arg(overhead / 1000.0, 0, 'f', 1).arg(posrate / 1000.0, 0, 'f', 1);
+	QString v = tr("%1kbit/s (Audio %2 %5, Position %4, Overhead %3)").arg(total / 1000.0, 0, 'f', 1).arg(audiorate / 1000.0, 0, 'f', 1).arg(overhead / 1000.0, 0, 'f', 1).arg(posrate / 1000.0, 0, 'f', 1).arg(::AudioInput::preferCELT(q,p) ? tr("CELT") : tr("Speex"));
 	qlBitrate->setText(v);
+
+	if (p == 1) {
+		qsQuality->setMinimum(24000);
+	} else {
+		qsQuality->setMinimum(8000);
+	}
+}
+
+void AudioInputDialog::on_qcbPushClick_clicked(bool b) {
+	qpbPushClickBrowseOn->setEnabled(b);
+	qpbPushClickBrowseOff->setEnabled(b);
+	qpbPushClickPreview->setEnabled(b);
+	qpbPushClickReset->setEnabled(b);
+	qlePushClickPathOn->setEnabled(b);
+	qlePushClickPathOff->setEnabled(b);
+	qlPushClickOn->setEnabled(b);
+	qlPushClickOff->setEnabled(b);
+}
+
+void AudioInputDialog::on_qpbPushClickBrowseOn_clicked() {
+	QString qsnew = AudioOutputSample::browseForSndfile();
+	if (!qsnew.isEmpty())
+		qlePushClickPathOn->setText(qsnew);
+}
+
+void AudioInputDialog::on_qpbPushClickBrowseOff_clicked() {
+	QString qsnew = AudioOutputSample::browseForSndfile();
+	if (!qsnew.isEmpty())
+		qlePushClickPathOff->setText(qsnew);
+}
+
+void AudioInputDialog::on_qpbPushClickPreview_clicked() {
+	AudioOutputPtr ao = g.ao;
+	if (ao) {
+		AudioOutputSample *s = ao->playSample(qlePushClickPathOn->text());
+		if (s)
+			connect(s, SIGNAL(playbackFinished()), this, SLOT(continuePlayback()));
+		else // If we fail to playback the first play on play at least off
+			ao->playSample(qlePushClickPathOff->text());
+
+	}
+}
+
+void AudioInputDialog::continuePlayback() {
+	AudioOutputPtr ao = g.ao;
+	if (ao) {
+		ao->playSample(qlePushClickPathOff->text());
+	}
+}
+
+void AudioInputDialog::on_qpbPushClickReset_clicked() {
+	qlePushClickPathOn->setText(Settings::cqsDefaultPushClickOn);
+	qlePushClickPathOff->setText(Settings::cqsDefaultPushClickOff);
 }
 
 void AudioInputDialog::on_qcbTransmit_currentIndexChanged(int v) {
@@ -296,9 +370,9 @@ void AudioInputDialog::on_Tick_timeout() {
 	abSpeech->iAbove = qsTransmitMax->value();
 
 	if (qrbAmplitude->isChecked()) {
-		abSpeech->iValue = lroundf((32767.f/96.0f) * (96.0f + ai->dPeakMic));
+		abSpeech->iValue = iroundf((32767.f/96.0f) * (96.0f + ai->dPeakMic));
 	} else {
-		abSpeech->iValue = lroundf(ai->fSpeechProb * 32767.0f);
+		abSpeech->iValue = iroundf(ai->fSpeechProb * 32767.0f);
 	}
 	abSpeech->update();
 }
@@ -345,16 +419,16 @@ void AudioOutputDialog::load(const Settings &r) {
 		loadComboBox(qcbSystem, i);
 
 	loadSlider(qsDelay, r.iOutputDelay);
-	loadSlider(qsVolume, static_cast<int>(lroundf(r.fVolume * 100.0f)));
-	loadSlider(qsOtherVolume, static_cast<int>(lroundf(r.fOtherVolume * 100.0f)));
+	loadSlider(qsVolume, iroundf(r.fVolume * 100.0f));
+	loadSlider(qsOtherVolume, iroundf(r.fOtherVolume * 100.0f));
 	loadSlider(qsJitter, r.iJitterBufferSize);
 	loadComboBox(qcbLoopback, r.lmLoopMode);
 	loadSlider(qsPacketDelay, static_cast<int>(r.dMaxPacketDelay));
-	loadSlider(qsPacketLoss, static_cast<int>(lroundf(r.dPacketLoss * 100.0f)));
-	loadSlider(qsMinDistance, lroundf(r.fAudioMinDistance * 10.0f));
-	loadSlider(qsMaxDistance, lroundf(r.fAudioMaxDistance * 10.0f));
-	loadSlider(qsMaxDistVolume, lroundf(r.fAudioMaxDistVolume * 100.0f));
-	loadSlider(qsBloom, lround(r.fAudioBloom * 100.0f));
+	loadSlider(qsPacketLoss, iroundf(r.dPacketLoss * 100.0f));
+	loadSlider(qsMinDistance, iroundf(r.fAudioMinDistance * 10.0f));
+	loadSlider(qsMaxDistance, iroundf(r.fAudioMaxDistance * 10.0f));
+	loadSlider(qsMaxDistVolume, iroundf(r.fAudioMaxDistVolume * 100.0f));
+	loadSlider(qsBloom, iroundf(r.fAudioBloom * 100.0f));
 	loadCheckBox(qcbHeadphones, r.bPositionalHeadphone);
 	loadCheckBox(qcbPositional, r.bPositionalAudio);
 }
@@ -418,7 +492,7 @@ void AudioOutputDialog::on_qcbSystem_currentIndexChanged(int) {
 }
 
 void AudioOutputDialog::on_qsJitter_valueChanged(int v) {
-	qlJitter->setText(tr("%1 ms").arg(v*20));
+	qlJitter->setText(tr("%1 ms").arg(v*10));
 }
 
 void AudioOutputDialog::on_qsVolume_valueChanged(int v) {
@@ -445,7 +519,7 @@ void AudioOutputDialog::on_qsPacketLoss_valueChanged(int v) {
 }
 
 void AudioOutputDialog::on_qsDelay_valueChanged(int v) {
-	qlDelay->setText(tr("%1ms").arg(v*20));
+	qlDelay->setText(tr("%1ms").arg(v*10));
 }
 
 void AudioOutputDialog::on_qcbLoopback_currentIndexChanged(int v) {
