@@ -32,7 +32,7 @@
 #include "AudioOutput.h"
 #include "ServerHandler.h"
 #include "MainWindow.h"
-#include "Player.h"
+#include "User.h"
 #include "Plugins.h"
 #include "Message.h"
 #include "Global.h"
@@ -84,51 +84,74 @@ AudioInputPtr AudioInputRegistrar::newFromChoice(QString choice) {
 
 
 AudioInput::AudioInput() {
-	speex_bits_init(&sbBits);
-	speex_bits_reset(&sbBits);
-	iFrames = 0;
+	adjustBandwidth(g.iMaxBandwidth, iAudioQuality, iAudioFrames);
 
-	esEncState=speex_encoder_init(&speex_wb_mode);
-	speex_encoder_ctl(esEncState,SPEEX_GET_FRAME_SIZE,&iFrameSize);
+	g.iAudioBandwidth = getNetworkBandwidth(iAudioQuality, iAudioFrames);
+
+	if (preferCELT(iAudioQuality, iAudioFrames))
+		umtType = MessageHandler::UDPVoiceCELTAlpha;
+	else
+		umtType = MessageHandler::UDPVoiceSpeex;
+
+	cCodec = NULL;
+	ceEncoder = NULL;
+
+	if (umtType != MessageHandler::UDPVoiceSpeex) {
+		iSampleRate = SAMPLE_RATE;
+		iFrameSize = SAMPLE_RATE / 100;
+
+		esSpeex = NULL;
+		qWarning("AudioInput: %d bits/s, %d hz, %d sample CELT", iAudioQuality, iSampleRate, iFrameSize);
+	} else {
+		iAudioFrames /= 2;
+
+		speex_bits_init(&sbBits);
+		speex_bits_reset(&sbBits);
+		esSpeex = speex_encoder_init(speex_lib_get_mode(SPEEX_MODEID_UWB));
+		speex_encoder_ctl(esSpeex,SPEEX_GET_FRAME_SIZE,&iFrameSize);
+		speex_encoder_ctl(esSpeex,SPEEX_GET_SAMPLING_RATE,&iSampleRate);
+
+		int iArg=1;
+		speex_encoder_ctl(esSpeex,SPEEX_SET_VBR, &iArg);
+
+		iArg = 0;
+		speex_encoder_ctl(esSpeex,SPEEX_SET_VAD, &iArg);
+		speex_encoder_ctl(esSpeex,SPEEX_SET_DTX, &iArg);
+
+		float fArg=8.0;
+		speex_encoder_ctl(esSpeex,SPEEX_SET_VBR_QUALITY, &fArg);
+
+		iArg = iAudioQuality;
+		speex_encoder_ctl(esSpeex, SPEEX_SET_VBR_MAX_BITRATE, &iArg);
+
+		iArg = 5;
+		speex_encoder_ctl(esSpeex,SPEEX_SET_COMPLEXITY, &iArg);
+		qWarning("AudioInput: %d bits/s, %d hz, %d sample Speex-UWB", iAudioQuality, iSampleRate, iFrameSize);
+	}
+	iEchoFreq = iMicFreq = iSampleRate;
 
 	mumble_drft_init(&fftTable, iFrameSize);
 
 	iFrameCounter = 0;
 	iSilentFrames = 0;
 	iHoldFrames = 0;
-	iBestBin = 0;
-
-	int iArg=1;
-	float fArg=0.0;
-	speex_encoder_ctl(esEncState,SPEEX_SET_VBR, &iArg);
-
-	iArg = 0;
-
-	speex_encoder_ctl(esEncState,SPEEX_SET_VAD, &iArg);
-	speex_encoder_ctl(esEncState,SPEEX_SET_DTX, &iArg);
-
-	fArg = static_cast<float>(g.s.iQuality);
-	speex_encoder_ctl(esEncState,SPEEX_SET_VBR_QUALITY, &fArg);
-	speex_encoder_ctl(esEncState,SPEEX_GET_BITRATE, &iArg);
-	speex_encoder_ctl(esEncState, SPEEX_SET_VBR_MAX_BITRATE, &iArg);
-
-	iArg = 5;
-	speex_encoder_ctl(esEncState,SPEEX_SET_COMPLEXITY, &iArg);
 
 	bResetProcessor = true;
 
+	bEchoMulti = false;
+
 	sppPreprocess = NULL;
 	sesEcho = NULL;
 	srsMic = srsEcho = NULL;
-	jb = jitter_buffer_init(10);
-	iJitterSeq = 1;
+	iJitterSeq = 0;
+	iMinBuffered = 1000;
 
 	psMic = new short[iFrameSize];
-	psSpeaker = new short[iFrameSize];
 	psClean = new short[iFrameSize];
 
+	psSpeaker = NULL;
+
 	iEchoChannels = iMicChannels = 0;
-	iEchoFreq = iMicFreq = SAMPLE_RATE;
 	iEchoFilled = iMicFilled = 0;
 	eMicFormat = eEchoFormat = SampleFloat;
 	iMicSampleSize = iEchoSampleSize = 0;
@@ -152,10 +175,18 @@ AudioInput::AudioInput() {
 AudioInput::~AudioInput() {
 	bRunning = false;
 	wait();
-	speex_bits_destroy(&sbBits);
-	speex_encoder_destroy(esEncState);
+
+	if (ceEncoder) {
+		cCodec->celt_encoder_destroy(ceEncoder);
+	} else if (esSpeex) {
+		speex_bits_destroy(&sbBits);
+		speex_encoder_destroy(esSpeex);
+	}
+
 	mumble_drft_clear(&fftTable);
-	jitter_buffer_destroy(jb);
+
+	foreach(short *buf, qlEchoFrames)
+		delete [] buf;
 
 	if (sppPreprocess)
 		speex_preprocess_state_destroy(sppPreprocess);
@@ -168,8 +199,9 @@ AudioInput::~AudioInput() {
 		speex_resampler_destroy(srsEcho);
 
 	delete [] psMic;
-	delete [] psSpeaker;
 	delete [] psClean;
+	if (psSpeaker)
+		delete [] psSpeaker;
 
 	if (pfMicInput)
 		delete [] pfMicInput;
@@ -306,19 +338,22 @@ void AudioInput::initializeMixer() {
 	if (pfOutput)
 		delete [] pfOutput;
 
-	if (iMicFreq != SAMPLE_RATE)
-		srsMic = speex_resampler_init(1, iMicFreq, SAMPLE_RATE, 3, &err);
+	if (iMicFreq != iSampleRate)
+		srsMic = speex_resampler_init(1, iMicFreq, iSampleRate, 3, &err);
 
-	iMicLength = (iFrameSize * iMicFreq) / SAMPLE_RATE;
+	iMicLength = (iFrameSize * iMicFreq) / iSampleRate;
 
 	pfMicInput = new float[iMicLength];
-	pfOutput = new float[iFrameSize];
+	pfOutput = new float[iFrameSize * qMax(1U,iEchoChannels)];
 
 	if (iEchoChannels > 0) {
-		if (iEchoFreq != SAMPLE_RATE)
-			srsEcho = speex_resampler_init(1, iEchoFreq, SAMPLE_RATE, 3, &err);
-		iEchoLength = (iFrameSize * iEchoFreq) / SAMPLE_RATE;
-		pfEchoInput = new float[iEchoLength];
+		bEchoMulti = g.s.bEchoMulti;
+		if (iEchoFreq != iSampleRate)
+			srsEcho = speex_resampler_init(bEchoMulti ? iEchoChannels : 1, iEchoFreq, iSampleRate, 3, &err);
+		iEchoLength = (iFrameSize * iEchoFreq) / iSampleRate;
+		iEchoMCLength = bEchoMulti ? iEchoLength * iEchoChannels : iEchoLength;
+		iEchoFrameSize = bEchoMulti ? iFrameSize * iEchoChannels : iFrameSize;
+		pfEchoInput = new float[iEchoMCLength];
 	} else {
 		srsEcho = NULL;
 		pfEchoInput = NULL;
@@ -327,8 +362,8 @@ void AudioInput::initializeMixer() {
 	imfMic = chooseMixer(iMicChannels, eMicFormat);
 	imfEcho = chooseMixer(iEchoChannels, eEchoFormat);
 
-	iMicSampleSize = iMicChannels * ((eMicFormat == SampleFloat) ? sizeof(float) : sizeof(short));
-	iEchoSampleSize = iEchoChannels * ((eEchoFormat == SampleFloat) ? sizeof(float) : sizeof(short));
+	iMicSampleSize = static_cast<int>(iMicChannels * ((eMicFormat == SampleFloat) ? sizeof(float) : sizeof(short)));
+	iEchoSampleSize = static_cast<int>(iEchoChannels * ((eEchoFormat == SampleFloat) ? sizeof(float) : sizeof(short)));
 
 	bResetProcessor = true;
 
@@ -365,18 +400,31 @@ void AudioInput::addMic(const void *data, unsigned int nsamp) {
 				psMic[j] = static_cast<short>(ptr[j] * mul);
 
 			if (iEchoChannels > 0) {
-				JitterBufferPacket jbp;
-				jbp.data = reinterpret_cast<char *>(psSpeaker);
-				jbp.len = iFrameSize * sizeof(short);
-				jbp.timestamp = 0;
-				jbp.span = 0;
-				jbp.sequence = 0;
-				jbp.user_data = 0;
+				short *echo = NULL;
 
-				spx_int32_t offs;
+				{
+					QMutexLocker l(&qmEcho);
 
-				jitter_buffer_get(jb, &jbp, 10, &offs);
-				jitter_buffer_tick(jb);
+					if (qlEchoFrames.isEmpty()) {
+						iJitterSeq = 0;
+						iMinBuffered = 1000;
+					} else {
+						iMinBuffered = qMin(iMinBuffered, qlEchoFrames.count());
+
+						if ((iJitterSeq > 100) && (iMinBuffered > 1)) {
+							iJitterSeq = 0;
+							iMinBuffered = 1000;
+							delete [] qlEchoFrames.takeFirst();
+						}
+						echo = qlEchoFrames.takeFirst();
+					}
+				}
+
+				if (echo) {
+					if (psSpeaker)
+						delete [] psSpeaker;
+					psSpeaker = echo;
+				}
 			}
 			encodeAudioFrame();
 		}
@@ -387,7 +435,17 @@ void AudioInput::addEcho(const void *data, unsigned int nsamp) {
 	while (nsamp > 0) {
 		unsigned int left = qMin(nsamp, iEchoLength - iEchoFilled);
 
-		imfEcho(pfEchoInput + iEchoFilled, data, left, iEchoChannels);
+		if (bEchoMulti) {
+			const unsigned int samples = left * iEchoChannels;
+			if (eEchoFormat == SampleFloat)
+				for (unsigned int i=0;i<samples;++i)
+					pfEchoInput[i] = reinterpret_cast<const float *>(data)[i];
+			else
+				for (unsigned int i=0;i<samples;++i)
+					pfEchoInput[i] = static_cast<float>(reinterpret_cast<const short *>(data)[i]) * (1.0f / 32768.f);
+		} else {
+			imfEcho(pfEchoInput + iEchoFilled, data, left, iEchoChannels);
+		}
 
 		iEchoFilled += left;
 		nsamp -= left;
@@ -402,103 +460,149 @@ void AudioInput::addEcho(const void *data, unsigned int nsamp) {
 		if (iEchoFilled == iEchoLength) {
 			iEchoFilled = 0;
 
-			STACKVAR(short, outbuff, iFrameSize);
 			float *ptr = srsEcho ? pfOutput : pfEchoInput;
 			if (srsEcho) {
 				spx_uint32_t inlen = iEchoLength;
 				spx_uint32_t outlen = iFrameSize;
-				speex_resampler_process_float(srsEcho, 0, pfEchoInput, &inlen, pfOutput, &outlen);
+				speex_resampler_process_interleaved_float(srsEcho, pfEchoInput, &inlen, pfOutput, &outlen);
 			}
+
+			short *outbuff = new short[iEchoFrameSize];
+
 			const float mul = 32768.f;
-			for (int j=0;j<iFrameSize;++j)
+			for (unsigned int j=0;j<iEchoFrameSize;++j)
 				outbuff[j] = static_cast<short>(ptr[j] * mul);
 
-			JitterBufferPacket jbp;
-			jbp.data = reinterpret_cast<char *>(outbuff);
-			jbp.len = iFrameSize * sizeof(short);
-			jbp.timestamp = ++iJitterSeq * 10;
-			jbp.span = 10;
-			jbp.sequence = static_cast<unsigned short>(iJitterSeq);
-			jbp.user_data = 0;
+			iJitterSeq = qMin(iJitterSeq+1,10000U);
 
-			jitter_buffer_put(jb, &jbp);
+			QMutexLocker l(&qmEcho);
+			qlEchoFrames.append(outbuff);
 		}
 	}
 }
 
-int AudioInput::getMaxBandwidth() {
-	int audiorate;
-
-	void *es;
-	float f = static_cast<float>(g.s.iQuality);
-	es = speex_encoder_init(&speex_wb_mode);
-	speex_encoder_ctl(es,SPEEX_SET_VBR_QUALITY, &f);
-	speex_encoder_ctl(es,SPEEX_GET_BITRATE,&audiorate);
-	speex_encoder_destroy(es);
-
-	audiorate /= 400/g.s.iFramesPerPacket;
-
-	// Overhead
-	audiorate += 20 + 8 + 4 + 3 + 1 + 2;
-
-	if (g.s.bTransmitPosition)
-		audiorate += 12;
-
-	if (NetworkConfig::TcpModeEnabled())
-		audiorate += 12;
-
-	audiorate = (audiorate * 50) / g.s.iFramesPerPacket;
-
-	return audiorate;
+bool AudioInput::preferCELT(int bitrate, int frames) {
+	return ((bitrate >= 32000) || (frames == 1));
 }
 
-void AudioInput::setMaxBandwidth(int bytespersec) {
-	int audiorate;
-	int baserate;
+void AudioInput::adjustBandwidth(int bitspersec, int &bitrate, int &frames) {
+	frames = g.s.iFramesPerPacket;
+	bitrate = g.s.iQuality;
 
-	void *es;
-	es = speex_encoder_init(&speex_wb_mode);
-
-	float f = static_cast<float>(g.s.iQuality);
-
-	do {
-		speex_encoder_ctl(es, SPEEX_SET_VBR_QUALITY, &f);
-		speex_encoder_ctl(es, SPEEX_GET_BITRATE, &baserate);
-		audiorate = baserate;
-
-		audiorate /= 400/g.s.iFramesPerPacket;
-
-		// Overhead
-		audiorate += 20 + 8 + 4 + 3 + 1 + 2;
-
-		if (g.s.bTransmitPosition)
-			audiorate += 12;
-
-		if (NetworkConfig::TcpModeEnabled())
-			audiorate += 12;
-
-		audiorate = (audiorate * 50) / g.s.iFramesPerPacket;
-
-		if (f <= 1.9)
-			break;
-
-		if (audiorate > bytespersec) {
-			f -= 1.0f;
+	if (bitspersec == -1) {
+		// No limit
+	} else {
+		if (getNetworkBandwidth(bitrate, frames) >= bitspersec) {
+			if ((frames <= 4) && (bitspersec <= 32000))
+				frames = 4;
+			else if ((frames == 1) && (bitspersec <= 64000))
+				frames = 2;
+			else if ((frames == 2) && (bitspersec <= 48000))
+				frames = 4;
+			if (getNetworkBandwidth(bitrate, frames) >= bitspersec) {
+				do {
+					bitrate -= 1000;
+				} while ((bitrate > 8000) && (getNetworkBandwidth(bitrate, frames) >= bitspersec));
+			}
 		}
-	} while (audiorate > bytespersec);
-
-	speex_encoder_destroy(es);
-
-	speex_encoder_ctl(esEncState, SPEEX_SET_VBR_QUALITY, &f);
-	speex_encoder_ctl(esEncState, SPEEX_SET_VBR_MAX_BITRATE, &baserate);
-
-	g.iAudioBandwidth = audiorate;
-	g.iAudioQuality = lroundf(f);
+	}
+	if (bitrate <= 8000)
+		bitrate = 8000;
 }
 
+void AudioInput::setMaxBandwidth(int bitspersec) {
+	if (bitspersec == g.iMaxBandwidth)
+		return;
+
+	int frames;
+	int bitrate;
+	adjustBandwidth(bitspersec, bitrate, frames);
+
+	g.iMaxBandwidth = bitspersec;
+
+	if (bitspersec != -1) {
+		if ((bitrate != g.s.iQuality) || (frames != g.s.iFramesPerPacket))
+			g.mw->msgBox(tr("Server maximum network bandwidth is only %1 kbit/s. Audio quality auto-adjusted to %2 kbit/s (%3ms)").arg(bitspersec / 1000).arg(bitrate / 1000).arg(frames*10));
+	}
+
+	AudioInputPtr ai = g.ai;
+	if (ai && (preferCELT(bitrate, frames) == (ai->umtType != MessageHandler::UDPVoiceSpeex))) {
+		g.iAudioBandwidth = getNetworkBandwidth(bitrate, frames);
+		ai->iAudioQuality = bitrate;
+		ai->iAudioFrames = frames;
+		return;
+	}
+	g.ai.reset();
+	while (! ai.unique()) {}
+	ai.reset();
+	g.ai = AudioInputRegistrar::newFromChoice(g.s.qsAudioInput);
+	if (g.ai)
+		g.ai->start(QThread::HighestPriority);
+}
+
+int AudioInput::getNetworkBandwidth(int bitrate, int frames) {
+	int overhead = 20 + 8 + 4 + 1 + 2 + (g.s.bTransmitPosition ? 12 : 0) + (NetworkConfig::TcpModeEnabled() ? 12 : 0);
+	overhead *= (800 / frames);
+	int bw = overhead + bitrate;
+
+	return bw;
+}
+
+
+/*
+ * FIXME: Experimental zero-latency callback-based preprocessing.
+
+extern "C" {
+	int speex_preprocess_run_cb(SpeexPreprocessState *st, float *ft);
+	SpeexPreprocessState *speex_preprocess_callback_init(int frame_size, int sampling_rate);
+};
+
+celt_int32_t celtBack(CELTEncoder *enc, void *rawdata, celt_int32_t format, celt_int32_t bits, celt_int32_t num, void *data) {
+	qWarning() << "CB" << enc << rawdata << format << bits << num << data;
+
+	static SpeexPreprocessState *st = NULL;
+	if (! st) {
+		int samp = 480;
+		int freq = 48000;
+
+		celt_encoder_ctl(enc, CELT_GET_SAMPLE_RATE, &freq);
+		celt_encoder_ctl(enc, CELT_GET_FRAME_SIZE, &samp);
+
+		qWarning() << "Init" << samp << freq;
+		st= speex_preprocess_callback_init(samp, freq);
+
+		int iArg;
+
+		iArg = 1;
+		speex_preprocess_ctl(st, SPEEX_PREPROCESS_SET_VAD, &iArg);
+		speex_preprocess_ctl(st, SPEEX_PREPROCESS_SET_AGC, &iArg);
+		speex_preprocess_ctl(st, SPEEX_PREPROCESS_SET_DENOISE, &iArg);
+		speex_preprocess_ctl(st, SPEEX_PREPROCESS_SET_DEREVERB, &iArg);
+
+		iArg = 30000;
+		speex_preprocess_ctl(st, SPEEX_PREPROCESS_SET_AGC_TARGET, &iArg);
+
+		iArg = 30000;
+		speex_preprocess_ctl(st, SPEEX_PREPROCESS_SET_AGC_TARGET, &iArg);
+
+		float v = 30000.0f / static_cast<float>(g.s.iMinLoudness);
+		iArg = iroundf(floorf(20.0f * log10f(v)));
+		speex_preprocess_ctl(st, SPEEX_PREPROCESS_SET_AGC_MAX_GAIN, &iArg);
+
+		iArg = g.s.iNoiseSuppress;
+		speex_preprocess_ctl(st, SPEEX_PREPROCESS_SET_NOISE_SUPPRESS, &iArg);
+	}
+	qWarning() << "Call";
+//	int res = 1;
+	int res = speex_preprocess_run_cb(st, (float *) rawdata);
+	qWarning() << "Result" << res;
+	return 1;
+}
+ */
+
 void AudioInput::encodeAudioFrame() {
 	int iArg;
-	ClientPlayer *p=ClientPlayer::get(g.uiSession);
+	ClientUser *p=ClientUser::get(g.uiSession);
 	int i;
 	float sum;
 	short max;
@@ -523,33 +627,7 @@ void AudioInput::encodeAudioFrame() {
 		max = static_cast<short>(abs(psMic[i]) > max ? abs(psMic[i]) : max);
 	dMaxMic = max;
 
-	if (g.bEchoTest) {
-		STACKVAR(float, fft, iFrameSize);
-		STACKVAR(float, power, iFrameSize);
-		float scale = 1.f / static_cast<float>(iFrameSize);
-		for (i=0;i<iFrameSize;i++)
-			fft[i] = static_cast<float>(psMic[i]) * scale;
-		mumble_drft_forward(&fftTable, fft);
-		float mp = 0.0f;
-		int bin = 0;
-		power[0]=power[1]=0.0f;
-		for (i=2;i < iFrameSize / 2;i++) {
-			power[i] = sqrtf(fft[2*i]*fft[2*i]+fft[2*i-1]*fft[2*i-1]);
-			if (power[i] > mp) {
-				bin = i;
-				mp = power[i];
-			}
-		}
-		for (i=2;i< iFrameSize / 2;i++) {
-			if (power[i] * 2 > mp) {
-				if (i != bin)
-					bin = 0;
-			}
-		}
-		iBestBin = bin * 2;
-	}
-
-	if (iEchoChannels > 0) {
+	if (psSpeaker && (iEchoChannels > 0)) {
 		sum=1.0f;
 		for (i=0;i<iFrameSize;i++)
 			sum += static_cast<float>(psSpeaker[i] * psSpeaker[i]);
@@ -568,45 +646,41 @@ void AudioInput::encodeAudioFrame() {
 		if (sesEcho)
 			speex_echo_state_destroy(sesEcho);
 
-		sppPreprocess = speex_preprocess_state_init(iFrameSize, SAMPLE_RATE);
+		sppPreprocess = speex_preprocess_state_init(iFrameSize, iSampleRate);
 
 		iArg = 1;
 		speex_preprocess_ctl(sppPreprocess, SPEEX_PREPROCESS_SET_VAD, &iArg);
-		speex_preprocess_ctl(sppPreprocess, SPEEX_PREPROCESS_SET_DENOISE, &iArg);
 		speex_preprocess_ctl(sppPreprocess, SPEEX_PREPROCESS_SET_AGC, &iArg);
+		speex_preprocess_ctl(sppPreprocess, SPEEX_PREPROCESS_SET_DENOISE, &iArg);
 		speex_preprocess_ctl(sppPreprocess, SPEEX_PREPROCESS_SET_DEREVERB, &iArg);
 
 		iArg = 30000;
 		speex_preprocess_ctl(sppPreprocess, SPEEX_PREPROCESS_SET_AGC_TARGET, &iArg);
 
 		float v = 30000.0f / static_cast<float>(g.s.iMinLoudness);
-		iArg = lroundf(floorf(20.0f * log10f(v)));
+		iArg = iroundf(floorf(20.0f * log10f(v)));
 		speex_preprocess_ctl(sppPreprocess, SPEEX_PREPROCESS_SET_AGC_MAX_GAIN, &iArg);
 
 		iArg = g.s.iNoiseSuppress;
 		speex_preprocess_ctl(sppPreprocess, SPEEX_PREPROCESS_SET_NOISE_SUPPRESS, &iArg);
 
 		if (iEchoChannels > 0) {
-			sesEcho = speex_echo_state_init(iFrameSize, iFrameSize*10);
-			iArg = SAMPLE_RATE;
-			speex_echo_ctl(sesEcho, SPEEX_SET_SAMPLING_RATE, &iArg);
+			sesEcho = speex_echo_state_init_mc(iFrameSize, iFrameSize*10, 1, bEchoMulti ? iEchoChannels : 1);
+			iArg = iSampleRate;
+			speex_echo_ctl(sesEcho, SPEEX_ECHO_SET_SAMPLING_RATE, &iArg);
 			speex_preprocess_ctl(sppPreprocess, SPEEX_PREPROCESS_SET_ECHO_STATE, sesEcho);
 
-			jitter_buffer_reset(jb);
 			qWarning("AudioInput: ECHO CANCELLER ACTIVE");
 		} else {
 			sesEcho = NULL;
 		}
 
-		iFrames = 0;
-		speex_bits_reset(&sbBits);
-
 		bResetProcessor = false;
 	}
 
 	int iIsSpeech;
 
-	if (sesEcho) {
+	if (sesEcho && psSpeaker) {
 		speex_echo_cancellation(sesEcho, psMic, psSpeaker, psClean);
 		iIsSpeech=speex_preprocess_run(sppPreprocess, psClean);
 		psSource = psClean;
@@ -649,9 +723,9 @@ void AudioInput::encodeAudioFrame() {
 	else if (g.s.atTransmit == Settings::PushToTalk)
 		iIsSpeech = g.s.uiDoublePush && ((g.uiDoublePush < g.s.uiDoublePush) || (g.tDoublePush.elapsed() < g.s.uiDoublePush));
 
-	iIsSpeech = iIsSpeech || (g.iPushToTalk > 0) || (g.iAltSpeak > 0);
+	iIsSpeech = iIsSpeech || (g.iPushToTalk > 0);
 
-	if (g.s.bMute || ((g.s.lmLoopMode != Settings::Local) && p && p->bMute) || g.bPushToMute) {
+	if (g.s.bMute || ((g.s.lmLoopMode != Settings::Local) && p && (p->bMute || p->bSuppress)) || g.bPushToMute || (g.iTarget < 0)) {
 		iIsSpeech = 0;
 	}
 
@@ -659,19 +733,25 @@ void AudioInput::encodeAudioFrame() {
 		iSilentFrames = 0;
 	} else {
 		iSilentFrames++;
-		if (iSilentFrames > 200)
+		if (iSilentFrames > 500)
 			iFrameCounter = 0;
 	}
 
-	if (p)
-		p->setTalking(iIsSpeech, (g.iAltSpeak > 0));
+	if (p) {
+		if (! iIsSpeech)
+			p->setTalking(ClientUser::TalkingOff);
+		else if (g.iTarget == 0)
+			p->setTalking(ClientUser::Talking);
+		else
+			p->setTalking(ClientUser::TalkingWhisperChannel);
+	}
 
 	if (g.s.bPushClick && (g.s.atTransmit == Settings::PushToTalk)) {
 		AudioOutputPtr ao = g.ao;
 		if (iIsSpeech && ! bPreviousVoice && ao)
-			ao->playSine(400.0f,1200.0f,5);
+			ao->playSample(g.s.qsPushClickOn);
 		else if (ao && !iIsSpeech && bPreviousVoice && ao)
-			ao->playSine(620.0f,-1200.0f,5);
+			ao->playSample(g.s.qsPushClickOff);
 	}
 	if (! iIsSpeech && ! bPreviousVoice) {
 		iBitrate = 0;
@@ -682,69 +762,122 @@ void AudioInput::encodeAudioFrame() {
 		return;
 	}
 
-	bPreviousVoice = iIsSpeech;
-
 	tIdle.restart();
+	/*
+		int r = celt_encoder_ctl(ceEncoder, CELT_SET_POST_MDCT_CALLBACK(celtBack, NULL));
+		qWarning() << "Set Callback" << r;
+	*/
 
-	if (! iIsSpeech) {
-		memset(psMic, 0, sizeof(short) * iFrameSize);
-	}
+	unsigned char buffer[512];
+	int len;
 
-	if (g.s.bTransmitPosition && g.p && ! g.bCenterPosition && (iFrames == 0) && g.p->fetch()) {
-		QByteArray q;
-		QDataStream ds(&q, QIODevice::WriteOnly);
-		ds << g.p->fPosition[0];
-		ds << g.p->fPosition[1];
-		ds << g.p->fPosition[2];
+	if (umtType != MessageHandler::UDPVoiceSpeex) {
+		CELTCodec *switchto = NULL;
 
-		speex_bits_pack(&sbBits, 13, 5);
-		speex_bits_pack(&sbBits, q.size(), 4);
-
-		const unsigned char *d=reinterpret_cast<const unsigned char*>(q.data());
-		for (i=0;i<q.size();i++) {
-			speex_bits_pack(&sbBits, d[i], 8);
+		if ((! g.uiSession || (g.s.lmLoopMode == Settings::Local)) && (! g.qmCodecs.isEmpty())) {
+			// Use latest for local loopback
+			QMap<int, CELTCodec *>::const_iterator i = g.qmCodecs.constEnd();
+			--i;
+			switchto = i.value();
+		} else {
+			if (cCodec && bPreviousVoice) {
+				// Currently talking, don't switch unless you must.
+				int v = cCodec->bitstreamVersion();
+				if ((v == g.iCodecAlpha) || (v == g.iCodecBeta))
+					switchto = cCodec;
+			}
 		}
+		if (! switchto) {
+			switchto = g.qmCodecs.value(g.bPreferAlpha ? g.iCodecAlpha : g.iCodecBeta);
+			umtType = g.bPreferAlpha ? MessageHandler::UDPVoiceCELTAlpha : MessageHandler::UDPVoiceCELTBeta;
+			if (! switchto) {
+				switchto = g.qmCodecs.value(g.bPreferAlpha ? g.iCodecBeta : g.iCodecAlpha);
+				umtType = g.bPreferAlpha ? MessageHandler::UDPVoiceCELTBeta : MessageHandler::UDPVoiceCELTAlpha;
+			}
+		}
+		if (switchto != cCodec) {
+			if (cCodec && ceEncoder) {
+				cCodec->celt_encoder_destroy(ceEncoder);
+				ceEncoder = NULL;
+			}
+			cCodec = switchto;
+			if (cCodec) {
+				ceEncoder = cCodec->encoderCreate();
+			}
+		} else if (cCodec && ! bPreviousVoice) {
+			cCodec->celt_encoder_ctl(ceEncoder, CELT_RESET_STATE);
+		}
+
+		if (! cCodec)
+			return;
+
+		cCodec->celt_encoder_ctl(ceEncoder,CELT_SET_VBR_RATE(iAudioQuality));
+		len = cCodec->celt_encode(ceEncoder, psSource, NULL, buffer, qMin(iAudioQuality / 800, 127));
+		iBitrate = len * 100 * 8;
+	} else {
+		int vbr = 0;
+		speex_encoder_ctl(esSpeex, SPEEX_GET_VBR_MAX_BITRATE, &vbr);
+		if (vbr != iAudioQuality) {
+			vbr = iAudioQuality;
+			speex_encoder_ctl(esSpeex, SPEEX_SET_VBR_MAX_BITRATE, &vbr);
+		}
+
+		if (! bPreviousVoice)
+			speex_encoder_ctl(esSpeex, SPEEX_RESET_STATE, NULL);
+
+		speex_encode_int(esSpeex, psSource, &sbBits);
+		len = speex_bits_write(&sbBits, reinterpret_cast<char *>(buffer), 127);
+		iBitrate = len * 50 * 8;
+		speex_bits_reset(&sbBits);
 	}
 
-	speex_encode_int(esEncState, psSource, &sbBits);
-	iFrames++;
+	flushCheck(QByteArray(reinterpret_cast<const char *>(buffer), len), ! iIsSpeech);
 
-	speex_encoder_ctl(esEncState, SPEEX_GET_BITRATE, &iBitrate);
+	if (! iIsSpeech)
+		iBitrate = 0;
 
-	flushCheck();
+	bPreviousVoice = iIsSpeech;
 }
 
-void AudioInput::flushCheck() {
-	if (bPreviousVoice && iFrames < g.s.iFramesPerPacket)
+void AudioInput::flushCheck(const QByteArray &frame, bool terminator) {
+	qlFrames << frame;
+	if (! terminator && qlFrames.count() < iAudioFrames)
 		return;
 
-	int flags = 0;
-	if (g.iAltSpeak > 0)
-		flags += MessageSpeex::AltSpeak;
+	int flags = g.iTarget;
 	if (g.s.lmLoopMode == Settings::Server)
-		flags += MessageSpeex::LoopBack;
+		flags = 0x1f;
 
-	if (! bPreviousVoice)
-		flags += MessageSpeex::EndSpeech;
+	flags |= (umtType << 5);
 
-	flags += (iFrames - 1) << 4;
+	char data[1024];
+	data[0] = static_cast<unsigned char>(flags);
 
-	int len = speex_bits_nbytes(&sbBits);
-	QByteArray qba(len + 1, 0);
-	qba[0] = static_cast<unsigned char>(flags);
+	PacketDataStream pds(data + 1, 1023);
+	pds << iFrameCounter - qlFrames.count();
 
-	speex_bits_write(&sbBits, qba.data() + 1, len);
+	if (terminator)
+		qlFrames << QByteArray();
 
-	MessageSpeex msPacket;
-	msPacket.qbaSpeexPacket = qba;
-	msPacket.iSeq = iFrameCounter;
-
-	if (g.s.lmLoopMode == Settings::Local) {
-		LoopPlayer::lpLoopy.addFrame(qba, msPacket.iSeq);
-	} else if (g.sh) {
-		g.sh->sendMessage(&msPacket);
+	for (int i=0;i<qlFrames.count(); ++i) {
+		const QByteArray &qba = qlFrames.at(i);
+		unsigned char head = static_cast<unsigned char>(qba.size());
+		if (i < qlFrames.count() - 1)
+			head |= 0x80;
+		pds.append(head);
+		pds.append(qba.constData(), qba.size());
 	}
 
-	iFrames = 0;
-	speex_bits_reset(&sbBits);
+	if (g.s.bTransmitPosition && g.p && ! g.bCenterPosition && g.p->fetch()) {
+		pds << g.p->fPosition[0];
+		pds << g.p->fPosition[1];
+		pds << g.p->fPosition[2];
+	}
+
+	if (g.s.lmLoopMode == Settings::Local)
+		LoopUser::lpLoopy.addFrame(QByteArray(data, pds.size() + 1));
+	else if (g.sh)
+		g.sh->sendMessage(data, pds.size() + 1);
+
+	qlFrames.clear();
 }
