@@ -84,15 +84,18 @@ class AudioInput : public QThread {
 		unsigned int iMicFreq, iEchoFreq;
 		unsigned int iMicLength, iEchoLength;
 		unsigned int iMicSampleSize, iEchoSampleSize;
+		unsigned int iEchoMCLength, iEchoFrameSize;
 
+		bool bEchoMulti;
 		int	iFrameSize;
 
 		QMutex qmSpeex;
-		SpeexBits sbBits;
 		SpeexPreprocessState *sppPreprocess;
 		SpeexEchoState *sesEcho;
 
-		void *esEncState;
+		CELTMode *cmMode;
+		CELTEncoder *ceEncoder;
+
 		drft_lookup fftTable;
 
 		short *psMic;
@@ -114,9 +117,8 @@ class AudioInput : public QThread {
 		int iSilentFrames;
 		int iHoldFrames;
 
-		int iFrames;
-
-		void flushCheck();
+		QList<QByteArray> qlFrames;
+		void flushCheck(const QByteArray &, bool terminator);
 
 		void initializeMixer();
 	signals:
@@ -130,8 +132,6 @@ class AudioInput : public QThread {
 		float dPeakMic, dPeakSpeaker, dPeakSignal, dMaxMic;
 		float fSpeechProb;
 
-		unsigned int iBestBin;
-
 		bool isTransmitting() {
 			return bPreviousVoice;
 		};
