@@ -30,10 +30,11 @@
 
 #include "AudioOutput.h"
 #include "AudioInput.h"
-#include "Player.h"
+#include "User.h"
 #include "Global.h"
 #include "Message.h"
 #include "Plugins.h"
+#include "PacketDataStream.h"
 
 // Remember that we cannot use static member classes that are not pointers, as the constructor
 // for AudioOutputRegistrar() might be called before they are initialized, as the constructor
@@ -87,21 +88,21 @@ bool AudioOutputRegistrar::usesOutputDelay() const {
 	return true;
 }
 
-AudioOutputPlayer::AudioOutputPlayer(const QString name) : qsName(name) {
+AudioOutputUser::AudioOutputUser(const QString name) : qsName(name) {
 	iBufferSize = 0;
 	pfBuffer = NULL;
 	pfVolume = NULL;
 	fPos[0]=fPos[1]=fPos[2]=0.0;
 }
 
-AudioOutputPlayer::~AudioOutputPlayer() {
+AudioOutputUser::~AudioOutputUser() {
 	if (pfBuffer)
 		delete [] pfBuffer;
 	if (pfVolume)
 		delete [] pfVolume;
 }
 
-void AudioOutputPlayer::resizeBuffer(unsigned int newsize) {
+void AudioOutputUser::resizeBuffer(unsigned int newsize) {
 	if (newsize > iBufferSize) {
 		float *n = new float[newsize];
 		if (pfBuffer) {
@@ -113,197 +114,255 @@ void AudioOutputPlayer::resizeBuffer(unsigned int newsize) {
 	}
 }
 
-AudioOutputSample::AudioOutputSample(const QString &filename, const QList<QByteArray> &packets, bool loop, unsigned int freq) : AudioOutputPlayer(filename) {
-	dsDecState=speex_decoder_init(&speex_wb_mode);
+SoundFile::SoundFile(const QString &fname) {
+	siInfo.frames = 0 ;
+	siInfo.channels = 1 ;
+	siInfo.samplerate = SAMPLE_RATE ;
+	siInfo.samplerate = 0 ;
+	siInfo.sections = 0 ;
+	siInfo.seekable = 0 ;
 
-	int iArg=1;
+	sfFile = NULL;
+
+	qfFile.setFileName(fname);
+
+	if (qfFile.open(QIODevice::ReadOnly)) {
+		static SF_VIRTUAL_IO svi = {&SoundFile::vio_get_filelen, &SoundFile::vio_seek, &SoundFile::vio_read, &SoundFile::vio_write, &SoundFile::vio_tell};
+
+		sfFile = sf_open_virtual(&svi, SFM_READ, &siInfo, this) ;
+	}
+}
+
+SoundFile::~SoundFile() {
+	if (sfFile)
+		sf_close(sfFile);
+}
+
+bool SoundFile::isOpen() const {
+	return (sfFile != NULL) && qfFile.isOpen();
+}
+
+int SoundFile::channels() const {
+	return siInfo.channels;
+}
+
+int SoundFile::samplerate() const {
+	return siInfo.samplerate;
+}
+
+int SoundFile::error() const {
+	return sf_error(sfFile);
+}
+
+QString SoundFile::strError() const {
+	return QLatin1String(sf_strerror(sfFile));
+}
+
+sf_count_t SoundFile::seek(sf_count_t frames, int whence) {
+	return sf_seek(sfFile, frames, whence);
+}
+
+sf_count_t SoundFile::read(float *ptr, sf_count_t items) {
+	return sf_read_float(sfFile, ptr, items);
+}
+
+sf_count_t SoundFile::vio_get_filelen(void *user_data) {
+	SoundFile *sf = reinterpret_cast<SoundFile *>(user_data);
+
+	if (! sf->qfFile.isOpen())
+		return -1;
+
+	return (sf->qfFile.size());
+}
+
+sf_count_t SoundFile::vio_seek(sf_count_t offset, int whence, void *user_data) {
+	SoundFile *sf = reinterpret_cast<SoundFile *>(user_data);
+
+	if (! sf->qfFile.isOpen())
+		return -1;
+
+	if (whence == SEEK_SET) {
+		sf->qfFile.seek(offset);
+	} else if (whence == SEEK_END) {
+		sf->qfFile.seek(sf->qfFile.size() - offset);
+	} else {
+		sf->qfFile.seek(sf->qfFile.pos() + offset);
+	}
+	return sf->qfFile.pos();
+}
+
+sf_count_t SoundFile::vio_read(void *ptr, sf_count_t count, void *user_data) {
+	SoundFile *sf = reinterpret_cast<SoundFile *>(user_data);
+
+	if (! sf->qfFile.isOpen())
+		return -1;
+
+	return sf->qfFile.read(reinterpret_cast<char *>(ptr), count);
+}
+
+sf_count_t SoundFile::vio_write(const void *ptr, sf_count_t count, void *user_data) {
+	SoundFile *sf = reinterpret_cast<SoundFile *>(user_data);
+
+	if (! sf->qfFile.isOpen())
+		return -1;
+
+	return sf->qfFile.write(reinterpret_cast<const char *>(ptr), count);
+}
+
+sf_count_t SoundFile::vio_tell(void *user_data) {
+	SoundFile *sf = reinterpret_cast<SoundFile *>(user_data);
+
+	if (! sf->qfFile.isOpen())
+		return -1;
+
+	return sf->qfFile.pos();
+}
+
+AudioOutputSample::AudioOutputSample(const QString &name, SoundFile *psndfile, bool loop, unsigned int freq) : AudioOutputUser(name) {
 	int err;
 
-	speex_decoder_ctl(dsDecState, SPEEX_SET_ENH, &iArg);
-	speex_decoder_ctl(dsDecState, SPEEX_GET_FRAME_SIZE, &iFrameSize);
+	sfHandle = psndfile;
+	iOutSampleRate = freq;
 
-	if (freq != SAMPLE_RATE)
-		srs = speex_resampler_init(1, SAMPLE_RATE, freq, 3, &err);
-	else
+	// Check if the file is good
+	if (sfHandle->channels() <= 0 || sfHandle->channels() > 2) {
+		sfHandle = NULL;
+		return;
+	}
+
+	/* qWarning() << "Channels: " << sfHandle->channels();
+	qWarning() << "Samplerate: " << sfHandle->samplerate();
+	qWarning() << "Target Sr.: " << freq;
+	qWarning() << "Format: " << sfHandle->format() << endl; */
+
+	// If the frequencies don't match initialize the resampler
+	if (sfHandle->samplerate() != freq) {
+		srs = speex_resampler_init(1, sfHandle->samplerate(), iOutSampleRate, 3, &err);
+		if (err != RESAMPLER_ERR_SUCCESS) {
+			qWarning() << "Initialize " << sfHandle->samplerate() << " to " << iOutSampleRate << " resampler failed!";
+			srs = NULL;
+			sfHandle = NULL;
+			return;
+		}
+	} else {
 		srs = NULL;
+	}
 
-	iOutputSize = lroundf(ceilf(static_cast<float>(iFrameSize * freq) / static_cast<float>(SAMPLE_RATE)));
-
-	iBufferOffset = iBufferFilled = iLastConsume = 0;
-
-	bLoop = false;
-
-	iPacketIndex = -1;
-
-	qlPackets = packets;
-
-	speex_bits_init(&sbBits);
-
+	iLastConsume = iBufferFilled = 0;
 	bLoop = loop;
 }
 
 AudioOutputSample::~AudioOutputSample() {
-	speex_decoder_destroy(dsDecState);
-	speex_bits_destroy(&sbBits);
+	if (sfHandle) {
+		delete sfHandle;
+		sfHandle = NULL;
+	}
 }
 
-QList<QByteArray> AudioOutputSample::getPacketsFromFile(const QString &filename) {
-	QList<QByteArray> packets;
+SoundFile* AudioOutputSample::loadSndfile(const QString &filename) {
+	SoundFile *sf;
 
-	QFile f(filename);
-	if (! f.open(QIODevice::ReadOnly)) {
-		qWarning("AudioOutputSample: Failed to open %s for reading", qPrintable(filename));
-		return QList<QByteArray>();
+	// Create the filehandle and do a quick check if everything is ok
+	sf = new SoundFile(filename);
+
+	if (! sf->isOpen()) {
+		qWarning() << "File " << filename << " failed to open";
+		delete sf;
+		return NULL;
 	}
 
-	QByteArray qbaSample = f.readAll();
-	f.close();
-
-	if (qbaSample.startsWith(QString::fromLatin1("OggS").toUtf8())) {
-		ogg_sync_state sync;
-		ogg_page page;
-		ogg_packet packet;
-		ogg_stream_state stream;
-		bool stream_init = false;
-		bool eos = false;
-		int speex_serialno = -1;
-		int packetno = -1;
-		bool header_ok = false;
-		int nframes = -1;
-		int extra_headers = 0;
-
-		ogg_sync_init(&sync);
-		char *data = ogg_sync_buffer(&sync, qbaSample.length());
-		memcpy(data, qbaSample.constData(), qbaSample.length());
-		ogg_sync_wrote(&sync, qbaSample.length());
-
-		while (ogg_sync_pageout(&sync, &page)==1) {
-			if (! stream_init)  {
-				ogg_stream_init(&stream, ogg_page_serialno(&page));
-				stream_init = true;
-			} else if (ogg_page_serialno(&page) != stream.serialno) {
-				ogg_stream_reset_serialno(&stream, ogg_page_serialno(&page));
-			}
-			ogg_stream_pagein(&stream, &page);
-			while (! eos && ogg_stream_packetout(&stream, &packet) == 1) {
-				if (packet.bytes >= 5 && memcmp(packet.packet, "Speex", 5)==0) {
-					speex_serialno = stream.serialno;
-				}
-				if (speex_serialno == -1 || stream.serialno != speex_serialno)
-					break;
-
-				++packetno;
-
-				if (packetno == 0) {
-					SpeexHeader *header = speex_packet_to_header(reinterpret_cast<char *>(packet.packet), packet.bytes);
-
-					if (header && header->speex_version_id == 1 && header->rate == 16000 && header->nb_channels == 1) {
-						header_ok = true;
-						extra_headers = header->extra_headers;
-						nframes = header->frames_per_packet;
-					} else {
-						break;
-					}
-				} else if (packetno <= 1 + extra_headers) {
-				} else {
-					if (packet.e_o_s)
-						eos = true;
-					packets << QByteArray(reinterpret_cast<const char *>(packet.packet), packet.bytes);
-				}
-			}
-		}
-		ogg_sync_clear(&sync);
-		if (stream_init)
-			ogg_stream_clear(&stream);
-
-		if (! header_ok) {
-			qWarning("AudioOutputSample: %s contained invalid data", qPrintable(filename));
-		}
-	} else {
-		qWarning("AudioOutputSample: %s is not an ogg file", qPrintable(filename));
+	if (sf->error() != SF_ERR_NO_ERROR) {
+		qWarning() << "File " << filename << " couldn't be loaded: " << sf->strError();
+		delete sf;
+		return NULL;
 	}
-	return packets;
+
+	if (sf->channels() <= 0 || sf->channels() > 2) {
+		qWarning() << "File " << filename << " contains " << sf->channels() << " Channels, only 1 or 2 are supported.";
+		delete sf;
+		return NULL;
+	}
+	return sf;
 }
 
 bool AudioOutputSample::needSamples(unsigned int snum) {
+	// Forward the buffer
 	for (unsigned int i=iLastConsume;i<iBufferFilled;++i)
 		pfBuffer[i-iLastConsume]=pfBuffer[i];
 	iBufferFilled -= iLastConsume;
-
 	iLastConsume = snum;
 
+
+	// Check if we can satisfy request with current buffer
 	if (iBufferFilled >= snum)
-		return bLastAlive;
+		return true;
+
+	// Calculate the required buffersize to hold the results
+	unsigned int iInputFrames = iroundf(ceilf(static_cast<float>(snum * sfHandle->samplerate()) / static_cast<float>(iOutSampleRate)));
+	unsigned int iInputSamples = iInputFrames * sfHandle->channels();
 
 	float *pOut;
-	STACKVAR(float, fOut, iFrameSize);
+	bool mix = sfHandle->channels() > 1;
+	STACKVAR(float, fOut, iInputSamples);
+	STACKVAR(float, fMix, iInputFrames);
 
+	bool eof = false;
+	sf_count_t read;
+	do {
+		resizeBuffer(iBufferFilled + snum);
 
-	while (iBufferFilled < snum) {
-		resizeBuffer(iBufferFilled + iOutputSize);
+		// If we need to resample or mix write to the buffer on stack
+		pOut = (srs || mix) ? fOut : pfBuffer + iBufferFilled;
 
-		pOut = (srs) ? fOut : pfBuffer + iBufferFilled;
-
-		if (speex_bits_remaining(&sbBits) < 43) {
-			++iPacketIndex;
-			if (iPacketIndex >= qlPackets.count()) {
-				if (qlPackets.isEmpty() || !bLoop) {
-					memset(pfBuffer + iBufferFilled, 0, sizeof(float) * (snum-iBufferFilled));
-					return false;
-				} else {
-					iPacketIndex = 0;
-				}
+		// Try to read all samples needed to satifsy this request
+		if ((read = sfHandle->read(pOut, iInputSamples)) < iInputSamples) {
+			if (sfHandle->error() != SF_ERR_NO_ERROR || !bLoop) {
+				// We reached the eof or encountered an error, stuff with zeroes
+				memset(pOut, 0, sizeof(float) * (iInputSamples - read));
+				read = iInputSamples;
+				eof = true;
+			} else {
+				sfHandle->seek(SEEK_SET, 0);
 			}
-
-			const QByteArray &qba = qlPackets[iPacketIndex];
-
-			speex_bits_set_bit_buffer(&sbBits, reinterpret_cast<void *>(const_cast<char *>(qba.constData())), qba.length());
-			speex_bits_rewind(&sbBits);
 		}
 
-		if (speex_decode(dsDecState, &sbBits, pOut) != 0) {
-			memset(pfBuffer + iBufferFilled, 0, sizeof(float) * (snum-iBufferFilled));
-			return false;
+		if (mix) { // Mix the channels (only two channels)
+			read /= 2;
+			// If we need to resample after this write to extra buffer
+			pOut = srs ? fMix : pfBuffer + iBufferFilled;
+			for (unsigned int i = 0; i < read; i++)
+				pOut[i] = (fOut[i*2] + fOut[i*2 + 1]) * 0.5f;
+
 		}
 
-		spx_uint32_t inlen = iFrameSize;
-		spx_uint32_t outlen = iOutputSize;
-		if (srs)
-			speex_resampler_process_float(srs, 0, fOut, &inlen, pfBuffer + iBufferFilled, &outlen);
+		spx_uint32_t inlen = read;
+		spx_uint32_t outlen = snum;
+		if (srs) // If necessary resample
+			speex_resampler_process_float(srs, 0, pOut, &inlen, pfBuffer + iBufferFilled, &outlen);
+
 		iBufferFilled += outlen;
-	}
-
-	return true;
+	} while (iBufferFilled < snum);
+	return !eof;
 }
 
-AudioOutputSpeech::AudioOutputSpeech(ClientPlayer *player, unsigned int freq) : AudioOutputPlayer(player->qsName) {
-	p = player;
-
-	dsDecState=speex_decoder_init(&speex_wb_mode);
-
-	int iArg=1;
+AudioOutputSpeech::AudioOutputSpeech(ClientUser *user, unsigned int freq) : AudioOutputUser(user->qsName) {
 	int err;
+	p = user;
 
-	speex_decoder_ctl(dsDecState, SPEEX_SET_ENH, &iArg);
-	speex_decoder_ctl(dsDecState, SPEEX_GET_FRAME_SIZE, &iFrameSize);
+	iFrameSize = SAMPLE_RATE / 100;
+	cmMode = celt_mode_create(SAMPLE_RATE, 1, iFrameSize, NULL);
+	cdDecoder = celt_decoder_create(cmMode);
 
 	if (freq != SAMPLE_RATE)
 		srs = speex_resampler_init(1, SAMPLE_RATE, freq, 3, &err);
 	else
 		srs = NULL;
 
-	iOutputSize = lroundf(ceilf(static_cast<float>(iFrameSize * freq) / static_cast<float>(SAMPLE_RATE)));
+	iOutputSize = iroundf(ceilf(static_cast<float>(iFrameSize * freq) / static_cast<float>(SAMPLE_RATE)));
 
 	iBufferOffset = iBufferFilled = iLastConsume = 0;
 	bLastAlive = true;
 
-	SpeexCallback sc;
-	sc.callback_id = 1;
-	sc.func = speexCallback;
-	sc.data = this;
-
-	speex_decoder_ctl(dsDecState, SPEEX_SET_USER_HANDLER, &sc);
-
 	iMissCount = 0;
 	iMissedFrames = 0;
 
@@ -311,50 +370,51 @@ AudioOutputSpeech::AudioOutputSpeech(ClientPlayer *player, unsigned int freq) :
 	int margin = g.s.iJitterBufferSize * iFrameSize;
 	jitter_buffer_ctl(jbJitter, JITTER_BUFFER_SET_MARGIN, &margin);
 
-	speex_bits_init(&sbBits);
+	fFadeIn = new float[iFrameSize];
+	fFadeOut = new float[iFrameSize];
+
+	float mul = M_PI / (2.0f * static_cast<float>(iFrameSize));
+	for (unsigned int i=0;i<iFrameSize;++i)
+		fFadeIn[i] = fFadeOut[iFrameSize-i-1] = sinf(i * mul);
 }
 
 AudioOutputSpeech::~AudioOutputSpeech() {
-	speex_decoder_destroy(dsDecState);
+	celt_decoder_destroy(cdDecoder);
+	celt_mode_destroy(cmMode);
+
 	jitter_buffer_destroy(jbJitter);
-	speex_bits_destroy(&sbBits);
-}
 
-int AudioOutputSpeech::speexCallback(SpeexBits *bits, void *, void *data) {
-	AudioOutputSpeech *aos=reinterpret_cast<AudioOutputSpeech *>(data);
-
-	int len=speex_bits_unpack_unsigned(bits, 4);
-
-	QByteArray qba(len, 0);
-	unsigned char *ptr = reinterpret_cast<unsigned char *>(qba.data());
-
-	for (int i=0;i<len;i++)
-		ptr[i]=static_cast<unsigned char>(speex_bits_unpack_unsigned(bits, 8));
-	QDataStream ds(qba);
-	ds >> aos->fPos[0];
-	ds >> aos->fPos[1];
-	ds >> aos->fPos[2];
-	aos->iMissCount = 0;
-
-	return 0;
+	delete [] fFadeIn;
+	delete [] fFadeOut;
 }
 
 void AudioOutputSpeech::addFrameToBuffer(const QByteArray &qbaPacket, unsigned int iSeq) {
 	QMutexLocker lock(&qmJitter);
 
-	if (qbaPacket.size() < 1)
+	if (qbaPacket.size() < 2)
 		return;
 
-	unsigned int flags = qbaPacket.at(0);
-	unsigned int frames = (flags >> 4) + 1;
+	PacketDataStream pds(qbaPacket);
 
-	JitterBufferPacket jbp;
-	jbp.data = const_cast<char *>(qbaPacket.constData());
-	jbp.len = qbaPacket.size();
-	jbp.span = iFrameSize * frames;
-	jbp.timestamp = iFrameSize * iSeq;
+	pds.next();
 
-	jitter_buffer_put(jbJitter, &jbp);
+	int frames = 0;
+	unsigned int header = 0;
+	do {
+		header = pds.next();
+		frames++;
+		pds.skip(header & 0x7f);
+	} while ((header & 0x80) && pds.isValid());
+
+	if (pds.isValid()) {
+		JitterBufferPacket jbp;
+		jbp.data = const_cast<char *>(qbaPacket.constData());
+		jbp.len = qbaPacket.size();
+		jbp.span = iFrameSize * frames;
+		jbp.timestamp = iFrameSize * iSeq;
+
+		jitter_buffer_put(jbJitter, &jbp);
+	}
 }
 
 bool AudioOutputSpeech::needSamples(unsigned int snum) {
@@ -368,77 +428,102 @@ bool AudioOutputSpeech::needSamples(unsigned int snum) {
 		return bLastAlive;
 
 	float *pOut;
-	STACKVAR(float, fOut, iFrameSize);
+	STACKVAR(float, fOut, iFrameSize + 4096);
 
+	bool nextalive = bLastAlive;
 
 	while (iBufferFilled < snum) {
 		resizeBuffer(iBufferFilled + iOutputSize);
 
-		pOut = (srs) ? fOut : pfBuffer + iBufferFilled;
+		pOut = (srs) ? fOut : (pfBuffer + iBufferFilled);
 
-		if (p == &LoopPlayer::lpLoopy)
-			LoopPlayer::lpLoopy.fetchFrames();
-
-
-		if (speex_decode(dsDecState, &sbBits, pOut) == 0) {
-			jitter_buffer_tick(jbJitter);
-			bLastAlive = true;
+		if (! bLastAlive) {
+			memset(pOut, 0, iFrameSize * sizeof(float));
 		} else {
-			QMutexLocker lock(&qmJitter);
+			int ts = jitter_buffer_get_pointer_timestamp(jbJitter);
 
+			if (p == &LoopUser::lpLoopy)
+				LoopUser::lpLoopy.fetchFrames();
 
-			char data[4096];
-			JitterBufferPacket jbp;
-			jbp.data = data;
-			jbp.len = 4096;
+			if (qlFrames.isEmpty()) {
+				QMutexLocker lock(&qmJitter);
 
-			spx_int32_t startofs = 0;
+				char data[4096];
+				JitterBufferPacket jbp;
+				jbp.data = data;
+				jbp.len = 4096;
 
-			if (jitter_buffer_get(jbJitter, &jbp, iFrameSize, &startofs) == JITTER_BUFFER_OK) {
-				ucFlags = jbp.data[0];
-				fPos[0] = fPos[1] = fPos[2] = 0.0;
-				speex_bits_read_from(&sbBits, jbp.data + 1, jbp.len - 1);
-				speex_decode(dsDecState, &sbBits, pOut);
-				bLastAlive = true;
-			} else {
-				if (ucFlags & MessageSpeex::EndSpeech) {
-					memset(pOut, 0, sizeof(float) * iFrameSize);
-					bLastAlive = false;
+				spx_int32_t startofs = 0;
+
+				if (jitter_buffer_get(jbJitter, &jbp, iFrameSize, &startofs) == JITTER_BUFFER_OK) {
+					PacketDataStream pds(jbp.data, jbp.len);
+
+					iMissCount = 0;
+					ucFlags = pds.next();
+					bHasTerminator = false;
+
+					unsigned int header = 0;
+					do {
+						header = pds.next();
+						if (header)
+							qlFrames << pds.dataBlock(header & 0x7f);
+						else
+							bHasTerminator = true;
+					} while ((header & 0x80) && pds.isValid());
+
+					if (pds.left()) {
+						pds >> fPos[0];
+						pds >> fPos[1];
+						pds >> fPos[2];
+					} else {
+						fPos[0] = fPos[1] = fPos[2] = 0.0f;
+					}
 				} else {
 					iMissCount++;
-					if (iMissCount < 5) {
-						speex_decode(dsDecState, NULL, pOut);
-					} else {
-						memset(pOut, 0, sizeof(float) * iFrameSize);
-						bLastAlive = false;
-					}
+					if (iMissCount > 10)
+						nextalive = false;
 				}
+
+				jitter_buffer_update_delay(jbJitter, &jbp, NULL);
 			}
 
-			int activity;
-			speex_decoder_ctl(dsDecState, SPEEX_GET_ACTIVITY, &activity);
-			if (activity < 30)
-				jitter_buffer_update_delay(jbJitter, &jbp, NULL);
-
 			jitter_buffer_tick(jbJitter);
+
+			if (! qlFrames.isEmpty()) {
+				const QByteArray &qba = qlFrames.takeFirst();
+				celt_decode_float(cdDecoder, reinterpret_cast<const unsigned char *>(qba.data()), qba.size(), pOut);
+				if (qlFrames.isEmpty() && bHasTerminator)
+					nextalive = false;
+			} else {
+				celt_decode_float(cdDecoder, NULL, 0, pOut);
+			}
+
+			if (! nextalive) {
+				for (unsigned int i=0;i<iFrameSize;++i)
+					pOut[i] *= fFadeOut[i];
+			} else if (ts == 0) {
+				for (unsigned int i=0;i<iFrameSize;++i)
+					pOut[i] *= fFadeIn[i];
+			}
 		}
 		spx_uint32_t inlen = iFrameSize;
 		spx_uint32_t outlen = iOutputSize;
-		if (srs)
+		if (srs && bLastAlive)
 			speex_resampler_process_float(srs, 0, fOut, &inlen, pfBuffer + iBufferFilled, &outlen);
 		iBufferFilled += outlen;
 	}
 
 	if (p)
-		p->setTalking(bLastAlive, ((ucFlags & MessageSpeex::AltSpeak) ? true : false));
-	return bLastAlive;
+		p->setTalking(nextalive, ((ucFlags != 0) ? true : false));
+
+	bool tmp = bLastAlive;
+	bLastAlive = nextalive;
+	return tmp;
 }
 
 
 AudioOutput::AudioOutput() {
-	void *ds=speex_decoder_init(&speex_wb_mode);
-	speex_decoder_ctl(ds, SPEEX_GET_FRAME_SIZE, &iFrameSize);
-	speex_decoder_destroy(ds);
+	iFrameSize = SAMPLE_RATE / 100;
 	bRunning = false;
 
 	iChannels = 0;
@@ -501,7 +586,7 @@ float AudioOutput::calcGain(float dotproduct, float distance) {
 }
 
 void AudioOutput::wipe() {
-	foreach(AudioOutputPlayer *aop, qmOutputs)
+	foreach(AudioOutputUser *aop, qmOutputs)
 		removeBuffer(aop);
 }
 
@@ -513,44 +598,37 @@ const float *AudioOutput::getSpeakerPos(unsigned int &speakers) {
 	return NULL;
 }
 
-AudioSine *AudioOutput::playSine(float hz, float i, unsigned int frames, float volume) {
-	while ((iMixerFreq == 0) && isRunning()) {}
-
-	qrwlOutputs.lockForWrite();
-	AudioSine *as = new AudioSine(hz,i,frames, volume, iMixerFreq);
-	qmOutputs.insert(NULL, as);
-	qrwlOutputs.unlock();
-	return as;
-}
-
 AudioOutputSample *AudioOutput::playSample(const QString &filename, bool loop) {
-	QList<QByteArray> packets = AudioOutputSample::getPacketsFromFile(filename);
-	if (packets.isEmpty())
+	SoundFile *handle;
+	handle = AudioOutputSample::loadSndfile(filename);
+	if (handle == NULL)
 		return NULL;
 
 	while ((iMixerFreq == 0) && isRunning()) {}
 
 	qrwlOutputs.lockForWrite();
-	AudioOutputSample *aos = new AudioOutputSample(filename, packets, loop, iMixerFreq);
+	AudioOutputSample *aos = new AudioOutputSample(filename, handle, loop, iMixerFreq);
 	qmOutputs.insert(NULL, aos);
 	qrwlOutputs.unlock();
+
 	return aos;
 
 }
 
-void AudioOutput::addFrameToBuffer(ClientPlayer *player, const QByteArray &qbaPacket, unsigned int iSeq) {
+void AudioOutput::addFrameToBuffer(ClientUser *user, const QByteArray &qbaPacket, unsigned int iSeq) {
 	if (iChannels == 0)
 		return;
 	qrwlOutputs.lockForRead();
-	AudioOutputSpeech *aop = dynamic_cast<AudioOutputSpeech *>(qmOutputs.value(player));
+	AudioOutputSpeech *aop = dynamic_cast<AudioOutputSpeech *>(qmOutputs.value(user));
+
 	if (! aop) {
 		qrwlOutputs.unlock();
 
 		while ((iMixerFreq == 0) && isRunning()) {}
 
 		qrwlOutputs.lockForWrite();
-		aop = new AudioOutputSpeech(player, iMixerFreq);
-		qmOutputs.replace(player,aop);
+		aop = new AudioOutputSpeech(user, iMixerFreq);
+		qmOutputs.replace(user,aop);
 	}
 
 	aop->addFrameToBuffer(qbaPacket, iSeq);
@@ -558,13 +636,13 @@ void AudioOutput::addFrameToBuffer(ClientPlayer *player, const QByteArray &qbaPa
 	qrwlOutputs.unlock();
 }
 
-void AudioOutput::removeBuffer(const ClientPlayer *player) {
-	removeBuffer(qmOutputs.value(player));
+void AudioOutput::removeBuffer(const ClientUser *user) {
+	removeBuffer(qmOutputs.value(user));
 }
 
-void AudioOutput::removeBuffer(AudioOutputPlayer *aop) {
+void AudioOutput::removeBuffer(AudioOutputUser *aop) {
 	QWriteLocker locker(&qrwlOutputs);
-	QMultiHash<const ClientPlayer *, AudioOutputPlayer *>::iterator i;
+	QMultiHash<const ClientUser *, AudioOutputUser *>::iterator i;
 	for (i=qmOutputs.begin(); i != qmOutputs.end(); ++i) {
 		if (i.value() == aop) {
 			qmOutputs.erase(i);
@@ -690,20 +768,19 @@ void AudioOutput::initializeMixer(const unsigned int *chanmasks, bool forceheadp
 			}
 		}
 	}
-	iSampleSize = iChannels * ((eSampleFormat == SampleFloat) ? sizeof(float) : sizeof(short));
+	iSampleSize = static_cast<int>(iChannels * ((eSampleFormat == SampleFloat) ? sizeof(float) : sizeof(short)));
 	qWarning("AudioOutput: Initialized %d channel %d hz mixer", iChannels, iMixerFreq);
 }
 
 bool AudioOutput::mix(void *outbuff, unsigned int nsamp) {
-	AudioOutputPlayer *aop;
-	QList<AudioOutputPlayer *> qlMix;
-	QList<AudioOutputPlayer *> qlDel;
-
+	AudioOutputUser *aop;
+	QList<AudioOutputUser *> qlMix;
+	QList<AudioOutputUser *> qlDel;
 
 	if (g.s.fVolume < 0.01)
 		return false;
 
-	const float mul = g.s.fVolume / 32768.0f;
+	const float mul = g.s.fVolume;
 	const unsigned int nchan = iChannels;
 
 	qrwlOutputs.lockForRead();
@@ -772,6 +849,7 @@ bool AudioOutput::mix(void *outbuff, unsigned int nsamp) {
 			}
 			validListener = true;
 		}
+
 		foreach(aop, qlMix) {
 			const float * RESTRICT pfBuffer = aop->pfBuffer;
 
@@ -815,6 +893,7 @@ bool AudioOutput::mix(void *outbuff, unsigned int nsamp) {
 				}
 			}
 		}
+
 		// Clip
 		if (eSampleFormat == SampleFloat)
 			for (unsigned int i=0;i<nsamp*iChannels;i++)
@@ -831,63 +910,3 @@ bool AudioOutput::mix(void *outbuff, unsigned int nsamp) {
 
 	return (! qlMix.isEmpty());
 }
-
-AudioSine::AudioSine(float hz, float i, unsigned int frm, float vol, unsigned int freq) : AudioOutputPlayer(QLatin1String("Sine")) {
-	float hfreq = static_cast<float>(freq) / 2.0f;
-	v = 0.0;
-	inc = static_cast<float>(M_PI * hz / hfreq);
-	dinc = static_cast<float>(M_PI * i / (hfreq * hfreq));
-	volume = vol * 32768.f;
-	frames = frm;
-	cntr = 0;
-	tbin = 4;
-
-	if (inc == 0.0)
-		g.iAudioPathTime = 0;
-}
-
-AudioSine::~AudioSine() {
-}
-
-bool AudioSine::needSamples(unsigned int snum) {
-	resizeBuffer(snum);
-	if (frames > 0) {
-		frames--;
-
-		if (inc == 0.0) {
-			if (++cntr == 50) {
-				bSearch = true;
-				cntr = 0;
-				tbin *= 2;
-				if (tbin >= 64)
-					tbin = 4;
-			}
-
-			AudioInputPtr ai = g.ai;
-			if (ai && bSearch && ai->iBestBin == tbin) {
-				bSearch = false;
-				g.iAudioPathTime = cntr;
-			}
-
-			// FIXME: Not * snum, * something else
-			const float m = static_cast<float>(M_PI * static_cast<float>(tbin) / static_cast<float>(snum));
-			for (unsigned int i=0;i<snum;i++)
-				pfBuffer[i] = volume * sinf(m * static_cast<float>(i));
-
-			return true;
-		}
-
-		float t = v;
-
-		for (unsigned int i=0;i<snum;i++) {
-			pfBuffer[i]=sinf(t) * volume;
-			inc+=dinc;
-			t+=inc;
-		}
-		v = t;
-		return true;
-	} else {
-		memset(pfBuffer, 0, sizeof(float) * snum);
-		return false;
-	}
-}
