@@ -32,6 +32,16 @@
 #define _DATABASE_H
 
 #include "mumble_pch.hpp"
+#include "Settings.h"
+
+struct FavoriteServer {
+	QString qsName;
+	QString qsUsername;
+	QString qsPassword;
+	QString qsHostname;
+	QString qsUrl;
+	unsigned short usPort;
+};
 
 class Database : public QObject {
 	private:
@@ -39,9 +49,30 @@ class Database : public QObject {
 		Q_DISABLE_COPY(Database)
 	public:
 		Database();
+		static QList<FavoriteServer> getFavorites();
+		static void setFavorites(const QList<FavoriteServer> &servers);
+		static void setPassword(const QString &host, unsigned short port, const QString &user, const QString &pw);
+		static bool fuzzyMatch(QString &user, QString &pw, QString &host, unsigned short port);
+
+		static bool seenComment(const QString &hash, const QString &comment);
+		static void setSeenComment(const QString &hash, const QString &comment);
+
+		static QStringList getTokens(const QByteArray &digest);
+		static void setTokens(const QByteArray &digest, QStringList &tokens);
+
+		static QList<Shortcut> getShortcuts(const QByteArray &digest);
+		static bool setShortcuts(const QByteArray &digest, QList<Shortcut> &shortcuts);
+
+		static void addFriend(const QString &name, const QString &hash);
+		static void removeFriend(const QString &hash);
+		static const QString getFriend(const QString &hash);
+		static const QMap<QString, QString> getFriends();
+
 		static const QString getDigest(const QString &hostname, unsigned short port);
 		static void setDigest(const QString &hostname, unsigned short port, const QString &digest);
-		static bool fuzzyMatch(QString &user, QString &pw, QString &hostname, unsigned short port);
+
+		static bool getUdp(const QByteArray &digest);
+		static void setUdp(const QByteArray &digest, bool udp);
 };
 
 #else
