@@ -72,9 +72,6 @@ bool G15LCDDeviceUnix::enabled() {
 }
 
 void G15LCDDeviceUnix::setEnabled(bool b) {
-	if (bUnavailable)
-		return;
-
 	bEnabled = b;
 }
 
@@ -106,10 +103,6 @@ QString G15LCDDeviceUnix::name() const {
 	return QString::fromLatin1("Logitech Gamepanel (G15Daemon)");
 }
 
-LCDDevice::Type G15LCDDeviceUnix::type() const {
-	return LCDDevice::GraphicLCD;
-}
-
 QSize G15LCDDeviceUnix::size() const {
 	return QSize(G15_WIDTH, G15_HEIGHT);
 }
