@@ -34,6 +34,7 @@
 #include "Settings.h"
 #include "Timer.h"
 #include "Version.h"
+#include "ACL.h"
 
 // Global helper class to spread variables around across threads.
 
@@ -47,6 +48,7 @@ class Plugins;
 class QSettings;
 class Overlay;
 class LCD;
+class BonjourClient;
 
 struct Global {
 private:
@@ -64,21 +66,24 @@ public:
 	QSettings *qs;
 	Overlay *o;
 	LCD *lcd;
+	BonjourClient *bc;
 	QNetworkAccessManager *nam;
 	int iPushToTalk;
 	Timer tDoublePush;
 	quint64 uiDoublePush;
-	int iAltSpeak;
+	int iTarget;
 	bool bPushToMute;
 	bool bCenterPosition;
-	bool bEchoTest;
 	bool bPosTest;
 	int iAudioPathTime;
 	unsigned int uiSession;
+	ChanACL::Permissions pPermissions;
 	int iMaxBandwidth;
 	int iAudioBandwidth;
 	int iAudioQuality;
+	QDir qdBasePath;
 	Global();
+	~Global();
 };
 
 // Class to handle ordered initialization of globals.
