@@ -30,7 +30,7 @@
 */
 
 #include "LCD.h"
-#include "Player.h"
+#include "ClientUser.h"
 #include "Channel.h"
 #include "Global.h"
 #include "Message.h"
@@ -47,6 +47,10 @@ LCDEngineRegistrar::LCDEngineRegistrar(LCDEngineNew cons) {
 
 LCDEngineRegistrar::~LCDEngineRegistrar() {
 	qlInitializers->removeAll(n);
+	if (qlInitializers->isEmpty()) {
+		delete qlInitializers;
+		qlInitializers = NULL;
+	}
 }
 
 static ConfigWidget *LCDConfigDialogNew(Settings &st) {
@@ -107,23 +111,13 @@ LCDConfig::LCDConfig(Settings &st) : ConfigWidget(st) {
 		qtwi->setText(0, d->name());
 		qtwi->setToolTip(0, d->name());
 
-		LCDDevice::Type type = d->type();
-		QString qsType;
-		if (type == LCDDevice::GraphicLCD)
-			qsType = tr("Graphic");
-		else if (type == LCDDevice::CharacterLCD)
-			qsType = tr("Character");
-
-		qtwi->setText(1, qsType);
-		qtwi->setToolTip(1, qsType);
-
 		QSize lcdsize = d->size();
 		QString qsSize = QString::fromLatin1("%1x%2").arg(lcdsize.width()).arg(lcdsize.height());
-		qtwi->setText(2, qsSize);
-		qtwi->setToolTip(2, qsSize);
+		qtwi->setText(1, qsSize);
+		qtwi->setToolTip(1, qsSize);
 
-		qtwi->setCheckState(3, Qt::Unchecked);
-		qtwi->setToolTip(3, tr("Enable this device"));
+		qtwi->setCheckState(2, Qt::Unchecked);
+		qtwi->setToolTip(2, tr("Enable this device"));
 	}
 }
 
@@ -147,8 +141,8 @@ void LCDConfig::load(const Settings &r) {
 		qtwi->setCheckState(3, enabled ? Qt::Checked : Qt::Unchecked);
 	}
 
-	loadSlider(qsMinColWidth, r.iLCDPlayerViewMinColWidth);
-	loadSlider(qsSplitterWidth, r.iLCDPlayerViewSplitterWidth);
+	loadSlider(qsMinColWidth, r.iLCDUserViewMinColWidth);
+	loadSlider(qsSplitterWidth, r.iLCDUserViewSplitterWidth);
 }
 
 void LCDConfig::save() const {
@@ -159,8 +153,8 @@ void LCDConfig::save() const {
 		s.qmLCDDevices.insert(qsName, qtwi->checkState(3) == Qt::Checked);
 	}
 
-	s.iLCDPlayerViewMinColWidth = qsMinColWidth->value();
-	s.iLCDPlayerViewSplitterWidth = qsSplitterWidth->value();
+	s.iLCDUserViewMinColWidth = qsMinColWidth->value();
+	s.iLCDUserViewSplitterWidth = qsSplitterWidth->value();
 }
 
 void LCDConfig::accept() const {
@@ -168,7 +162,7 @@ void LCDConfig::accept() const {
 		bool enabled = s.qmLCDDevices.value(d->name());
 		d->setEnabled(enabled);
 	}
-	g.lcd->updatePlayerView();
+	g.lcd->updateUserView();
 }
 
 void LCDConfig::on_qsMinColWidth_valueChanged(int v) {
@@ -182,12 +176,23 @@ void LCDConfig::on_qsSplitterWidth_valueChanged(int v) {
 /* --- */
 
 LCD::LCD() : QObject() {
+
+#ifdef Q_OS_MAC
+	qfNormal.setStyleStrategy(QFont::NoAntialias);
+	qfNormal.setKerning(false);
+	qfNormal.setPointSize(10);
+	qfNormal.setFixedPitch(true);
+	qfNormal.setFamily(QString::fromLatin1("Andale Mono"));
+#else
 	qfNormal = QFont(QString::fromLatin1("Arial"), 7);
+#endif
+
 	qfItalic = qfNormal;
 	qfItalic.setItalic(true);
 
 	qfBold = qfNormal;
 	qfBold.setWeight(QFont::Black);
+
 	qfItalicBold = qfBold;
 	qfItalic.setItalic(true);
 
@@ -206,13 +211,14 @@ LCD::LCD() : QObject() {
 		bool enabled = g.s.qmLCDDevices.contains(d->name()) ? g.s.qmLCDDevices.value(d->name()) : true;
 		d->setEnabled(enabled);
 	}
-	qiLogo = QImage(QLatin1String("skin:mumble.48x48.png")).convertToFormat(QImage::Format_MonoLSB);
-	updatePlayerView();
+	qiLogo = QIcon(QLatin1String("skin:mumble.svg")).pixmap(48,48).toImage().convertToFormat(QImage::Format_MonoLSB);
+	qiLogo.invertPixels();
+	updateUserView();
 }
 
 void LCD::tick() {
 	iFrameIndex ++;
-	updatePlayerView();
+	updateUserView();
 }
 
 void LCD::initBuffers() {
@@ -232,7 +238,7 @@ void LCD::destroyBuffers() {
 	qhImages.clear();
 
 	foreach(unsigned char *buf, qhImageBuffers)
-		delete buf;
+		delete [] buf;
 	qhImageBuffers.clear();
 }
 
@@ -247,19 +253,19 @@ static bool entriesSort(const ListEntry &a, const ListEntry &b) {
 	return a.qsString < b.qsString;
 }
 
-void LCD::updatePlayerView() {
+void LCD::updateUserView() {
 	if (qhImages.count() == 0)
 		return;
 
 	QStringList qslTalking;
-	Player *me = g.uiSession ? ClientPlayer::get(g.uiSession) : NULL;
+	User *me = g.uiSession ? ClientUser::get(g.uiSession) : NULL;
 	Channel *home = me ? me->cChannel : NULL;
 	bool alert = false;
 
 	foreach(const QSize &size, qhImages.keys()) {
 		QImage *img = qhImages.value(size);
 		QPainter painter(img);
-		painter.setRenderHints(0, true);
+		painter.setRenderHints(QPainter::Antialiasing | QPainter::TextAntialiasing, false);
 		painter.setPen(Qt::color1);
 		painter.setFont(qfNormal);
 
@@ -270,12 +276,12 @@ void LCD::updatePlayerView() {
 			qmOld.clear();
 			qmSpeaking.clear();
 			qmNameCache.clear();
-			painter.drawImage(0,0,qiLogo);
-			painter.drawText(60,20, tr("Not connected"));
+			painter.drawImage(0, 0, qiLogo);
+			painter.drawText(60, 20, tr("Not connected"));
 			continue;
 		}
 
-		foreach(Player *p, me->cChannel->qlPlayers) {
+		foreach(User *p, me->cChannel->qlUsers) {
 			if (! qmNew.contains(p->uiSession)) {
 				qmNew.insert(p->uiSession, Timer());
 				qmNameCache.insert(p->uiSession, p->qsName);
@@ -284,7 +290,7 @@ void LCD::updatePlayerView() {
 		}
 
 		foreach(unsigned int session, qmNew.keys()) {
-			Player *p = ClientPlayer::get(session);
+			User *p = ClientUser::get(session);
 			if (!p || (p->cChannel != me->cChannel)) {
 				qmNew.remove(session);
 				qmOld.insert(session, Timer());
@@ -311,8 +317,9 @@ void LCD::updatePlayerView() {
 		QMap<unsigned int, Timer> speaking;
 
 		foreach(Channel *c, home->allLinks()) {
-			foreach(Player *p, c->qlPlayers) {
-				bool bTalk = p->bTalking;
+			foreach(User *p, c->qlUsers) {
+				ClientUser *u = static_cast<ClientUser *>(p);
+				bool bTalk = (u->tsState != ClientUser::TalkingOff);
 				if (bTalk) {
 					speaking.insert(p->uiSession, Timer());
 				} else if (qmSpeaking.contains(p->uiSession)) {
@@ -350,16 +357,16 @@ void LCD::updatePlayerView() {
 
 		const int iWidth = size.width();
 		const int iHeight = size.height();
-		const int iPlayersPerColumn = iHeight / iFontHeight;
-		const int iSplitterWidth = g.s.iLCDPlayerViewSplitterWidth;
-		const int iPlayerColumns = (entries.count() + iPlayersPerColumn - 1) / iPlayersPerColumn;
+		const int iUsersPerColumn = iHeight / iFontHeight;
+		const int iSplitterWidth = g.s.iLCDUserViewSplitterWidth;
+		const int iUserColumns = (entries.count() + iUsersPerColumn - 1) / iUsersPerColumn;
 
-		int iColumns = iPlayerColumns;
+		int iColumns = iUserColumns;
 		int iColumnWidth = 1;
 
 		while (iColumns >= 1) {
 			iColumnWidth = (iWidth - (iColumns-1)*iSplitterWidth) / iColumns;
-			if (iColumnWidth >= g.s.iLCDPlayerViewMinColWidth)
+			if (iColumnWidth >= g.s.iLCDUserViewMinColWidth)
 				break;
 			--iColumns;
 		}
@@ -369,7 +376,7 @@ void LCD::updatePlayerView() {
 
 
 		foreach(const ListEntry &le, entries) {
-			if (row >= iPlayersPerColumn) {
+			if (row >= iUsersPerColumn) {
 				row = 0;
 				++col;
 			}
@@ -393,16 +400,10 @@ void LCD::updatePlayerView() {
 	}
 
 	foreach(LCDDevice *d, devmgr.qlDevices) {
-		LCDDevice::Type type = d->type();
-		if (type == LCDDevice::GraphicLCD) {
-			QImage *img = qhImages[d->size()];
-			if (! img)
-				continue;
-			d->blitImage(img, alert);
-		} else if (type == LCDDevice::CharacterLCD) {
-			/* TODO. */
-			qWarning("LCD: Unable to draw to Character LCD.");
-		}
+		QImage *img = qhImages[d->size()];
+		if (! img)
+			continue;
+		d->blitImage(img, alert);
 	}
 }
 
@@ -420,6 +421,8 @@ LCDEngine::LCDEngine() : QObject() {
 }
 
 LCDEngine::~LCDEngine() {
+	foreach(LCDDevice *lcd, qlDevices)
+		delete lcd;
 }
 
 LCDDevice::LCDDevice() {
