@@ -36,7 +36,7 @@
 #include "Timer.h"
 #include "ui_LCD.h"
 
-class Player;
+class User;
 class LCDDevice;
 
 class LCDConfig : public ConfigWidget, public Ui::LCDConfig {
@@ -70,14 +70,12 @@ class LCDEngine : public QObject {
 
 class LCDDevice {
 	public:
-		enum Type { GraphicLCD, CharacterLCD };
 		LCDDevice();
 		virtual ~LCDDevice();
 		virtual bool enabled() = 0;
 		virtual void setEnabled(bool e) = 0;
 		virtual void blitImage(QImage *img, bool alert = false) = 0;
 		virtual QString name() const = 0;
-		virtual Type type() const = 0;
 		virtual QSize size() const = 0;
 };
 
@@ -116,7 +114,7 @@ class LCD : public QObject {
 	public:
 		LCD();
 		~LCD();
-		void updatePlayerView();
+		void updateUserView();
 		bool hasDevices();
 };
 
