@@ -1,4 +1,5 @@
 /* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+   Copyright (C) 2009, Stefan Hacker <dd0t@users.sourceforge.net>
 
    All rights reserved.
 
@@ -75,13 +76,13 @@ LogConfig::LogConfig(Settings &st) : ConfigWidget(st) {
 		twi->setToolTip(ColNotification, tr("Toggle pop-up notifications for %1 events").arg(messageName));
 		twi->setToolTip(ColTTS, tr("Toggle Text-To-Speech for %1 events").arg(messageName));
 		twi->setToolTip(ColStaticSound, tr("Click here to toggle sound notification for %1 events").arg(messageName));
-		twi->setToolTip(ColStaticSoundPath, tr("Path to soundfile used for sound notifications in the case of %1 events<br />Single click to play<br />Doubleclick to change").arg(messageName));
+		twi->setToolTip(ColStaticSoundPath, tr("Path to sound file used for sound notifications in the case of %1 events<br />Single click to play<br />Double-click to change").arg(messageName));
 
 		twi->setWhatsThis(ColConsole, tr("Click here to toggle console output for %1 events.<br />If checked, this option makes Mumble output all %1 events in its message log.").arg(messageName));
 		twi->setWhatsThis(ColNotification, tr("Click here to toggle pop-up notifications for %1 events.<br />If checked, a notification pop-up will be created by Mumble for every %1 event.").arg(messageName));
-		twi->setWhatsThis(ColTTS, tr("Click here to toggle Text-To-Speech for %1 events.<br />If checked, Mumble uses Text-To-Speech to read %1 events out loud to you. Text-To-Speech is also able to read the contents of the event which is not true for soundfiles. Text-To-Speech and soundfiles cannot be used at the same time.").arg(messageName));
-		twi->setWhatsThis(ColStaticSound, tr("Click here to toggle sound notification for %1 events.<br />If checked, Mumble uses a soundfile predefined by you to indicate %1 events. Soundfiles and Text-To-Speech cannot be used at the same time.").arg(messageName));
-		twi->setWhatsThis(ColStaticSoundPath, tr("Path to soundfile used for sound notifications in the case of %1 events.<br />Single click to play<br />Doubleclick to change<br />Be sure that sound notifications for these events are enabled or this field will not have any effect.").arg(messageName));
+		twi->setWhatsThis(ColTTS, tr("Click here to toggle Text-To-Speech for %1 events.<br />If checked, Mumble uses Text-To-Speech to read %1 events out loud to you. Text-To-Speech is also able to read the contents of the event which is not true for sound files. Text-To-Speech and sound files cannot be used at the same time.").arg(messageName));
+		twi->setWhatsThis(ColStaticSound, tr("Click here to toggle sound notification for %1 events.<br />If checked, Mumble uses a sound file predefined by you to indicate %1 events. Sound files and Text-To-Speech cannot be used at the same time.").arg(messageName));
+		twi->setWhatsThis(ColStaticSoundPath, tr("Path to sound file used for sound notifications in the case of %1 events.<br />Single click to play<br />Double-click to change<br />Ensure that sound notifications for these events are enabled or this field will not have any effect.").arg(messageName));
 	}
 }
 
@@ -108,6 +109,7 @@ void LogConfig::load(const Settings &r) {
 
 	loadSlider(qsVolume, r.iTTSVolume);
 	qsbThreshold->setValue(r.iTTSThreshold);
+	qcbWhisperFriends->setChecked(r.bWhisperFriends);
 }
 
 void LogConfig::save() const {
@@ -130,6 +132,7 @@ void LogConfig::save() const {
 
 	s.iTTSVolume=qsVolume->value();
 	s.iTTSThreshold=qsbThreshold->value();
+	s.bWhisperFriends = qcbWhisperFriends->isChecked();
 }
 
 void LogConfig::accept() const {
@@ -150,7 +153,7 @@ void LogConfig::on_qtwMessages_itemChanged(QTreeWidgetItem* i, int column) {
 		case ColStaticSound:
 			if (i->checkState(ColStaticSound)) {
 				i->setCheckState(ColTTS, Qt::Unchecked);
-				if (i->text(ColStaticSoundPath).isEmpty()) browseForSpxFile();
+				if (i->text(ColStaticSoundPath).isEmpty()) browseForAudioFile();
 			}
 			break;
 		default:
@@ -163,37 +166,30 @@ void LogConfig::on_qtwMessages_itemClicked(QTreeWidgetItem * item, int column) {
 		AudioOutputPtr ao = g.ao;
 		if (ao) {
 			if (!ao->playSample(item->text(ColStaticSoundPath), false))
-				browseForSpxFile();
+				browseForAudioFile();
 		}
 	}
 }
 
 void LogConfig::on_qtwMessages_itemDoubleClicked(QTreeWidgetItem * item, int column) {
 	if (item && column == ColStaticSoundPath)
-		browseForSpxFile();
+		browseForAudioFile();
 }
 
-void LogConfig::browseForSpxFile() {
-	QString file = QFileDialog::getOpenFileName(this, tr("Choose sound file"), QString(), QLatin1String("*.spx"));
-	if (! file.isEmpty()) {
-		if (AudioOutputSample::getPacketsFromFile(file).isEmpty()) {
-			QMessageBox::critical(this,
-			                      tr("Invalid sound file"),
-			                      tr("The file '%1' does not exist or is not a valid speex file.").arg(file));
-			return;
-		}
+void LogConfig::browseForAudioFile() {
+	QString file = AudioOutputSample::browseForSndfile();
+	if (!file.isEmpty()) {
 		QTreeWidgetItem *i = qtwMessages->selectedItems()[0];
 		i->setText(ColStaticSoundPath, file);
 		i->setCheckState(ColStaticSound, Qt::Checked);
-
 	}
 }
 
-
 Log::Log(QObject *p) : QObject(p) {
 	tts=new TextToSpeech(this);
 	tts->setVolume(g.s.iTTSVolume);
 	uiLastId = 0;
+	qdDate = QDate::currentDate();
 
 #ifdef Q_OS_MAC
 	QStringList qslAllEvents;
@@ -217,25 +213,25 @@ Log::Log(QObject *p) : QObject(p) {
 }
 
 const char *Log::msgNames[] = {
-	QT_TR_NOOP("Debug"),
-	QT_TR_NOOP("Critical"),
-	QT_TR_NOOP("Warning"),
-	QT_TR_NOOP("Information"),
-	QT_TR_NOOP("Server Connected"),
-	QT_TR_NOOP("Server Disconnected"),
-	QT_TR_NOOP("Player Joined Server"),
-	QT_TR_NOOP("Player Left Server"),
-	QT_TR_NOOP("Player kicked (you or by you)"),
-	QT_TR_NOOP("Player kicked"),
-	QT_TR_NOOP("You self-muted/deafened"),
-	QT_TR_NOOP("Other self-muted/deafened"),
-	QT_TR_NOOP("Player muted (you)"),
-	QT_TR_NOOP("Player muted (by you)"),
-	QT_TR_NOOP("Player muted (other)"),
-	QT_TR_NOOP("Player Joined Channel"),
-	QT_TR_NOOP("Player Left Channel"),
-	QT_TR_NOOP("Permission Denied"),
-	QT_TR_NOOP("Text Message")
+	QT_TRANSLATE_NOOP("Log", "Debug"),
+	QT_TRANSLATE_NOOP("Log", "Critical"),
+	QT_TRANSLATE_NOOP("Log", "Warning"),
+	QT_TRANSLATE_NOOP("Log", "Information"),
+	QT_TRANSLATE_NOOP("Log", "Server Connected"),
+	QT_TRANSLATE_NOOP("Log", "Server Disconnected"),
+	QT_TRANSLATE_NOOP("Log", "User Joined Server"),
+	QT_TRANSLATE_NOOP("Log", "User Left Server"),
+	QT_TRANSLATE_NOOP("Log", "User kicked (you or by you)"),
+	QT_TRANSLATE_NOOP("Log", "User kicked"),
+	QT_TRANSLATE_NOOP("Log", "You self-muted/deafened"),
+	QT_TRANSLATE_NOOP("Log", "Other self-muted/deafened"),
+	QT_TRANSLATE_NOOP("Log", "User muted (you)"),
+	QT_TRANSLATE_NOOP("Log", "User muted (by you)"),
+	QT_TRANSLATE_NOOP("Log", "User muted (other)"),
+	QT_TRANSLATE_NOOP("Log", "User Joined Channel"),
+	QT_TRANSLATE_NOOP("Log", "User Left Channel"),
+	QT_TRANSLATE_NOOP("Log", "Permission Denied"),
+	QT_TRANSLATE_NOOP("Log", "Text Message")
 };
 
 QString Log::msgName(MsgType t) const {
@@ -250,8 +246,118 @@ void Log::clearIgnore() {
 	qmIgnore.clear();
 }
 
+QString Log::imageToImg(const QByteArray &format, const QByteArray &image) {
+	QString fmt = QLatin1String(format);
+
+	if (fmt.isEmpty())
+		fmt = QLatin1String("qt");
+
+	QByteArray rawbase = image.toBase64();
+	QByteArray encoded;
+	int i = 0;
+	int begin = 0, end = 0;
+	do {
+		begin = i*72;
+		end = begin+72;
+
+		encoded.append(QUrl::toPercentEncoding(QLatin1String(rawbase.mid(begin,72))));
+		if (end < rawbase.length())
+			encoded.append('\n');
+
+		++i;
+	} while (end < rawbase.length());
+
+	return QString::fromLatin1("<img src=\"data:image/%1;base64,%2\" />").arg(fmt).arg(QLatin1String(encoded));
+}
+
+QString Log::imageToImg(QImage img) {
+	if ((img.width() > 480) || (img.height() > 270)) {
+		img = img.scaled(480, 270, Qt::KeepAspectRatio, Qt::SmoothTransformation);
+	}
+
+	int quality = 100;
+	QByteArray format = "PNG";
+
+	QByteArray qba;
+	{
+		QBuffer qb(&qba);
+		qb.open(QIODevice::WriteOnly);
+
+		QImageWriter imgwrite(&qb, format);
+		imgwrite.write(img);
+	}
+
+	while ((qba.length() >= 65536) && (quality > 0)) {
+		qba.clear();
+		QBuffer qb(&qba);
+		qb.open(QIODevice::WriteOnly);
+
+		format = "JPEG";
+
+		QImageWriter imgwrite(&qb, format);
+		imgwrite.setQuality(quality);
+		imgwrite.write(img);
+		quality -= 10;
+	}
+	if (qba.length() < 65536) {
+		return imageToImg(format, qba);
+	}
+	return QString();
+}
+
+QString Log::validHtml(const QString &html, bool allowReplacement) {
+	QDesktopWidget dw;
+	ValidDocument qtd(allowReplacement);
+	bool valid = false;
+
+	QStringList qslValid;
+	qslValid << QLatin1String("mumble");
+	qslValid << QLatin1String("http");
+	qslValid << QLatin1String("https");
+	qslValid << QLatin1String("ftp");
+
+	QRectF qr = dw.availableGeometry(dw.screenNumber(g.mw));
+	qtd.setTextWidth(qr.width() / 2);
+
+	qtd.setHtml(html);
+	valid = qtd.isValid();
+
+	for (QTextBlock qtb = qtd.begin(); qtb != qtd.end(); qtb = qtb.next()) {
+		for (QTextBlock::iterator qtbi = qtb.begin(); qtbi != qtb.end(); ++qtbi) {
+			const QTextFragment &qtf = qtbi.fragment();
+			QTextCharFormat qcf = qtf.charFormat();
+			if (! qcf.anchorHref().isEmpty()) {
+				QUrl url(qcf.anchorHref());
+				if (! url.isValid() || ! qslValid.contains(url.scheme()))
+					valid = false;
+			}
+			if (qcf.isImageFormat()) {
+				QTextImageFormat qtif = qcf.toImageFormat();
+				QUrl url(qtif.name());
+				if (! url.isValid())
+					valid = false;
+			}
+		}
+	}
+
+	qtd.adjustSize();
+	QSizeF s = qtd.size();
+
+	if (!valid || (s.width() > qr.width()) || (s.height() > qr.height())) {
+		qtd.setPlainText(html);
+		qtd.adjustSize();
+		s = qtd.size();
+
+		if ((s.width() > qr.width()) || (s.height() > qr.height()))
+			return tr("[[ Text object too large to display ]]");
+		return qtd.toHtml();
+	}
+
+	return Qt::mightBeRichText(html) ? html : (QLatin1String("<span></span>")+html);
+}
+
 void Log::log(MsgType mt, const QString &console, const QString &terse) {
-	QTime now = QTime::currentTime();
+	QDateTime dt = QDateTime::currentDateTime();
 
 	int ignore = qmIgnore.value(mt);
 	if (ignore) {
@@ -268,6 +374,14 @@ void Log::log(MsgType mt, const QString &console, const QString &terse) {
 	if ((flags & Settings::LogConsole)) {
 		QTextCursor tc=g.mw->qteLog->textCursor();
 		tc.movePosition(QTextCursor::End);
+
+		if (qdDate != dt.date()) {
+			qdDate = dt.date();
+			tc.insertBlock();
+			tc.insertHtml(tr("[Date changed to %1]\n").arg(qdDate.toString(Qt::DefaultLocaleShortDate)));
+			tc.movePosition(QTextCursor::End);
+		}
+
 		if (plain.contains(QRegExp(QLatin1String("[\\r\\n]")))) {
 			QTextFrameFormat qttf;
 			qttf.setBorder(1);
@@ -277,7 +391,7 @@ void Log::log(MsgType mt, const QString &console, const QString &terse) {
 		} else if (! g.mw->qteLog->document()->isEmpty()) {
 			tc.insertBlock();
 		}
-		tc.insertHtml(QString::fromLatin1("[%2] %1\n").arg(console).arg(now.toString(Qt::LocalDate)));
+		tc.insertHtml(QString::fromLatin1("[%2] %1\n").arg(validHtml(console, true), dt.time().toString(Qt::DefaultLocaleShortDate)));
 		tc.movePosition(QTextCursor::End);
 		g.mw->qteLog->setTextCursor(tc);
 		g.mw->qteLog->ensureCursorVisible();
@@ -302,6 +416,7 @@ void Log::log(MsgType mt, const QString &console, const QString &terse) {
 				break;
 		}
 
+#ifdef USE_DBUS
 		QDBusMessage response;
 		QVariantMap hints;
 		hints.insert(QLatin1String("desktop-entry"), QLatin1String("mumble"));
@@ -334,6 +449,9 @@ void Log::log(MsgType mt, const QString &console, const QString &terse) {
 		if (response.type()==QDBusMessage::ReplyMessage && response.arguments().count() == 1) {
 			uiLastId = response.arguments().at(0).toUInt();
 		} else {
+#else
+		if (true) {
+#endif
 			if (g.mw->qstiIcon->isSystemTrayAvailable() && g.mw->qstiIcon->supportsMessages()) {
 				QSystemTrayIcon::MessageIcon msgIcon;
 
@@ -363,8 +481,11 @@ void Log::log(MsgType mt, const QString &console, const QString &terse) {
 #endif
 	}
 
-	// Message notification with static sounds
+	// Don't make any noise if we are self deafened
+	if (g.s.bDeaf)
+		return;
 
+	// Message notification with static sounds
 	if ((flags & Settings::LogSoundfile)) {
 		QString sSound = g.s.qmMessageSounds.value(mt);
 		AudioOutputPtr ao = g.ao;
@@ -385,15 +506,45 @@ void Log::log(MsgType mt, const QString &console, const QString &terse) {
 		tts->say(terse);
 }
 
+ValidDocument::ValidDocument(bool allowhttp, QObject *p) : QTextDocument(p) {
+	bValid = true;
+	qslValidImage << QLatin1String("data");
+	if (allowhttp) {
+		qslValidImage << QLatin1String("http");
+		qslValidImage << QLatin1String("https");
+	}
+}
+
+QVariant ValidDocument::loadResource(int type, const QUrl &url) {
+	QVariant v = QLatin1String("PlaceHolder");
+	if ((type == QTextDocument::ImageResource) && qslValidImage.contains(url.scheme()))
+		return QTextDocument::loadResource(type, url);
+	bValid = false;
+	return v;
+}
+
+bool ValidDocument::isValid() const {
+	return bValid;
+}
+
 LogDocument::LogDocument(QObject *p) : QTextDocument(p) {
 }
 
 QVariant LogDocument::loadResource(int type, const QUrl &url) {
-	if ((type != QTextDocument::ImageResource))
+	if (type != QTextDocument::ImageResource)
 		return QLatin1String("No external resources allowed.");
+	if (g.s.iMaxImageSize <= 0)
+		return QLatin1String("Image download disabled.");
 
-	QImage qi(0, 0, QImage::Format_Mono);
+	if (url.scheme() == QLatin1String("data")) {
+		QVariant v = QTextDocument::loadResource(type, url);
+		addResource(type, url, v);
+		return v;
+	}
 
+	qWarning() << "LogDocument::loadResource " << type << url.toString();
+
+	QImage qi(1, 1, QImage::Format_Mono);
 	addResource(type, url, qi);
 
 	if (! url.isValid() || url.isRelative())
@@ -402,21 +553,34 @@ QVariant LogDocument::loadResource(int type, const QUrl &url) {
 	if ((url.scheme() != QLatin1String("http")) && (url.scheme() != QLatin1String("https")))
 		return qi;
 
-	qWarning() << "LogDocument::loadResource " << url.toString();
-
 	QNetworkRequest req(url);
 	QNetworkReply *rep = g.nam->get(req);
+	connect(rep, SIGNAL(metaDataChanged()), this, SLOT(receivedHead()));
 	connect(rep, SIGNAL(finished()), this, SLOT(finished()));
 	return qi;
 }
 
+void LogDocument::receivedHead() {
+	QNetworkReply *rep = qobject_cast<QNetworkReply *>(sender());
+	QVariant length = rep->header(QNetworkRequest::ContentLengthHeader);
+	if (length == QVariant::Invalid || length.toInt() > g.s.iMaxImageSize) {
+		qWarning() << "Image "<< rep->url().toString() <<" (" << length.toInt() << " byte) to big, request aborted. ";
+		rep->abort();
+	}
+}
+
 void LogDocument::finished() {
 	QNetworkReply *rep = qobject_cast<QNetworkReply *>(sender());
 
 	if (rep->error() == QNetworkReply::NoError) {
 		QVariant qv = rep->readAll();
-		addResource(QTextDocument::ImageResource, rep->request().url(), qv);
-		g.mw->qteLog->setDocument(this);
-	}
+		QImage qi;
+
+		if (qi.loadFromData(qv.toByteArray()) && qi.width() <= g.s.iMaxImageWidth && qi.height() <= g.s.iMaxImageHeight) {
+			addResource(QTextDocument::ImageResource, rep->request().url(), qi);
+			g.mw->qteLog->setDocument(this);
+		} else qWarning() << "Image "<< rep->url().toString() <<" (" << qi.width() << "x" << qi.height() <<") to large.";
+	} else qWarning() << "Image "<< rep->url().toString() << " download failed.";
+
 	rep->deleteLater();
 }
