@@ -56,7 +56,7 @@ class LogConfig : public ConfigWidget, public Ui::LogConfig {
 		void on_qtwMessages_itemChanged(QTreeWidgetItem*, int);
 		void on_qtwMessages_itemClicked(QTreeWidgetItem*, int);
 		void on_qtwMessages_itemDoubleClicked(QTreeWidgetItem*, int);
-		void browseForSpxFile();
+		void browseForAudioFile();
 };
 
 class Log : public QObject {
@@ -65,7 +65,7 @@ class Log : public QObject {
 		Q_OBJECT
 		Q_DISABLE_COPY(Log)
 	public:
-		enum MsgType { DebugInfo, CriticalError, Warning, Information, ServerConnected, ServerDisconnected, PlayerJoin, PlayerLeave, YouKicked, PlayerKicked, SelfMute, OtherSelfMute, YouMuted, YouMutedOther, OtherMutedOther, ChannelJoin, ChannelLeave, PermissionDenied, TextMessage };
+		enum MsgType { DebugInfo, CriticalError, Warning, Information, ServerConnected, ServerDisconnected, UserJoin, UserLeave, YouKicked, UserKicked, SelfMute, OtherSelfMute, YouMuted, YouMutedOther, OtherMutedOther, ChannelJoin, ChannelLeave, PermissionDenied, TextMessage };
 		static const MsgType firstMsgType = DebugInfo;
 		static const MsgType lastMsgType = TextMessage;
 	protected:
@@ -73,17 +73,21 @@ class Log : public QObject {
 		static const char *msgNames[];
 		TextToSpeech *tts;
 		unsigned int uiLastId;
+		QDate qdDate;
 	public:
 		Log(QObject *p = NULL);
 		QString msgName(MsgType t) const;
 		void setIgnore(MsgType t, int ignore = 1 << 30);
 		void clearIgnore();
+		static QString validHtml(const QString &html, bool allowReplacement = false);
 	public slots:
 		void log(MsgType t, const QString &console, const QString &terse=QString());
 };
 
 class LogDocument : public QTextDocument {
+	private:
 		Q_OBJECT
+		Q_DISABLE_COPY(LogDocument)
 	public:
 		QVariant qvVariant;
 		int iType;
@@ -91,6 +95,7 @@ class LogDocument : public QTextDocument {
 		LogDocument(QObject *p);
 		QVariant loadResource(int, const QUrl &);
 	public slots:
+		void receivedHead();
 		void finished();
 };
 
