@@ -33,9 +33,10 @@
 
 #include "../../overlay/overlay.h"
 #include "ConfigDialog.h"
+#include "ClientUser.h"
 #include "ui_Overlay.h"
 
-class Player;
+class User;
 
 class SharedMemoryPrivate;
 class SharedMemory {
@@ -58,7 +59,7 @@ class OverlayConfig : public ConfigWidget, public Ui::OverlayConfig {
 		Q_DISABLE_COPY(OverlayConfig)
 	protected:
 		QFont qfFont;
-		QColor qcPlayer, qcAltTalking, qcTalking, qcChannel, qcChannelTalking;
+		QColor qcUser, qcWhisper, qcTalking, qcChannel, qcChannelTalking;
 
 		static void setColorLabel(QLabel *label, QColor col);
 	public:
@@ -68,9 +69,9 @@ class OverlayConfig : public ConfigWidget, public Ui::OverlayConfig {
 	public slots:
 		void on_qsMaxHeight_valueChanged(int v);
 		void on_qpbSetFont_clicked();
-		void on_qpbPlayer_clicked();
+		void on_qpbUser_clicked();
 		void on_qpbTalking_clicked();
-		void on_qpbAltTalking_clicked();
+		void on_qpbWhisper_clicked();
 		void on_qpbChannel_clicked();
 		void on_qpbChannelTalking_clicked();
 		void accept() const;
@@ -79,8 +80,14 @@ class OverlayConfig : public ConfigWidget, public Ui::OverlayConfig {
 		bool expert(bool);
 };
 
+class OverlayPrivate : public QObject {
+	private:
+		Q_OBJECT
+		Q_DISABLE_COPY(OverlayPrivate);
+	public:
+		OverlayPrivate() : QObject() {};
+};
 
-class OverlayPrivate;
 class Overlay : public QObject {
 		friend class OverlayConfig;
 	private:
@@ -93,20 +100,19 @@ class Overlay : public QObject {
 
 		struct TextLine {
 			QString qsText;
-			int iPlayer;
+			unsigned int uiSession;
 			quint32 uiColor;
 			Decoration dDecor;
-			TextLine(const QString &t, quint32 c, int p = -1, Decoration d = None) : qsText(t), iPlayer(p), uiColor(c), dDecor(d) { };
+			int iPriority;
+			TextLine(const QString &t, quint32 c, int priority = 0, unsigned int session = 0, Decoration d = None) : qsText(t), uiSession(session), uiColor(c), dDecor(d), iPriority(priority) { };
+			bool operator <(const TextLine &o) const;
 		};
 
-		typedef QPair<unsigned int, QByteArray> UserTexture;
 		QByteArray qbaMuted, qbaDeafened;
 		QList<TextLine> qlCurrentTexts;
 		QHash<QString, unsigned char *> qhTextures;
-		QHash<int, UserTexture> qhUserTextures;
 		QHash<QString, short> qhWidths;
-		QHash<int, QString> qhQueried;
-		QSet<int> qsForce;
+		QSet<unsigned int> qsForce;
 		QLibrary *qlOverlay;
 		QTimer *qtTimer;
 		QFont qfFont;
@@ -120,7 +126,7 @@ class Overlay : public QObject {
 		Overlay();
 		~Overlay();
 		bool isActive() const;
-		void textureResponse(int id, const QByteArray &texture);
+		void verifyTexture(ClientUser *cp);
 	public slots:
 		void on_Timer_timeout();
 		void updateOverlay();
