@@ -37,7 +37,7 @@
 #include <errno.h>
 #include <string.h>
 
-#include "Player.h"
+#include "User.h"
 #include "Channel.h"
 #include "Overlay.h"
 #include "Global.h"
@@ -147,5 +147,5 @@ void Overlay::setActive(bool) {
 
 void Overlay::on_Timer_timeout() {
 	if (sm.sm)
-		sm.sm->lastAppAlive = time(NULL);
+		sm.sm->lastAppAlive = static_cast<int>(time(NULL));
 }
