@@ -66,16 +66,32 @@ class Plugins : public QObject {
 		QList<PluginInfo *> qlPlugins;
 		PluginInfo *locked;
 		PluginInfo *prevlocked;
+		void clearPlugins();
+		int iPluginTry;
+		QMap<QString, QString> qmPluginHash;
+		QString qsSystemPlugins;
+		QString qsUserPlugins;
+#ifdef Q_OS_WIN
+		HANDLE hToken;
+		TOKEN_PRIVILEGES tpPrevious;
+		DWORD cbPrevious;
+#endif
 	public:
+		std::string ssContext, ssContextSent;
+		std::wstring swsIdentity, swsIdentitySent;
 		bool bValid;
 		bool bUnlink;
 		float fPosition[3], fFront[3], fTop[3];
+		float fCameraPosition[3], fCameraFront[3], fCameraTop[3];
 
 		Plugins(QObject *p = NULL);
+		~Plugins();
 	public slots:
 		void on_Timer_timeout();
 		void rescanPlugins();
 		bool fetch();
+		void checkUpdates();
+		void finished();
 };
 
 #else
