@@ -32,16 +32,20 @@
 #include "MainWindow.h"
 #include "AudioInput.h"
 #include "AudioOutput.h"
+#include "Cert.h"
 #include "Message.h"
-#include "Player.h"
+#include "User.h"
 #include "Connection.h"
 #include "Global.h"
 #include "Database.h"
 #include "PacketDataStream.h"
 #include "NetworkConfig.h"
+#include "OSInfo.h"
+#include "SSL.h"
 
-ServerHandlerMessageEvent::ServerHandlerMessageEvent(QByteArray &msg, bool flush) : QEvent(static_cast<QEvent::Type>(SERVERSEND_EVENT)) {
+ServerHandlerMessageEvent::ServerHandlerMessageEvent(const QByteArray &msg, unsigned int mtype, bool flush) : QEvent(static_cast<QEvent::Type>(SERVERSEND_EVENT)) {
 	qbaMsg = msg;
+	uiType = mtype;
 	bFlush = flush;
 }
 
@@ -52,6 +56,8 @@ ServerHandler::ServerHandler() {
 	// For some strange reason, on Win32, we have to call supportsSsl before the cipher list is ready.
 	qWarning("OpenSSL Support: %d", QSslSocket::supportsSsl());
 
+	MumbleSSL::addSystemCA();
+
 	QList<QSslCipher> pref;
 	foreach(QSslCipher c, QSslSocket::defaultCiphers()) {
 		if (c.usedBits() < 128)
@@ -128,88 +134,134 @@ void ServerHandler::udpReady() {
 		if (! cConnection->csCrypt.isValid())
 			continue;
 
+		if (buflen < 5)
+			continue;
+
 		if (! cConnection->csCrypt.decrypt(reinterpret_cast<const unsigned char *>(encrypted), reinterpret_cast<unsigned char *>(buffer), buflen)) {
 			if (cConnection->csCrypt.tLastGood.elapsed() > 5000000ULL) {
 				if (cConnection->csCrypt.tLastRequest.elapsed() > 5000000ULL) {
 					cConnection->csCrypt.tLastRequest.restart();
-					MessageCryptSync mcs;
-					sendMessage(&mcs);
+					MumbleProto::CryptSetup mpcs;
+					sendMessage(mpcs);
 				}
 			}
 			continue;
 		}
 
-		PacketDataStream pds(buffer, buflen-4);
+		PacketDataStream pds(buffer + 1, buflen-5);
 
-		quint32 msgType, uiSession;
-		pds >> msgType >> uiSession;
+		MessageHandler::UDPMessageType msgType = static_cast<MessageHandler::UDPMessageType>((buffer[0] >> 5) & 0x7);
+		unsigned int msgFlags = buffer[0] & 0x1f;
 
-		if (msgType == Message::Ping) {
+		if (msgType == MessageHandler::UDPPing) {
 			quint64 t;
 			pds >> t;
-			Connection::updatePing(cConnection->dUDPPingAvg, cConnection->dUDPPingVar, cConnection->uiUDPPackets, tTimestamp.elapsed() - t);
-		} else if (msgType == Message::Speex) {
-			ClientPlayer *p = ClientPlayer::get(uiSession);
-			AudioOutputPtr ao = g.ao;
-			if (ao) {
-				if (p) {
-					if (! p->bLocalMute) {
-						unsigned int iSeq;
-						pds >> iSeq;
-						QByteArray qbaSpeexPacket(pds.dataBlock(pds.left()));
-						ao->addFrameToBuffer(p, qbaSpeexPacket, iSeq);
-					}
-				} else {
-					ao->removeBuffer(p);
-				}
-			}
+			accUDP(static_cast<double>(tTimestamp.elapsed() - t) / 1000.0);
+		} else if (msgType == MessageHandler::UDPVoiceCELT) {
+			handleVoicePacket(msgFlags, pds, msgType);
+		} else if (msgType == MessageHandler::UDPVoiceSpeex) {
+			handleVoicePacket(msgFlags, pds, msgType);
 		}
 	}
 }
 
-void ServerHandler::sendMessage(Message *mMsg) {
-	bool mayUdp = (mMsg->messageType() == Message::Speex) || (mMsg->messageType() == Message::Ping);
-	mMsg->uiSession = g.uiSession;
+void ServerHandler::handleVoicePacket(unsigned int msgFlags, PacketDataStream &pds, MessageHandler::UDPMessageType type) {
+	unsigned int uiSession;
+	pds >> uiSession;
+	ClientUser *p = ClientUser::get(uiSession);
+	AudioOutputPtr ao = g.ao;
+	if (ao && p && ! p->bLocalMute && !(((msgFlags & 0x3f) == 2) && g.s.bWhisperFriends && p->qsFriendName.isEmpty())) {
+		unsigned int iSeq;
+		pds >> iSeq;
+		QByteArray qba;
+		qba.reserve(pds.left() + 1);
+		qba.append(static_cast<char>(msgFlags));
+		qba.append(pds.dataBlock(pds.left()));
 
-	if (mayUdp && !NetworkConfig::TcpModeEnabled()) {
-		QMutexLocker qml(&qmUdp);
-		if (! qusUdp)
-			return;
-		if (! cConnection->csCrypt.isValid())
-			return;
-		unsigned char buffer[65536];
-		unsigned char crypto[65540];
-		PacketDataStream pds(buffer, 65536);
-		mMsg->messageToNetwork(pds);
-		cConnection->csCrypt.encrypt(buffer, crypto, pds.size());
-		qusUdp->writeDatagram(reinterpret_cast<const char *>(crypto), pds.size() + 4, qhaRemote, usPort);
+		ao->addFrameToBuffer(p, qba, iSeq, type);
+	}
+}
+
+void ServerHandler::sendMessage(const char *data, int len, bool force) {
+	STACKVAR(unsigned char, crypto, len+4);
+
+	QMutexLocker qml(&qmUdp);
+
+	if (! qusUdp)
+		return;
+	if (! cConnection->csCrypt.isValid())
+		return;
+
+	if (!force && (NetworkConfig::TcpModeEnabled() || !bUdp)) {
+		QByteArray qba;
+
+		qba.resize(len + 4);
+		unsigned char *uc = reinterpret_cast<unsigned char *>(qba.data());
+		uc[0] = MessageHandler::UDPTunnel;
+		uc[1] = static_cast<unsigned char>((len >> 16) & 0xFF);
+		uc[2] = static_cast<unsigned char>((len >> 8) & 0xFF);
+		uc[3] = static_cast<unsigned char>(len & 0xFF);
+		memcpy(uc + 4, data, len);
+
+		QApplication::postEvent(this, new ServerHandlerMessageEvent(qba, MessageHandler::UDPTunnel, true));
 	} else {
-		QByteArray qbaBuffer;
-		mMsg->messageToNetwork(qbaBuffer);
+		cConnection->csCrypt.encrypt(reinterpret_cast<const unsigned char *>(data), crypto, len);
+		qusUdp->writeDatagram(reinterpret_cast<const char *>(crypto), len + 4, qhaRemote, usPort);
+	}
+}
 
-		ServerHandlerMessageEvent *shme=new ServerHandlerMessageEvent(qbaBuffer, mayUdp);
+void ServerHandler::sendProtoMessage(const ::google::protobuf::Message &msg, unsigned int msgType) {
+	QByteArray qba;
+
+	if (QThread::currentThread() != thread()) {
+		Connection::messageToNetwork(msg, msgType, qba);
+		ServerHandlerMessageEvent *shme=new ServerHandlerMessageEvent(qba, 0, false);
 		QApplication::postEvent(this, shme);
+	} else {
+		cConnection->sendMessage(msg, msgType, qba);
 	}
 }
 
 void ServerHandler::run() {
+	qbaDigest = QByteArray();
 	QSslSocket *qtsSock = new QSslSocket(this);
+
+	if (! g.s.bSuppressIdentity && CertWizard::validateCert(g.s.kpCertificate)) {
+		qtsSock->setPrivateKey(g.s.kpCertificate.second);
+		qtsSock->setLocalCertificate(g.s.kpCertificate.first.at(0));
+		QList<QSslCertificate> certs = qtsSock->caCertificates();
+		certs << g.s.kpCertificate.first;
+		qtsSock->setCaCertificates(certs);
+	}
+
 	cConnection = ConnectionPtr(new Connection(this, qtsSock));
 
 	qlErrors.clear();
 	qscCert.clear();
 
 	connect(qtsSock, SIGNAL(encrypted()), this, SLOT(serverConnectionConnected()));
-	connect(cConnection.get(), SIGNAL(connectionClosed(QString)), this, SLOT(serverConnectionClosed(QString)));
-	connect(cConnection.get(), SIGNAL(message(QByteArray &)), this, SLOT(message(QByteArray &)));
+	connect(cConnection.get(), SIGNAL(connectionClosed(const QString &)), this, SLOT(serverConnectionClosed(const QString &)));
+	connect(cConnection.get(), SIGNAL(message(unsigned int, const QByteArray &)), this, SLOT(message(unsigned int, const QByteArray &)));
 	connect(cConnection.get(), SIGNAL(handleSslErrors(const QList<QSslError> &)), this, SLOT(setSslErrors(const QList<QSslError> &)));
+
+	bUdp = false;
+
 	qtsSock->connectToHostEncrypted(qsHostName, usPort);
 
+	tTimestamp.restart();
+
 	QTimer *ticker = new QTimer(this);
 	connect(ticker, SIGNAL(timeout()), this, SLOT(sendPing()));
 	ticker->start(5000);
 
-	g.mw->rtLast = MessageServerReject::None;
+	g.mw->rtLast = MumbleProto::Reject_RejectType_None;
+
+	accUDP = accTCP = accClean;
+
+	uiVersion = 0;
+	qsRelease = QString();
+	qsOS = QString();
+	qsOSVersion = QString();
 
 	exec();
 
@@ -229,7 +281,12 @@ void ServerHandler::run() {
 
 	ticker->stop();
 	cConnection->disconnectSocket(true);
+
+	ConnectionPtr cptr = cConnection;
 	cConnection.reset();
+	while (! cptr.unique()) {
+		msleep(100);
+	}
 }
 
 void ServerHandler::setSslErrors(const QList<QSslError> &errors) {
@@ -242,75 +299,93 @@ void ServerHandler::setSslErrors(const QList<QSslError> &errors) {
 
 void ServerHandler::sendPing() {
 	CryptState &cs = cConnection->csCrypt;
-	MessagePingStats mps;
-	mps.uiTimestamp = tTimestamp.elapsed();
-	mps.uiGood = cs.uiGood;
-	mps.uiLate = cs.uiLate;
-	mps.uiLost = cs.uiLost;
-	mps.uiResync = cs.uiResync;
-	mps.dUDPPingAvg = cConnection->dUDPPingAvg;
-	mps.dUDPPingVar = cConnection->dUDPPingVar;
-	mps.dTCPPingAvg = cConnection->dTCPPingAvg;
-	mps.dTCPPingVar = cConnection->dTCPPingVar;
-	sendMessage(&mps);
 
-	if (!NetworkConfig::TcpModeEnabled()) {
-		MessagePing mp;
-		mp.uiTimestamp = mps.uiTimestamp;
-		sendMessage(&mp);
+	quint64 t = tTimestamp.elapsed();
+
+	if (qusUdp) {
+		unsigned char buffer[256];
+		PacketDataStream pds(buffer + 1, 255);
+		buffer[0] = MessageHandler::UDPPing << 5;
+		pds << t;
+		sendMessage(reinterpret_cast<const char *>(buffer), pds.size() + 1, true);
 	}
+
+	MumbleProto::Ping mpp;
+
+	mpp.set_timestamp(t);
+	mpp.set_good(cs.uiGood);
+	mpp.set_late(cs.uiLate);
+	mpp.set_lost(cs.uiLost);
+	mpp.set_resync(cs.uiResync);
+
+	mpp.set_udp_ping_avg(static_cast<float>(boost::accumulators::mean(accUDP)));
+	mpp.set_udp_ping_var(static_cast<float>(boost::accumulators::variance(accUDP)));
+	mpp.set_udp_packets(static_cast<int>(boost::accumulators::count(accUDP)));
+	mpp.set_tcp_ping_avg(static_cast<float>(boost::accumulators::mean(accTCP)));
+	mpp.set_tcp_ping_var(static_cast<float>(boost::accumulators::variance(accTCP)));
+	mpp.set_tcp_packets(static_cast<int>(boost::accumulators::count(accTCP)));
+	sendMessage(mpp);
 }
 
-void ServerHandler::message(QByteArray &qbaMsg) {
-	Message *mMsg = Message::networkToMessage(qbaMsg);
-	if (! mMsg)
-		return;
+void ServerHandler::message(unsigned int msgType, const QByteArray &qbaMsg) {
+	const char *ptr = qbaMsg.constData();
+	if (msgType == MessageHandler::UDPTunnel) {
+		if (qbaMsg.length() < 1)
+			return;
 
-	ClientPlayer *p = ClientPlayer::get(mMsg->uiSession);
-	AudioOutputPtr ao = g.ao;
+		MessageHandler::UDPMessageType msgType = static_cast<MessageHandler::UDPMessageType>((ptr[0] >> 5) & 0x7);
+		unsigned int msgFlags = ptr[0] & 0x1f;
+		PacketDataStream pds(qbaMsg.constData() + 1, qbaMsg.size());
 
-	if (mMsg->messageType() == Message::Speex) {
-		if (ao) {
-			if (p) {
-				MessageSpeex *msMsg=static_cast<MessageSpeex *>(mMsg);
-				if (! p->bLocalMute)
-					ao->addFrameToBuffer(p, msMsg->qbaSpeexPacket, msMsg->iSeq);
-			} else {
-				// Eek, we just got a late packet for a player already removed. Remove
-				// the buffer and pretend this never happened.
-				// If ~AudioOutputPlayer or decendants uses the Player object now,
-				// Bad Things happen.
-				ao->removeBuffer(p);
+		if (msgType == MessageHandler::UDPVoiceCELT)
+			handleVoicePacket(msgFlags, pds, msgType);
+		else if (msgType == MessageHandler::UDPVoiceSpeex)
+			handleVoicePacket(msgFlags, pds, msgType);
+	} else if (msgType == MessageHandler::Ping) {
+		MumbleProto::Ping msg;
+		if (msg.ParseFromArray(qbaMsg.constData(), qbaMsg.size())) {
+			CryptState &cs = cConnection->csCrypt;
+			cs.uiRemoteGood = msg.good();
+			cs.uiRemoteLate = msg.late();
+			cs.uiRemoteLost = msg.lost();
+			cs.uiRemoteResync = msg.resync();
+			accTCP(static_cast<double>(tTimestamp.elapsed() - msg.timestamp()) / 1000.0);
+
+			if (((cs.uiRemoteGood == 0) || (cs.uiGood == 0)) && bUdp && (tTimestamp.elapsed() > 20000000ULL)) {
+				bUdp = false;
+				if (! NetworkConfig::TcpModeEnabled()) {
+					if ((cs.uiRemoteGood == 0) && (cs.uiRemoteGood == 0))
+						g.mw->msgBox(tr("UDP packets can not be sent to or received from the server. Switching to TCP mode."));
+					else if (cs.uiRemoteGood == 0)
+						g.mw->msgBox(tr("UDP packets can not be sent to the server. Switching to TCP mode."));
+					else
+						g.mw->msgBox(tr("UDP packets can not be received from the server. Switching to TCP mode."));
+
+					Database::setUdp(qbaDigest, false);
+				}
+			} else if (!bUdp && (cs.uiRemoteGood > 3) && (cs.uiGood > 3)) {
+				bUdp = true;
+				if (! NetworkConfig::TcpModeEnabled()) {
+					g.mw->msgBox(tr("UDP packets can be sent to and received from the server. Switching back to UDP mode."));
+
+					Database::setUdp(qbaDigest, true);
+				}
 			}
 		}
-	} else if (mMsg->messageType() == Message::PingStats) {
-		MessagePingStats *mpsMsg = static_cast<MessagePingStats *>(mMsg);
-		CryptState &cs = cConnection->csCrypt;
-		cs.uiRemoteGood = mpsMsg->uiGood;
-		cs.uiRemoteLate = mpsMsg->uiLate;
-		cs.uiRemoteLost = mpsMsg->uiLost;
-		cs.uiRemoteResync = mpsMsg->uiResync;
-		Connection::updatePing(cConnection->dTCPPingAvg, cConnection->dTCPPingVar, cConnection->uiTCPPackets, tTimestamp.elapsed() - mpsMsg->uiTimestamp);
 	} else {
-		if (mMsg->messageType() == Message::ServerLeave) {
-			if (ao)
-				ao->removeBuffer(p);
-		}
-		ServerHandlerMessageEvent *shme=new ServerHandlerMessageEvent(qbaMsg, false);
+		ServerHandlerMessageEvent *shme=new ServerHandlerMessageEvent(qbaMsg, msgType, false);
 		QApplication::postEvent(g.mw, shme);
 	}
-
-	delete mMsg;
 }
 
 void ServerHandler::disconnect() {
 	// Actual TCP object is in a different thread, so signal it
 	QByteArray qbaBuffer;
-	ServerHandlerMessageEvent *shme=new ServerHandlerMessageEvent(qbaBuffer, false);
+	ServerHandlerMessageEvent *shme=new ServerHandlerMessageEvent(qbaBuffer, 0, false);
 	QApplication::postEvent(this, shme);
 }
 
-void ServerHandler::serverConnectionClosed(QString reason) {
+void ServerHandler::serverConnectionClosed(const QString &reason) {
 	AudioOutputPtr ao = g.ao;
 	if (ao)
 		ao->wipe();
@@ -319,25 +394,56 @@ void ServerHandler::serverConnectionClosed(QString reason) {
 }
 
 void ServerHandler::serverConnectionConnected() {
+	cConnection->setToS();
+
 	qscCert = cConnection->peerCertificateChain();
 	qscCipher = cConnection->sessionCipher();
 
-	cConnection->setToS();
+	if (! qscCert.isEmpty()) {
+		const QSslCertificate &qsc = qscCert.last();
+		qbaDigest = QCryptographicHash::hash(qsc.publicKey().toDer(), QCryptographicHash::Sha1);
+		bUdp = Database::getUdp(qbaDigest);
+	} else {
+		bUdp = true;
+	}
 
-	MessageServerAuthenticate msaMsg;
-	msaMsg.qsUsername = qsUserName;
-	msaMsg.qsPassword = qsPassword;
+	MumbleProto::Version mpv;
+	mpv.set_release(u8(QLatin1String(MUMBLE_RELEASE)));
 
-	cConnection->sendMessage(&msaMsg);
+	QRegExp rx(QLatin1String("(\\d+)\\.(\\d+)\\.(\\d+)"));
+	if (rx.exactMatch(QLatin1String(MUMTEXT(MUMBLE_VERSION_STRING)))) {
+		int major = rx.cap(1).toInt();
+		int minor = rx.cap(2).toInt();
+		int patch = rx.cap(3).toInt();
+		mpv.set_version((major << 16) | (minor << 8) | patch);
+	}
+	mpv.set_os(u8(OSInfo::getOS()));
+	mpv.set_os_version(u8(OSInfo::getOSVersion()));
+	sendMessage(mpv);
+
+	MumbleProto::Authenticate mpa;
+	mpa.set_username(u8(qsUserName));
+	mpa.set_password(u8(qsPassword));
+
+	QStringList tokens = Database::getTokens(qbaDigest);
+	foreach(const QString &qs, tokens)
+		mpa.add_tokens(u8(qs));
+
+	sendMessage(mpa);
 
 	{
 		QMutexLocker qml(&qmUdp);
 
+		qhaRemote = cConnection->peerAddress();
+
 		qusUdp = new QUdpSocket(this);
-		qusUdp->bind();
+		if (qhaRemote.protocol() == QAbstractSocket::IPv6Protocol)
+			qusUdp->bind(QHostAddress(QHostAddress::AnyIPv6), 0);
+		else
+			qusUdp->bind(QHostAddress(QHostAddress::Any), 0);
+
 		connect(qusUdp, SIGNAL(readyRead()), this, SLOT(udpReady()));
 
-		qhaRemote = cConnection->peerAddress();
 
 #if defined(Q_OS_UNIX)
 		int val = 0xe0;
