@@ -35,16 +35,18 @@
 
 #include "mumble_pch.hpp"
 #include "Timer.h"
-
+#include "Message.h"
+#include "PacketDataStream.h"
 
 class Connection;
 class Message;
 
 class ServerHandlerMessageEvent : public QEvent {
 	public:
+		unsigned int uiType;
 		QByteArray qbaMsg;
 		bool bFlush;
-		ServerHandlerMessageEvent(QByteArray &msg, bool flush);
+		ServerHandlerMessageEvent(const QByteArray &msg, unsigned int type, bool flush = false);
 };
 
 typedef boost::shared_ptr<Connection> ConnectionPtr;
@@ -59,6 +61,7 @@ class ServerHandler : public QThread {
 		QString qsPassword;
 		QString qsDigest;
 		unsigned short usPort;
+		bool bUdp;
 
 #ifdef Q_OS_WIN
 		HANDLE hQoS;
@@ -68,28 +71,45 @@ class ServerHandler : public QThread {
 		QHostAddress qhaRemote;
 		QUdpSocket *qusUdp;
 		QMutex qmUdp;
-		Timer tTimestamp;
+
+		void handleVoicePacket(unsigned int msgFlags, PacketDataStream &pds, MessageHandler::UDPMessageType type);
 	public:
+		Timer tTimestamp;
 		QList<QSslError> qlErrors;
 		QList<QSslCertificate> qscCert;
 		QSslCipher qscCipher;
 		ConnectionPtr cConnection;
+		QByteArray qbaDigest;
+
+		unsigned int uiVersion;
+		QString qsRelease;
+		QString qsOS;
+		QString qsOSVersion;
+
+		boost::accumulators::accumulator_set<double, boost::accumulators::stats<boost::accumulators::tag::mean, boost::accumulators::tag::variance, boost::accumulators::tag::count> > accTCP, accUDP, accClean;
 
 		ServerHandler();
 		~ServerHandler();
 		void setConnectionInfo(const QString &host, unsigned short port, const QString &username, const QString &pw);
 		void getConnectionInfo(QString &host, unsigned short &port, QString &username, QString &pw);
 		void customEvent(QEvent *evt);
-		void sendMessage(Message *m);
+
+		void sendProtoMessage(const ::google::protobuf::Message &msg, unsigned int msgType);
+		void sendMessage(const char *data, int len, bool force = false);
+
+#define MUMBLE_MH_MSG(x) void sendMessage(const MumbleProto:: x &msg) { sendProtoMessage(msg, MessageHandler:: x); }
+		MUMBLE_MH_ALL
+#undef MUMBLE_MH_MSG
+
 		void disconnect();
 		void run();
 	signals:
-		void disconnected(QString reason);
+		void disconnected(QAbstractSocket::SocketError, QString reason);
 		void connected();
 	protected slots:
-		void message(QByteArray &);
+		void message(unsigned int, const QByteArray &);
 		void serverConnectionConnected();
-		void serverConnectionClosed(QString);
+		void serverConnectionClosed(QAbstractSocket::SocketError, const QString &);
 		void setSslErrors(const QList<QSslError> &);
 		void udpReady();
 		void sendPing();
