@@ -32,52 +32,145 @@
 #include "Log.h"
 #include "Global.h"
 #include "AudioInput.h"
+#include "Cert.h"
+
+bool Shortcut::isServerSpecific() const {
+	if (qvData.canConvert<ShortcutTarget>()) {
+		const ShortcutTarget &sc = qvariant_cast<ShortcutTarget>(qvData);
+		return sc.isServerSpecific();
+	}
+	return false;
+}
 
 bool Shortcut::operator <(const Shortcut &other) const {
 	return (iIndex < other.iIndex);
 }
 
+bool Shortcut::operator ==(const Shortcut &other) const {
+	return (iIndex == other.iIndex) && (qlButtons == other.qlButtons) && (qvData == other.qvData) && (bSuppress == other.bSuppress);
+}
+
+ShortcutTarget::ShortcutTarget() {
+	bUsers = true;
+	iChannel = -3;
+	bLinks = bChildren = bForceCenter = false;
+}
+
+bool ShortcutTarget::isServerSpecific() const {
+	return (! bUsers && (iChannel >= 0));
+}
+
+bool ShortcutTarget::operator ==(const ShortcutTarget &o) const {
+	if ((bUsers != o.bUsers) || (bForceCenter != o.bForceCenter))
+		return false;
+	if (bUsers)
+		return (qlUsers == o.qlUsers) && (qlSessions == o.qlSessions);
+	else
+		return (iChannel == o.iChannel) && (bLinks == o.bLinks) && (bChildren == o.bChildren) && (qsGroup == o.qsGroup);
+}
+
+quint32 qHash(const ShortcutTarget &t) {
+	quint32 h = t.bForceCenter ? 0x55555555 : 0xaaaaaaaa;
+	if (t.bUsers) {
+		foreach(unsigned int u, t.qlSessions)
+			h ^= u;
+	} else {
+		h ^= t.iChannel;
+		if (t.bLinks)
+			h ^= 0x80000000;
+		if (t.bChildren)
+			h ^= 0x40000000;
+		h ^= qHash(t.qsGroup);
+		h = ~h;
+	}
+	return h;
+}
+
+quint32 qHash(const QList<ShortcutTarget> &l) {
+	quint32 h = l.count();
+	foreach(const ShortcutTarget &st, l)
+		h ^= qHash(st);
+	return h;
+}
+
+QDataStream &operator<<(QDataStream &qds, const ShortcutTarget &st) {
+	qds << st.bUsers << st.bForceCenter;
+
+	if (st.bUsers)
+		return qds << st.qlUsers;
+	else
+		return qds << st.iChannel << st.qsGroup << st.bLinks << st.bChildren;
+}
+
+QDataStream &operator>>(QDataStream &qds, ShortcutTarget &st) {
+	qds >> st.bUsers >> st.bForceCenter;
+	if (st.bUsers)
+		return qds >> st.qlUsers;
+	else
+		return qds >> st.iChannel >> st.qsGroup >> st.bLinks >> st.bChildren;
+}
+
+const QString Settings::cqsDefaultPushClickOn = QLatin1String(":/on.ogg");
+const QString Settings::cqsDefaultPushClickOff = QLatin1String(":/off.ogg");
+
 Settings::Settings() {
+	qRegisterMetaType<ShortcutTarget>("ShortcutTarget");
+	qRegisterMetaTypeStreamOperators<ShortcutTarget>("ShortcutTarget");
+	qRegisterMetaType<QVariant>("QVariant");
+
 	atTransmit = VAD;
 	bTransmitPosition = false;
 	bMute = bDeaf = false;
 	bTTS = true;
 	iTTSVolume = 75;
 	iTTSThreshold = 250;
-	iQuality = 6;
+	iQuality = 32000;
 	fVolume = 1.0f;
 	fOtherVolume = 0.5f;
 	iMinLoudness = 1000;
 	iVoiceHold = 50;
 	iJitterBufferSize = 1;
-	iFramesPerPacket = 2;
+	iFramesPerPacket = 1;
 	iNoiseSuppress = -30;
 	iIdleTime = 0;
 	vsVAD = SignalToNoise;
 	fVADmin = 0.80f;
 	fVADmax = 0.98f;
-	bPushClick = false;
 
-	bPlayerTop = false;
+	bPushClick = false;
+	qsPushClickOn = cqsDefaultPushClickOn;
+	qsPushClickOff = cqsDefaultPushClickOff;
+
+	bUserTop = false;
+
+	bWhisperFriends = false;
 
 	uiDoublePush = 0;
 	bExpert = false;
 
 #ifdef NO_UPDATE_CHECK
 	bUpdateCheck = false;
+	bPluginCheck = false;
 #else
 	bUpdateCheck = true;
+	bPluginCheck = true;
 #endif
 
 	bFirstTime = true;
-	ceExpand = ChannelsWithPlayers;
+	ceExpand = ChannelsWithUsers;
 	ceChannelDrag = Ask;
+	bShowChatbar = true;
 	bMinimalView = false;
 	bHideFrame = false;
 	bAlwaysOnTop = false;
 	bAskOnQuit = true;
 	bHideTray = true;
 	bUsage = true;
+	bAdvancedACLCfg = false;
+	iWindowLayout = LayoutClassic;
+
+	bHideUnreachable = true;
+	bHideEmpty = false;
 
 	iOutputDelay = 5;
 
@@ -85,6 +178,7 @@ Settings::Settings() {
 	qsALSAOutput=QLatin1String("default");
 
 	bEcho = false;
+	bEchoMulti = true;
 
 	iPortAudioInput = -1; // default device
 	iPortAudioOutput = -1; // default device
@@ -112,14 +206,14 @@ Settings::Settings() {
 	qfOverlayFont = QFont(QLatin1String("Arial"), 20);
 #endif
 	fOverlayHeight = .03f;
-	qcOverlayPlayer = QColor(255,255,255,128);
+	qcOverlayUser = QColor(255,255,255,128);
 	qcOverlayTalking = QColor(255,255,196,255);
-	qcOverlayAltTalking = QColor(255,128,128,255);
+	qcOverlayWhisper = QColor(255,128,128,255);
 	qcOverlayChannel = QColor(192,192,255,192);
 	qcOverlayChannelTalking = QColor(224,224,255,255);
 
-	iLCDPlayerViewMinColWidth = 50;
-	iLCDPlayerViewSplitterWidth = 2;
+	iLCDUserViewMinColWidth = 50;
+	iLCDUserViewSplitterWidth = 2;
 
 	// Network settings
 	bTCPCompat = false;
@@ -127,7 +221,12 @@ Settings::Settings() {
 	ptProxyType = NoProxy;
 	usProxyPort = 0;
 
-#ifdef AUDIO_TEST
+	iMaxImageSize = ciDefaultMaxImageSize;
+	iMaxImageWidth = 1024; // Allow 1024x1024 resolution
+	iMaxImageHeight = 1024;
+	bSuppressIdentity = false;
+
+#if defined(AUDIO_TEST)
 	lmLoopMode = Local;
 #else
 	lmLoopMode = None;
@@ -143,8 +242,6 @@ Settings::Settings() {
 
 	qmMessages[Log::DebugInfo] = Settings::LogConsole;
 	qmMessages[Log::Information] = Settings::LogConsole;
-
-	iServerRow = -1;
 }
 
 bool Settings::doEcho() const {
@@ -191,6 +288,8 @@ void Settings::load() {
 	LOADENUM(atTransmit, "audio/transmit");
 	SAVELOAD(uiDoublePush, "audio/doublepush");
 	SAVELOAD(bPushClick, "audio/pushclick");
+	SAVELOAD(qsPushClickOn, "audio/pushclickon");
+	SAVELOAD(qsPushClickOff, "audio/pushclickoff");
 	SAVELOAD(iQuality, "audio/quality");
 	SAVELOAD(iMinLoudness, "audio/loudness");
 	SAVELOAD(fVolume, "audio/volume");
@@ -207,10 +306,12 @@ void Settings::load() {
 	SAVELOAD(fAudioMaxDistVolume, "audio/maxdistancevolume");
 	SAVELOAD(fAudioBloom, "audio/bloom");
 	SAVELOAD(bEcho, "audio/echo");
+	SAVELOAD(bEchoMulti, "audio/echomulti");
 	SAVELOAD(bPositionalAudio, "audio/positional");
 	SAVELOAD(bPositionalHeadphone, "audio/headphone");
 	SAVELOAD(qsAudioInput, "audio/input");
 	SAVELOAD(qsAudioOutput, "audio/output");
+	SAVELOAD(bWhisperFriends, "audio/whisperfriends");
 
 	SAVELOAD(iJitterBufferSize, "net/jitterbuffer");
 	SAVELOAD(iFramesPerPacket, "net/framesperpacket");
@@ -254,7 +355,7 @@ void Settings::load() {
 	SAVELOAD(bOverlayRight, "overlay/right");
 	SAVELOAD(qfOverlayFont, "overlay/font");
 	SAVELOAD(fOverlayHeight, "overlay/height");
-	SAVELOAD(qcOverlayPlayer, "overlay/player");
+	SAVELOAD(qcOverlayUser, "overlay/user");
 	SAVELOAD(qcOverlayTalking, "overlay/talking");
 	SAVELOAD(qcOverlayChannel, "overlay/channel");
 	SAVELOAD(qcOverlayChannelTalking, "overlay/channeltalking");
@@ -262,11 +363,15 @@ void Settings::load() {
 	// Network settings
 	SAVELOAD(bTCPCompat, "net/tcponly");
 	SAVELOAD(bReconnect, "net/reconnect");
+	SAVELOAD(bSuppressIdentity, "net/suppress");
 	LOADENUM(ptProxyType, "net/proxytype");
 	SAVELOAD(qsProxyHost, "net/proxyhost");
 	SAVELOAD(usProxyPort, "net/proxyport");
 	SAVELOAD(qsProxyUsername, "net/proxyusername");
 	SAVELOAD(qsProxyPassword, "net/proxypassword");
+	SAVELOAD(iMaxImageSize, "net/maximagesize");
+	SAVELOAD(iMaxImageWidth, "net/maximagewidth");
+	SAVELOAD(iMaxImageHeight, "net/maximageheight");
 
 	SAVELOAD(bExpert, "ui/expert");
 	SAVELOAD(qsLanguage, "ui/language");
@@ -276,24 +381,36 @@ void Settings::load() {
 	LOADENUM(ceChannelDrag, "ui/drag");
 	SAVELOAD(bAlwaysOnTop, "ui/alwaysontop");
 	SAVELOAD(bAskOnQuit, "ui/askonquit");
+	SAVELOAD(bShowChatbar, "ui/shotchatbar");
 	SAVELOAD(bMinimalView, "ui/minimalview");
 	SAVELOAD(bHideFrame, "ui/hideframe");
-	SAVELOAD(bPlayerTop, "ui/playertop");
+	SAVELOAD(bUserTop, "ui/usertop");
 	SAVELOAD(bFirstTime, "ui/firsttime");
 	SAVELOAD(qbaMainWindowGeometry, "ui/geometry");
 	SAVELOAD(qbaMainWindowState, "ui/state");
 	SAVELOAD(qbaMinimalViewGeometry, "ui/minimalviewgeometry");
 	SAVELOAD(qbaConfigGeometry, "ui/ConfigGeometry");
+	SAVELOAD(iWindowLayout, "ui/WindowLayout");
 	SAVELOAD(qbaSplitterState, "ui/splitter");
 	SAVELOAD(qbaHeaderState, "ui/header");
 	SAVELOAD(qsUsername, "ui/username");
-	SAVELOAD(iServerRow, "ui/serverrow");
+	SAVELOAD(qsLastServer, "ui/server");
+	SAVELOAD(bHideUnreachable, "ui/hideunreachable");
+	SAVELOAD(bHideEmpty, "ui/hideempty");
+#ifndef NO_UPDATE_CHECK
 	SAVELOAD(bUpdateCheck, "ui/updatecheck");
+	SAVELOAD(bPluginCheck, "ui/plugincheck");
+#endif
 	SAVELOAD(bHideTray, "ui/hidetray");
 	SAVELOAD(bUsage, "ui/usage");
+	SAVELOAD(bAdvancedACLCfg, "ui/advancedaclcfg");
 
-	SAVELOAD(iLCDPlayerViewMinColWidth, "lcd/playerview/mincolwidth");
-	SAVELOAD(iLCDPlayerViewSplitterWidth, "lcd/playerview/splitterwidth");
+	SAVELOAD(iLCDUserViewMinColWidth, "lcd/userview/mincolwidth");
+	SAVELOAD(iLCDUserViewSplitterWidth, "lcd/userview/splitterwidth");
+
+	QByteArray qba = qvariant_cast<QByteArray>(g.qs->value(QLatin1String("net/certificate")));
+	if (! qba.isEmpty())
+		kpCertificate = CertWizard::importCert(qba);
 
 	int nshorts = g.qs->beginReadArray(QLatin1String("shortcuts"));
 	for (int i=0;i<nshorts;i++) {
@@ -305,6 +422,7 @@ void Settings::load() {
 		SAVELOAD(s.iIndex, "index");
 		SAVELOAD(s.qlButtons, "keys");
 		SAVELOAD(s.bSuppress, "suppress");
+		s.qvData = g.qs->value(QLatin1String("data"));
 		if (s.iIndex >= -1)
 			qlShortcuts << s;
 	}
@@ -342,6 +460,8 @@ void Settings::save() {
 	SAVELOAD(atTransmit, "audio/transmit");
 	SAVELOAD(uiDoublePush, "audio/doublepush");
 	SAVELOAD(bPushClick, "audio/pushclick");
+	SAVELOAD(qsPushClickOn, "audio/pushclickon");
+	SAVELOAD(qsPushClickOff, "audio/pushclickoff");
 	SAVELOAD(iQuality, "audio/quality");
 	SAVELOAD(iMinLoudness, "audio/loudness");
 	SAVELOAD(fVolume, "audio/volume");
@@ -358,10 +478,12 @@ void Settings::save() {
 	SAVELOAD(fAudioMaxDistVolume, "audio/maxdistancevolume");
 	SAVELOAD(fAudioBloom, "audio/bloom");
 	SAVELOAD(bEcho, "audio/echo");
+	SAVELOAD(bEchoMulti, "audio/echomulti");
 	SAVELOAD(bPositionalAudio, "audio/positional");
 	SAVELOAD(bPositionalHeadphone, "audio/headphone");
 	SAVELOAD(qsAudioInput, "audio/input");
 	SAVELOAD(qsAudioOutput, "audio/output");
+	SAVELOAD(bWhisperFriends, "audio/whisperfriends");
 
 	SAVELOAD(iJitterBufferSize, "net/jitterbuffer");
 	SAVELOAD(iFramesPerPacket, "net/framesperpacket");
@@ -405,7 +527,7 @@ void Settings::save() {
 	SAVELOAD(bOverlayRight, "overlay/right");
 	SAVELOAD(qfOverlayFont, "overlay/font");
 	SAVELOAD(fOverlayHeight, "overlay/height");
-	SAVELOAD(qcOverlayPlayer, "overlay/player");
+	SAVELOAD(qcOverlayUser, "overlay/user");
 	SAVELOAD(qcOverlayTalking, "overlay/talking");
 	SAVELOAD(qcOverlayChannel, "overlay/channel");
 	SAVELOAD(qcOverlayChannelTalking, "overlay/channeltalking");
@@ -418,6 +540,9 @@ void Settings::save() {
 	SAVELOAD(usProxyPort, "net/proxyport");
 	SAVELOAD(qsProxyUsername, "net/proxyusername");
 	SAVELOAD(qsProxyPassword, "net/proxypassword");
+	SAVELOAD(iMaxImageSize, "net/maximagesize");
+	SAVELOAD(iMaxImageWidth, "net/maximagewidth");
+	SAVELOAD(iMaxImageHeight, "net/maximageheight");
 
 	SAVELOAD(bExpert, "ui/expert");
 	SAVELOAD(qsLanguage, "ui/language");
@@ -427,32 +552,44 @@ void Settings::save() {
 	SAVELOAD(ceChannelDrag, "ui/drag");
 	SAVELOAD(bAlwaysOnTop, "ui/alwaysontop");
 	SAVELOAD(bAskOnQuit, "ui/askonquit");
+	SAVELOAD(bShowChatbar, "ui/showchatbar");
 	SAVELOAD(bMinimalView, "ui/minimalview");
 	SAVELOAD(bHideFrame, "ui/hideframe");
-	SAVELOAD(bPlayerTop, "ui/playertop");
+	SAVELOAD(bUserTop, "ui/usertop");
 	SAVELOAD(bFirstTime, "ui/firsttime");
 	SAVELOAD(qbaMainWindowGeometry, "ui/geometry");
 	SAVELOAD(qbaMainWindowState, "ui/state");
 	SAVELOAD(qbaMinimalViewGeometry, "ui/minimalviewgeometry");
 	SAVELOAD(qbaConfigGeometry, "ui/ConfigGeometry");
+	SAVELOAD(iWindowLayout, "ui/WindowLayout");
 	SAVELOAD(qbaSplitterState, "ui/splitter");
 	SAVELOAD(qbaHeaderState, "ui/header");
 	SAVELOAD(qsUsername, "ui/username");
-	SAVELOAD(iServerRow, "ui/serverrow");
+	SAVELOAD(qsLastServer, "ui/server");
+	SAVELOAD(bHideUnreachable, "ui/hideunreachable");
+	SAVELOAD(bHideEmpty, "ui/hideempty");
 	SAVELOAD(bUpdateCheck, "ui/updatecheck");
+	SAVELOAD(bPluginCheck, "ui/plugincheck");
 	SAVELOAD(bHideTray, "ui/hidetray");
 	SAVELOAD(bUsage, "ui/usage");
+	SAVELOAD(bAdvancedACLCfg, "ui/advancedaclcfg");
 
-	SAVELOAD(iLCDPlayerViewMinColWidth, "lcd/playerview/mincolwidth");
-	SAVELOAD(iLCDPlayerViewSplitterWidth, "lcd/playerview/splitterwidth");
+	SAVELOAD(iLCDUserViewMinColWidth, "lcd/userview/mincolwidth");
+	SAVELOAD(iLCDUserViewSplitterWidth, "lcd/userview/splitterwidth");
+
+	QByteArray qba = CertWizard::exportCert(kpCertificate);
+	g.qs->setValue(QLatin1String("net/certificate"), qba);
 
 	g.qs->beginWriteArray(QLatin1String("shortcuts"));
 	int idx = 0;
 	foreach(const Shortcut &s, qlShortcuts) {
-		g.qs->setArrayIndex(idx++);
-		g.qs->setValue(QLatin1String("index"), s.iIndex);
-		g.qs->setValue(QLatin1String("keys"), s.qlButtons);
-		g.qs->setValue(QLatin1String("suppress"), s.bSuppress);
+		if (! s.isServerSpecific()) {
+			g.qs->setArrayIndex(idx++);
+			g.qs->setValue(QLatin1String("index"), s.iIndex);
+			g.qs->setValue(QLatin1String("keys"), s.qlButtons);
+			g.qs->setValue(QLatin1String("suppress"), s.bSuppress);
+			g.qs->setValue(QLatin1String("data"), s.qvData);
+		}
 	}
 	g.qs->endArray();
 
