@@ -33,7 +33,7 @@
 
 #include "mumble_pch.hpp"
 
-// Global helper class to spread variables around across threads
+// Global helper classes to spread variables around across threads
 // especially helpfull to initialize things like the stored
 // preference for audio transmission, since the GUI elements
 // will be created long before the AudioInput object, and the
@@ -43,27 +43,60 @@
 struct Shortcut {
 	int iIndex;
 	QList<QVariant> qlButtons;
+	QVariant qvData;
 	bool bSuppress;
 	bool operator <(const Shortcut &) const;
+	bool isServerSpecific() const;
+	bool operator ==(const Shortcut &) const;
 };
 
+struct ShortcutTarget {
+	bool bUsers;
+	QStringList qlUsers;
+	QList<unsigned int> qlSessions;
+	int iChannel;
+	QString qsGroup;
+	bool bLinks;
+	bool bChildren;
+	bool bForceCenter;
+	ShortcutTarget();
+	bool isServerSpecific() const;
+	bool operator <(const ShortcutTarget &) const;
+	bool operator ==(const ShortcutTarget &) const;
+};
+
+quint32 qHash(const ShortcutTarget &);
+quint32 qHash(const QList<ShortcutTarget> &);
+
+QDataStream &operator<<(QDataStream &, const ShortcutTarget &);
+QDataStream &operator>>(QDataStream &, ShortcutTarget &);
+Q_DECLARE_METATYPE(ShortcutTarget);
+
 struct Settings {
 	enum AudioTransmit { Continous, VAD, PushToTalk };
 	enum VADSource { Amplitude, SignalToNoise };
 	enum LoopMode { None, Local, Server };
 	enum OverlayShow { Nothing, Talking, All };
-	enum ChannelExpand { NoChannels, ChannelsWithPlayers, AllChannels };
+	enum ChannelExpand { NoChannels, ChannelsWithUsers, AllChannels };
 	enum ChannelDrag { Ask, DoNothing, Move };
+	typedef QPair<QList<QSslCertificate>, QSslKey> KeyPair;
+
 	AudioTransmit atTransmit;
 	quint64 uiDoublePush;
 
 	bool bExpert;
 
-	bool bTransmitPosition;
 	bool bPushClick;
+	static const QString cqsDefaultPushClickOn;
+	static const QString cqsDefaultPushClickOff;
+	QString qsPushClickOn;
+	QString qsPushClickOff;
+
+	bool bTransmitPosition;
 	bool bMute, bDeaf;
 	bool bTTS;
-	bool bPlayerTop;
+	bool bUserTop;
+	bool bWhisperFriends;
 	int iTTSVolume, iTTSThreshold;
 	int iQuality, iMinLoudness, iVoiceHold, iJitterBufferSize;
 	int iNoiseSuppress;
@@ -89,6 +122,7 @@ struct Settings {
 	QByteArray qbaDXInput, qbaDXOutput;
 
 	bool bEcho;
+	bool bEchoMulti;
 	bool bPositionalAudio;
 	bool bPositionalHeadphone;
 	float fAudioMinDistance, fAudioMaxDistance, fAudioMaxDistVolume, fAudioBloom;
@@ -102,14 +136,14 @@ struct Settings {
 	bool bOverlayLeft, bOverlayRight, bOverlayTop, bOverlayBottom;
 	QFont qfOverlayFont;
 	float fOverlayHeight;
-	QColor qcOverlayPlayer;
+	QColor qcOverlayUser;
 	QColor qcOverlayTalking;
-	QColor qcOverlayAltTalking;
+	QColor qcOverlayWhisper;
 	QColor qcOverlayChannel;
 	QColor qcOverlayChannelTalking;
 
-	int iLCDPlayerViewMinColWidth;
-	int iLCDPlayerViewSplitterWidth;
+	int iLCDUserViewMinColWidth;
+	int iLCDUserViewSplitterWidth;
 	QMap<QString, bool> qmLCDDevices;
 
 	QList<Shortcut> qlShortcuts;
@@ -124,19 +158,26 @@ struct Settings {
 	bool bFirstTime;
 	QByteArray qbaMainWindowGeometry, qbaMainWindowState, qbaMinimalViewGeometry, qbaSplitterState, qbaHeaderState;
 	QByteArray qbaConfigGeometry;
+	enum WindowLayout { LayoutClassic, LayoutStacked, LayoutHybrid, LayoutCustom };
+	int iWindowLayout;
 	ChannelExpand ceExpand;
 	ChannelDrag ceChannelDrag;
+	bool bShowChatbar;
 	bool bMinimalView;
 	bool bHideFrame;
 	bool bAlwaysOnTop;
 	bool bAskOnQuit;
 	bool bHideTray;
 	bool bUsage;
+	bool bAdvancedACLCfg;
 
 	QString qsUsername;
-	int iServerRow;
+	QString qsLastServer;
+	bool bHideUnreachable;
+	bool bHideEmpty;
 
 	bool bUpdateCheck;
+	bool bPluginCheck;
 
 	// Network settings
 	enum ProxyType { NoProxy, HttpProxy, Socks5Proxy };
@@ -145,6 +186,13 @@ struct Settings {
 	QString qsProxyHost, qsProxyUsername, qsProxyPassword;
 	unsigned short usProxyPort;
 
+	static const int ciDefaultMaxImageSize = 50 * 1024; // Restrict to 50KiB as a default
+	int iMaxImageSize;
+	int iMaxImageWidth;
+	int iMaxImageHeight;
+	KeyPair kpCertificate;
+	bool bSuppressIdentity;
+
 	// Nonsaved
 	LoopMode lmLoopMode;
 	float dPacketLoss;
