@@ -29,44 +29,24 @@
 */
 
 #include "WASAPI.h"
-#include "Player.h"
+#include "User.h"
 #include "Global.h"
 #include "MainWindow.h"
 #include "Timer.h"
 
-// Note that these interfaces are incomplete. They're not published, so I
-// have no idea what they're actually called or what the methods are
-// supposed to be named.
+
+// Now that Win7 is published, which includes public versions of these
+// interfaces, we simply inherit from those but use the "old" IIDs.
 
 MIDL_INTERFACE("33969B1D-D06F-4281-B837-7EAAFD21A9C0")
-IRemoteAudioSession :
-public IUnknown {
-	virtual HRESULT STDMETHODCALLTYPE _a() = 0;
-	virtual HRESULT STDMETHODCALLTYPE _b() = 0; // 0x10
-	virtual HRESULT STDMETHODCALLTYPE _c() = 0;
-	virtual HRESULT STDMETHODCALLTYPE _d() = 0;
-	virtual HRESULT STDMETHODCALLTYPE _e() = 0;
-	virtual HRESULT STDMETHODCALLTYPE _f() = 0; // 0x20
-	virtual HRESULT STDMETHODCALLTYPE _g() = 0;
-	virtual HRESULT STDMETHODCALLTYPE _h() = 0;
-	virtual HRESULT STDMETHODCALLTYPE _i() = 0;
-	virtual HRESULT STDMETHODCALLTYPE _j() = 0; // 0x30
-	virtual HRESULT STDMETHODCALLTYPE _k() = 0;
-	virtual HRESULT STDMETHODCALLTYPE GetProcessId(DWORD *) = 0;
-};
-
-// No idea what it's REFIID actually is.
-MIDL_INTERFACE("94BE9D30-53AC-4802-829C-F13E5AD34776")
-IAudioQuerier :
-public IUnknown {
-	virtual HRESULT STDMETHODCALLTYPE GetNumSessions(DWORD *) = 0;
-	virtual HRESULT STDMETHODCALLTYPE QuerySession(DWORD, IUnknown **) = 0;
+IVistaAudioSessionControl2 :
+public IAudioSessionControl2 {
 };
 
 MIDL_INTERFACE("94BE9D30-53AC-4802-829C-F13E5AD34775")
 IAudioSessionQuery :
 public IUnknown {
-	virtual HRESULT STDMETHODCALLTYPE GetQueryInterface(IAudioQuerier **) = 0;
+	virtual HRESULT STDMETHODCALLTYPE GetQueryInterface(IAudioSessionEnumerator **) = 0;
 };
 
 class WASAPIInputRegistrar : public AudioInputRegistrar {
@@ -85,7 +65,6 @@ class WASAPIOutputRegistrar : public AudioOutputRegistrar {
 		virtual const QList<audioDevice> getDeviceChoices();
 		virtual void setDeviceChoice(const QVariant &, Settings &);
 		bool canMuteOthers() const;
-		bool usesOutputDelay() const;
 };
 
 class WASAPIInit : public DeferInit {
@@ -170,10 +149,6 @@ bool WASAPIOutputRegistrar::canMuteOthers() const {
 	return true;
 }
 
-bool WASAPIOutputRegistrar::usesOutputDelay() const {
-	return false;
-}
-
 const QHash<QString, QString> WASAPISystem::getInputDevices() {
 	return getDevices(eCapture);
 }
@@ -317,6 +292,19 @@ void WASAPIInput::run() {
 			qWarning("WASAPIInput: Failed to open input device");
 			goto cleanup;
 		}
+		wchar_t *devname = NULL;
+		hr = pMicDevice->GetId(&devname);
+		if (FAILED(hr)) {
+			qWarning("WASAPIInput: Failed to query input device");
+			goto cleanup;
+		}
+		pMicDevice->Release();
+		hr = pEnumerator->GetDevice(devname, &pMicDevice);
+		if (FAILED(hr)) {
+			qWarning("WASAPIInput: Failed to reopen default input device");
+			goto cleanup;
+		}
+		CoTaskMemFree(devname);
 	}
 
 	if (g.s.doEcho()) {
@@ -335,6 +323,19 @@ void WASAPIInput::run() {
 				qWarning("WASAPIInput: Failed to open echo device");
 				goto cleanup;
 			}
+			wchar_t *devname = NULL;
+			hr = pEchoDevice->GetId(&devname);
+			if (FAILED(hr)) {
+				qWarning("WASAPIInput: Failed to query echo device");
+				goto cleanup;
+			}
+			pEchoDevice->Release();
+			hr = pEnumerator->GetDevice(devname, &pEchoDevice);
+			if (FAILED(hr)) {
+				qWarning("WASAPIInput: Failed to reopen default echo device");
+				goto cleanup;
+			}
+			CoTaskMemFree(devname);
 		}
 	}
 
@@ -530,13 +531,25 @@ WASAPIOutput::~WASAPIOutput() {
 }
 
 void WASAPIOutput::setVolumes(IMMDevice *pDevice, bool talking) {
-	IAudioSessionManager *pAudioSessionManager = NULL;
-	IAudioSessionQuery *pMysticQuery = NULL;
-	IAudioQuerier *pAudioQuerier = NULL;
-	DWORD max = 0;
+	IAudioSessionManager2 *pAudioSessionManager = NULL;
+	int max = 0;
 	HRESULT hr;
 	DWORD dwMumble = GetCurrentProcessId();
 
+	static int version = -1;
+	if (version == -1) {
+		OSVERSIONINFOEXW ovi;
+		memset(&ovi, 0, sizeof(ovi));
+
+		ovi.dwOSVersionInfoSize=sizeof(ovi);
+		GetVersionEx(reinterpret_cast<OSVERSIONINFOW *>(&ovi));
+
+		if ((ovi.dwMajorVersion <= 6) && (ovi.dwBuildNumber < 7100))
+			version = 0;
+		else
+			version = 1;
+	}
+
 	if (talking) {
 		qmVolumes.clear();
 		if (qFuzzyCompare(g.s.fOtherVolume, 1.0f))
@@ -545,55 +558,73 @@ void WASAPIOutput::setVolumes(IMMDevice *pDevice, bool talking) {
 		return;
 	}
 
-	;
-	if (SUCCEEDED(hr = pDevice->Activate(__uuidof(IAudioSessionManager), CLSCTX_ALL, NULL, (void **) &pAudioSessionManager))) {
-		if (SUCCEEDED(hr = pAudioSessionManager->QueryInterface(__uuidof(IAudioSessionQuery), (void **) &pMysticQuery))) {
-			if (SUCCEEDED(hr = pMysticQuery->GetQueryInterface(&pAudioQuerier))) {
-				if (SUCCEEDED(hr = pAudioQuerier->GetNumSessions(&max))) {
-					for (int i=0;i<max;++i) {
-						IUnknown *pUnknown = NULL;
-						if (SUCCEEDED(hr = pAudioQuerier->QuerySession(i, &pUnknown))) {
-							IRemoteAudioSession *prem = NULL;
-							if (SUCCEEDED(hr = pUnknown->QueryInterface(__uuidof(IRemoteAudioSession), (void **) &prem))) {
-								DWORD pid = 0;
-								if (SUCCEEDED(hr = prem->GetProcessId(&pid)) && pid && (pid != dwMumble)) {
-									ISimpleAudioVolume *isav = NULL;
-									if (SUCCEEDED(hr = prem->QueryInterface(__uuidof(ISimpleAudioVolume), (void **) &isav))) {
-										if (talking) {
-											BOOL bMute = TRUE;
-											if (SUCCEEDED(hr = isav->GetMute(&bMute)) && ! bMute) {
-												float fVolume = 1.0f;
-												if (SUCCEEDED(hr = isav->GetMasterVolume(&fVolume)) && ! qFuzzyCompare(fVolume,0.0f)) {
-													float fSetVolume = fVolume * g.s.fOtherVolume;
-													if (SUCCEEDED(hr = isav->SetMasterVolume(fSetVolume, NULL))) {
-														hr = isav->GetMasterVolume(&fSetVolume);
-														qmVolumes.insert(pid, VolumePair(fVolume,fSetVolume));
+
+	if (SUCCEEDED(hr = pDevice->Activate(version ? __uuidof(IAudioSessionManager2) : __uuidof(IAudioSessionManager), CLSCTX_ALL, NULL, (void **) &pAudioSessionManager))) {
+		IAudioSessionEnumerator *pEnumerator = NULL;
+		IAudioSessionQuery *pMysticQuery = NULL;
+		if (version == 0) {
+			if (SUCCEEDED(hr = pAudioSessionManager->QueryInterface(__uuidof(IAudioSessionQuery), (void **) &pMysticQuery))) {
+				hr = pMysticQuery->GetQueryInterface(&pEnumerator);
+			}
+		} else {
+			hr = pAudioSessionManager->GetSessionEnumerator(&pEnumerator);
+		}
+		if (SUCCEEDED(hr)) {
+			if (SUCCEEDED(hr = pEnumerator->GetCount(&max))) {
+				for (int i=0;i<max;++i) {
+					IAudioSessionControl *pControl = NULL;
+					if (SUCCEEDED(hr = pEnumerator->GetSession(i, &pControl))) {
+						IAudioSessionControl2 *pControl2 = NULL;
+						if (SUCCEEDED(hr = pControl->QueryInterface(version ? __uuidof(IAudioSessionControl2) : __uuidof(IVistaAudioSessionControl2), (void **) &pControl2)))  {
+							DWORD pid;
+							if (SUCCEEDED(hr = pControl2->GetProcessId(&pid)) && (pid != dwMumble)) {
+								AudioSessionState ass;
+								if (SUCCEEDED(hr = pControl2->GetState(&ass)) && (! talking || (ass != AudioSessionStateExpired))) {
+									GUID group;
+									if (SUCCEEDED(hr = pControl2->GetGroupingParam(&group))) {
+										QUuid quuid(group);
+										if (qmVolumes.contains(quuid) != talking) {
+											ISimpleAudioVolume *pVolume = NULL;
+											if (SUCCEEDED(hr = pControl2->QueryInterface(__uuidof(ISimpleAudioVolume), (void **) &pVolume))) {
+												if (talking) {
+													BOOL bMute = TRUE;
+													if (SUCCEEDED(hr = pVolume->GetMute(&bMute)) && ! bMute) {
+														float fVolume = 1.0f;
+														if (!qmVolumes.contains(quuid) && SUCCEEDED(hr = pVolume->GetMasterVolume(&fVolume)) && ! qFuzzyCompare(fVolume,0.0f)) {
+															float fSetVolume = fVolume * g.s.fOtherVolume;
+															if (SUCCEEDED(hr = pVolume->SetMasterVolume(fSetVolume, NULL))) {
+																hr = pVolume->GetMasterVolume(&fSetVolume);
+																qmVolumes.insert(quuid, VolumePair(fVolume,fSetVolume));
+															}
+														}
+													}
+												} else {
+													VolumePair vp = qmVolumes.value(quuid);
+													if (vp.first) {
+														float fVolume = 1.0f;
+														hr = pVolume->GetMasterVolume(&fVolume);
+														if (qFuzzyCompare(fVolume, vp.second)) {
+															pVolume->SetMasterVolume(vp.first, NULL);
+														}
+														qmVolumes.remove(quuid);
 													}
 												}
-											}
-										} else {
-											VolumePair vp = qmVolumes.value(pid);
-											if (vp.first) {
-												float fVolume = 1.0f;
-												hr = isav->GetMasterVolume(&fVolume);
-												if (qFuzzyCompare(fVolume, vp.second)) {
-													isav->SetMasterVolume(vp.first, NULL);
-												}
+												pVolume->Release();
 											}
 										}
-										isav->Release();
 									}
 								}
-								prem->Release();
 							}
-							pUnknown->Release();
+							pControl2->Release();
 						}
+						pControl->Release();
 					}
 				}
-				pAudioQuerier->Release();
 			}
-			pMysticQuery->Release();
+			pEnumerator->Release();
 		}
+		if (pMysticQuery)
+			pMysticQuery->Release();
 		pAudioSessionManager->Release();
 	}
 }
@@ -619,6 +650,7 @@ void WASAPIOutput::run() {
 	unsigned int chanmasks[32];
 	QMap<DWORD, float> qmVolumes;
 	bool lastspoke = false;
+	REFERENCE_TIME bufferDuration = (g.s.iOutputDelay > 1) ? (g.s.iOutputDelay + 1) * 100000 : 0;
 
 	CoInitialize(NULL);
 
@@ -652,6 +684,19 @@ void WASAPIOutput::run() {
 			qWarning("WASAPIOutput: Failed to open output device");
 			goto cleanup;
 		}
+		wchar_t *devname = NULL;
+		hr = pDevice->GetId(&devname);
+		if (FAILED(hr)) {
+			qWarning("WASAPIOutput: Failed to query output device");
+			goto cleanup;
+		}
+		pDevice->Release();
+		hr = pEnumerator->GetDevice(devname, &pDevice);
+		if (FAILED(hr)) {
+			qWarning("WASAPIOutput: Failed to reopen default output device");
+			goto cleanup;
+		}
+		CoTaskMemFree(devname);
 	}
 
 	pEnumerator->Release();
@@ -673,7 +718,7 @@ void WASAPIOutput::run() {
 		goto cleanup;
 	}
 
-	hr = pAudioClient->Initialize(AUDCLNT_SHAREMODE_SHARED, AUDCLNT_STREAMFLAGS_EVENTCALLBACK, 0, 0, pwfx, NULL);
+	hr = pAudioClient->Initialize(AUDCLNT_SHAREMODE_SHARED, AUDCLNT_STREAMFLAGS_EVENTCALLBACK, bufferDuration, 0, pwfx, NULL);
 	if (FAILED(hr)) {
 		qWarning("WASAPIOutput: Initialize failed");
 		goto cleanup;
@@ -684,7 +729,7 @@ void WASAPIOutput::run() {
 	pAudioClient->GetStreamLatency(&latency);
 
 	qWarning("WASAPIOutput: Periods %lldus %lldus (latency %lldus)", def / 10LL, min / 10LL, latency / 10LL);
-	qWarning("WASAPIOutput: Buffer is %dus", (bufferFrameCount * 1000000) / iMixerFreq);
+	qWarning("WASAPIOutput: Buffer is %dus (%d)", (bufferFrameCount * 1000000) / iMixerFreq, g.s.iOutputDelay);
 
 	hr = pAudioClient->GetService(__uuidof(IAudioRenderClient), (void**)&pRenderClient);
 	if (FAILED(hr)) {
@@ -725,16 +770,16 @@ void WASAPIOutput::run() {
 
 		packetLength = bufferFrameCount - numFramesAvailable;
 
-		while (packetLength >= wantLength) {
-			hr = pRenderClient->GetBuffer(wantLength, &pData);
+		while (packetLength > 0) {
+			hr = pRenderClient->GetBuffer(packetLength, &pData);
 			if (FAILED(hr))
 				goto cleanup;
 
-			bool mixed = mix(reinterpret_cast<float *>(pData), wantLength);
+			bool mixed = mix(reinterpret_cast<float *>(pData), packetLength);
 			if (mixed)
-				hr = pRenderClient->ReleaseBuffer(wantLength, 0);
+				hr = pRenderClient->ReleaseBuffer(packetLength, 0);
 			else
-				hr = pRenderClient->ReleaseBuffer(wantLength, AUDCLNT_BUFFERFLAGS_SILENT);
+				hr = pRenderClient->ReleaseBuffer(packetLength, AUDCLNT_BUFFERFLAGS_SILENT);
 			if (FAILED(hr))
 				goto cleanup;
 
