@@ -192,7 +192,7 @@ void MurmurDBus::idToNameSlot(QString &name, int id) {
 	if (reply.isValid())
 		name = reply.value();
 	else {
-		server->log(QString("DBus Authenticator failed getUserName for %1: %2").arg(id).arg(reply.error().message()));
+		server->log(QString("DBus Authenticator failed getUserName for %1: %2").arg(QString::number(id), QString(reply.error().message())));
 		removeAuthenticator();
 	}
 }
@@ -216,61 +216,65 @@ void MurmurDBus::nameToIdSlot(int &id, const QString &name) {
 	}
 }
 
-void MurmurDBus::registerPlayerSlot(int &res, const QString &name) {
+void MurmurDBus::registerUserSlot(int &res, const QMap<QString, QString> &info) {
 	QDBusInterface remoteApp(qsAuthService,qsAuthPath,QString(),qdbc);
-	QDBusReply<int> reply = remoteApp.call(bReentrant ? QDBus::BlockWithGui : QDBus::Block, "registerPlayer", name);
-	if (reply.isValid())
+	QDBusReply<int> reply = remoteApp.call(bReentrant ? QDBus::BlockWithGui : QDBus::Block, "registerPlayer", info.value("name"));
+	if (reply.isValid()) {
 		res = reply.value();
+		if ((info.count() > 1) && (res > 0))
+			setInfoSlot(res, res, info);
+	}
 }
 
-void MurmurDBus::unregisterPlayerSlot(int &res, int id) {
+void MurmurDBus::unregisterUserSlot(int &res, int id) {
 	QDBusInterface remoteApp(qsAuthService,qsAuthPath,QString(),qdbc);
 	QDBusReply<int> reply = remoteApp.call(bReentrant ? QDBus::BlockWithGui : QDBus::Block, "unregisterPlayer", id);
 	if (reply.isValid())
 		res = reply.value();
 }
 
-void MurmurDBus::getRegistrationSlot(int &res, int id, QString &name, QString &email) {
+void MurmurDBus::getRegistrationSlot(int &res, int id, QMap<QString, QString> &info) {
 	QDBusInterface remoteApp(qsAuthService,qsAuthPath,QString(),qdbc);
 	QDBusReply<RegisteredPlayer> reply = remoteApp.call(bReentrant ? QDBus::BlockWithGui : QDBus::Block, "getRegistration", id);
 	if (reply.isValid()) {
 		const RegisteredPlayer &r = reply.value();
-		name = r.name;
-		email = r.email;
+		info.insert("name", r.name);
+		if (! r.email.isEmpty())
+			info.insert("email", r.email);
 		res = 1;
 	}
 }
 
-void  MurmurDBus::getRegisteredPlayersSlot(const QString &filter, QMap<int, QPair<QString, QString> > &m) {
+void  MurmurDBus::getRegisteredPlayersSlot(const QString &filter, QMap<int, QString > &m) {
 	QDBusInterface remoteApp(qsAuthService,qsAuthPath,QString(),qdbc);
 	QDBusReply<QList<RegisteredPlayer> > reply = remoteApp.call(bReentrant ? QDBus::BlockWithGui : QDBus::Block, "getRegisteredPlayers", filter);
 	if (reply.isValid()) {
 		const QList<RegisteredPlayer> &r = reply.value();
 		foreach(const RegisteredPlayer &p, r) {
-			m.insert(p.id, QPair<QString, QString>(p.name, p.email));
+			m.insert(p.id, p.name);
 		}
 	}
 }
 
-void MurmurDBus::setNameSlot(int &res, int id, const QString &name) {
-	QDBusInterface remoteApp(qsAuthService,qsAuthPath,QString(),qdbc);
-	QDBusReply<int> reply = remoteApp.call(bReentrant ? QDBus::BlockWithGui : QDBus::Block, "setName",id, name);
-	if (reply.isValid())
-		res = reply.value();
-}
-
-void MurmurDBus::setEmailSlot(int &res, int id, const QString &email) {
-	QDBusInterface remoteApp(qsAuthService,qsAuthPath,QString(),qdbc);
-	QDBusReply<int> reply = remoteApp.call(bReentrant ? QDBus::BlockWithGui : QDBus::Block, "setEmail",id, email);
-	if (reply.isValid())
-		res = reply.value();
-}
-
-void MurmurDBus::setPwSlot(int &res, int id, const QString &pw) {
-	QDBusInterface remoteApp(qsAuthService,qsAuthPath,QString(),qdbc);
-	QDBusReply<int> reply = remoteApp.call(bReentrant ? QDBus::BlockWithGui : QDBus::Block, "setPW",id, pw);
-	if (reply.isValid())
-		res = reply.value();
+void MurmurDBus::setInfoSlot(int &res, int id, const QMap<QString, QString> &info) {
+	if (info.contains("name")) {
+		QDBusInterface remoteApp(qsAuthService,qsAuthPath,QString(),qdbc);
+		QDBusReply<int> reply = remoteApp.call(bReentrant ? QDBus::BlockWithGui : QDBus::Block, "setName",id, info.value("name"));
+		if (reply.isValid())
+			res = reply.value();
+	}
+	if (info.contains("pw")) {
+		QDBusInterface remoteApp(qsAuthService,qsAuthPath,QString(),qdbc);
+		QDBusReply<int> reply = remoteApp.call(bReentrant ? QDBus::BlockWithGui : QDBus::Block, "setPW",id, info.value("pw"));
+		if (reply.isValid())
+			res = reply.value();
+	}
+	if (info.contains("email")) {
+		QDBusInterface remoteApp(qsAuthService,qsAuthPath,QString(),qdbc);
+		QDBusReply<int> reply = remoteApp.call(bReentrant ? QDBus::BlockWithGui : QDBus::Block, "setEmail",id, info.value("email"));
+		if (reply.isValid())
+			res = reply.value();
+	}
 }
 
 void MurmurDBus::setTextureSlot(int &res, int id, const QByteArray &texture) {
@@ -313,8 +317,8 @@ void MurmurDBus::authenticateSlot(int &res, QString &uname, const QString &pw) {
 }
 
 #define PLAYER_SETUP_VAR(var) \
-  User *pPlayer = server->qhUsers.value(var); \
-  if (! pPlayer) { \
+  ServerUser *pUser = server->qhUsers.value(var); \
+  if (! pUser) { \
     qdbc.send(msg.createErrorReply("net.sourceforge.mumble.Error.session", "Invalid session id")); \
     return; \
   }
@@ -332,8 +336,8 @@ void MurmurDBus::authenticateSlot(int &res, QString &uname, const QString &pw) {
 
 void MurmurDBus::getPlayers(QList<PlayerInfoExtended> &a) {
 	a.clear();
-	foreach(Player *p, server->qhUsers) {
-		if (p->sState == Player::Authenticated)
+	foreach(User *p, server->qhUsers) {
+		if (p->sState == User::Authenticated)
 			a << PlayerInfoExtended(p);
 	}
 }
@@ -356,72 +360,30 @@ void MurmurDBus::kickPlayer(unsigned int session, const QString &reason, const Q
 	if (!c)
 		return;
 
-	MessagePlayerKick mpk;
-	mpk.uiSession = 0;
-	mpk.uiVictim = session;
-	mpk.qsReason=reason;
-	server->sendAll(&mpk);
+	MumbleProto::UserRemove mpur;
+	mpur.set_session(session);
+	mpur.set_reason(u8(reason));
+	server->sendAll(mpur);
 	c->disconnectSocket();
 }
 
 void MurmurDBus::getPlayerState(unsigned int session, const QDBusMessage &msg, PlayerInfo &pi) {
 	PLAYER_SETUP;
-	pi = PlayerInfo(pPlayer);
+	pi = PlayerInfo(pUser);
 }
 
 void MurmurDBus::setPlayerState(const PlayerInfo &npi, const QDBusMessage &msg) {
 	PLAYER_SETUP_VAR(npi.session);
 	CHANNEL_SETUP_VAR(npi.channel);
-	PlayerInfo pi(pPlayer);
+	PlayerInfo pi(pUser);
 
-	bool changed = false;
-
-	bool deaf = npi.deaf;
-	bool mute = npi.mute;
-	if (deaf)
-		mute = true;
-	if (! mute)
-		deaf = false;
-
-	if ((pi.deaf != deaf) && (deaf || (!deaf && mute))) {
-		pPlayer->bDeaf = deaf;
-		pPlayer->bMute = mute;
-		MessagePlayerDeaf mpd;
-		mpd.uiSession = 0;
-		mpd.uiVictim=pPlayer->uiSession;
-		mpd.bDeaf = deaf;
-		server->sendAll(&mpd);
-		changed = true;
-	} else if ((pi.deaf != deaf) || (pi.mute != mute)) {
-		pPlayer->bDeaf = deaf;
-		pPlayer->bMute = mute;
-
-		MessagePlayerMute mpm;
-		mpm.uiSession = 0;
-		mpm.uiVictim=pPlayer->uiSession;
-		mpm.bMute=mute;
-		server->sendAll(&mpm);
-		changed = true;
-	}
-
-	if (cChannel->iId != pi.channel) {
-		server->playerEnterChannel(pPlayer, cChannel);
-		MessagePlayerMove mpm;
-		mpm.uiSession = 0;
-		mpm.uiVictim = pPlayer->uiSession;
-		mpm.iChannelId = cChannel->iId;
-		server->sendAll(&mpm);
-		changed = true;
-	}
-
-	if (changed)
-		emit playerStateChanged(PlayerInfo(pPlayer));
+	server->setUserState(pUser, cChannel, npi.mute, npi.deaf, npi.suppressed);
 }
 
 void MurmurDBus::sendMessage(unsigned int session, const QString &text, const QDBusMessage &msg) {
 	PLAYER_SETUP;
 
-	server->sendTextMessage(NULL, pPlayer, false, text);
+	server->sendTextMessage(NULL, pUser, false, text);
 }
 
 void MurmurDBus::sendMessageChannel(int id, bool tree, const QString &text, const QDBusMessage &msg) {
@@ -433,16 +395,15 @@ void MurmurDBus::sendMessageChannel(int id, bool tree, const QString &text, cons
 void MurmurDBus::addChannel(const QString &name, int chanparent, const QDBusMessage &msg, int &newid) {
 	CHANNEL_SETUP_VAR(chanparent);
 
-	Channel *nc = server->addChannel(cChannel, name, QString());
+	Channel *nc = server->addChannel(cChannel, name);
 	server->updateChannel(nc);
 	newid = nc->iId;
 
-	MessageChannelAdd mca;
-	mca.uiSession = 0;
-	mca.qsName = name;
-	mca.iParent = chanparent;
-	mca.iId = nc->iId;
-	server->sendAll(&mca);
+	MumbleProto::ChannelState mpcs;
+	mpcs.set_channel_id(nc->iId);
+	mpcs.set_parent(cChannel->iId);
+	mpcs.set_name(u8(name));
+	server->sendAll(mpcs);
 }
 
 void MurmurDBus::removeChannel(int id, const QDBusMessage &msg) {
@@ -451,7 +412,7 @@ void MurmurDBus::removeChannel(int id, const QDBusMessage &msg) {
 		qdbc.send(msg.createErrorReply("net.sourceforge.mumble.Error.channel", "Invalid channel id"));
 		return;
 	}
-	server->removeChannel(cChannel, NULL);
+	server->removeChannel(cChannel);
 }
 
 void MurmurDBus::getChannelState(int id, const QDBusMessage &msg, ChannelInfo &state) {
@@ -461,93 +422,18 @@ void MurmurDBus::getChannelState(int id, const QDBusMessage &msg, ChannelInfo &s
 
 void MurmurDBus::setChannelState(const ChannelInfo &nci, const QDBusMessage &msg) {
 	CHANNEL_SETUP_VAR(nci.id);
-	ChannelInfo ci(cChannel);
-
 	CHANNEL_SETUP_VAR2(cParent, nci.parent);
 
-	bool changed = false;
-	bool update = false;
-
 	QSet<Channel *> newset;
 	foreach(int id, nci.links) {
 		CHANNEL_SETUP_VAR2(cLink, id);
 		newset << cLink;
 	}
 
-	if (cChannel->qsName != nci.name) {
-		cChannel->qsName = nci.name;
-		MessageChannelRename mcr;
-		mcr.uiSession = 0;
-		mcr.iId = cChannel->iId;
-		mcr.qsName = nci.name;
-		server->sendAll(&mcr);
-		update = true;
-		changed = true;
+	if (! server->setChannelState(cChannel, cParent, nci.name, newset)) {
+		qdbc.send(msg.createErrorReply("net.sourceforge.mumble.Error.channel", "Moving channel to subchannel"));
+		return;
 	}
-
-	if ((cParent != cChannel) && (cParent != cChannel->cParent)) {
-
-		Channel *p = cParent;
-		while (p) {
-			if (p == cChannel) {
-				qdbc.send(msg.createErrorReply("net.sourceforge.mumble.Error.channel", "Moving channel to subchannel"));
-				return;
-			}
-			p = p->cParent;
-		}
-
-		cChannel->cParent->removeChannel(cChannel);
-		cParent->addChannel(cChannel);
-
-		MessageChannelMove mcm;
-		mcm.uiSession = 0;
-		mcm.iId = nci.id;
-		mcm.iParent = nci.parent;
-		server->sendAll(&mcm);
-
-		update = true;
-		changed = true;
-	}
-
-	QSet<Channel *> oldset = cChannel->qsPermLinks;
-
-	if (newset != oldset) {
-		// Remove
-
-		foreach(Channel *l, oldset) {
-			if (! newset.contains(l)) {
-				server->removeLink(cChannel, l);
-
-				MessageChannelLink mcl;
-				mcl.uiSession = 0;
-				mcl.iId = nci.id;
-				mcl.qlTargets << l->iId;
-				mcl.ltType = MessageChannelLink::Unlink;
-				server->sendAll(&mcl);
-			}
-		}
-
-		// Add
-		foreach(Channel *l, newset) {
-			if (! oldset.contains(l)) {
-				server->addLink(cChannel, l);
-
-				MessageChannelLink mcl;
-				mcl.uiSession = 0;
-				mcl.iId = nci.id;
-				mcl.qlTargets << l->iId;
-				mcl.ltType = MessageChannelLink::Link;
-				server->sendAll(&mcl);
-			}
-		}
-
-		changed = true;
-	}
-
-	if (update)
-		server->updateChannel(cChannel);
-	if (changed)
-		emit channelStateChanged(ChannelInfo(cChannel));
 }
 
 void MurmurDBus::getACL(int id, const QDBusMessage &msg, QList<ACLInfo> &acls, QList<GroupInfo> &groups, bool &inherit) {
@@ -640,10 +526,10 @@ void MurmurDBus::setACL(int id, const QList<ACLInfo> &acls, const QList<GroupInf
 		a = new ChanACL(cChannel);
 		a->bApplyHere = ai.applyHere;
 		a->bApplySubs = ai.applySubs;
-		a->iPlayerId = ai.playerid;
+		a->iUserId = ai.playerid;
 		a->qsGroup = ai.group;
-		a->pDeny = static_cast<ChanACL::Permissions>(ai.deny);
-		a->pAllow = static_cast<ChanACL::Permissions>(ai.allow);
+		a->pDeny = static_cast<ChanACL::Permissions>(ai.deny) & ChanACL::All;
+		a->pAllow = static_cast<ChanACL::Permissions>(ai.allow) & ChanACL::All;
 	}
 
 	server->clearACLCache();
@@ -652,47 +538,33 @@ void MurmurDBus::setACL(int id, const QList<ACLInfo> &acls, const QList<GroupInf
 
 void MurmurDBus::getBans(QList<BanInfo> &bi) {
 	bi.clear();
-	QPair<quint32,int> ban;
-	foreach(ban, server->qlBans) {
-		bi << BanInfo(ban);
+	foreach(const Ban &b, server->qlBans) {
+		if (! b.haAddress.isV6())
+			bi << BanInfo(b);
 	}
 }
 
-void MurmurDBus::setBans(const QList<BanInfo> &bans, const QDBusMessage &) {
-	server->qlBans.clear();
-	foreach(BanInfo bi, bans)
-		server->qlBans << QPair<quint32,int>(bi.address,bi.bits);
-	server->saveBans();
+void MurmurDBus::setBans(const QList<BanInfo> &, const QDBusMessage &) {
 }
 
 void MurmurDBus::getPlayerNames(const QList<int> &ids, const QDBusMessage &, QStringList &names) {
 	names.clear();
 	foreach(int id, ids) {
-		if (! server->qhUserNameCache.contains(id)) {
-			QString name=server->getUserName(id);
-			if (! name.isEmpty()) {
-				server->qhUserNameCache[id]=name;
-			}
-		}
-		names << server->qhUserNameCache.value(id);
+		names << server->getUserName(id);
 	}
 }
 
 void MurmurDBus::getPlayerIds(const QStringList &names, const QDBusMessage &, QList<int> &ids) {
 	ids.clear();
 	foreach(QString name, names) {
-		if (! server->qhUserIDCache.contains(name)) {
-			int id=server->getUserID(name);
-			if (id != -1) {
-				server->qhUserIDCache[name]=id;
-			}
-		}
-		ids << server->qhUserIDCache.value(name);
+		ids << server->getUserID(name);
 	}
 }
 
 void MurmurDBus::registerPlayer(const QString &name, const QDBusMessage &msg, int &id) {
-	id = server->registerPlayer(name);
+	QMap<QString, QString> info;
+	info.insert("name", name);
+	id = server->registerUser(info);
 	if (id < 0) {
 		qdbc.send(msg.createErrorReply("net.sourceforge.mumble.Error.playername", "Illegal player name"));
 		return;
@@ -700,58 +572,53 @@ void MurmurDBus::registerPlayer(const QString &name, const QDBusMessage &msg, in
 }
 
 void MurmurDBus::unregisterPlayer(int id, const QDBusMessage &msg) {
-	if (! server->unregisterPlayer(id)) {
+	if (! server->unregisterUser(id)) {
 		qdbc.send(msg.createErrorReply("net.sourceforge.mumble.Error.playerid", "Invalid player id"));
 		return;
 	}
 }
 
-void MurmurDBus::getRegistration(int id, const QDBusMessage &msg, RegisteredPlayer &player) {
-	QString name, email;
-	if (! server->getRegistration(id, name, email)) {
+void MurmurDBus::getRegistration(int id, const QDBusMessage &msg, RegisteredPlayer &user) {
+	QMap<QString, QString> info = server->getRegistration(id);
+	if (info.isEmpty()) {
 		qdbc.send(msg.createErrorReply("net.sourceforge.mumble.Error.playerid", "Invalid player id"));
 		return;
 	}
 
-	player.id = id;
-	player.name = name;
-	player.email = email;
+	user.id = id;
+	user.name = info.value("name");
+	user.email = info.value("email");
 }
 
 void MurmurDBus::setRegistration(int id, const QString &name, const QString &email, const QString &pw, const QDBusMessage &msg) {
-	RegisteredPlayer player;
-	player.id = id;
-	player.name = name;
-	player.email = email;
-	player.pw = pw;
-	updateRegistration(player, msg);
+	RegisteredPlayer user;
+	user.id = id;
+	user.name = name;
+	user.email = email;
+	user.pw = pw;
+	updateRegistration(user, msg);
 }
 
-void MurmurDBus::updateRegistration(const RegisteredPlayer &player, const QDBusMessage &msg) {
-	QString name, email;
-	if (! server->getRegistration(player.id, name, email)) {
+void MurmurDBus::updateRegistration(const RegisteredPlayer &user, const QDBusMessage &msg) {
+	QMap<QString, QString> info;
+
+	if (! user.name.isEmpty())
+		info.insert("name", user.name);
+
+	if (! user.email.isEmpty())
+		info.insert("email", user.email);
+
+	if (! user.pw.isEmpty())
+		info.insert("pw", user.pw);
+
+	if (info.isEmpty() || !server->setInfo(user.id, info)) {
 		qdbc.send(msg.createErrorReply("net.sourceforge.mumble.Error.playerid", "Invalid player id"));
 		return;
 	}
-
-	if ((! player.name.isEmpty()) && (name != player.name)) {
-		if (! server->setName(player.id, player.name)) {
-			qdbc.send(msg.createErrorReply("net.sourceforge.mumble.Error.playername", "Illegal player name"));
-			return;
-		}
-	}
-
-	if ((! player.email.isEmpty()) && (email != player.email)) {
-		server->setEmail(player.id, player.email);
-	}
-
-	if (! player.pw.isEmpty()) {
-		server->setPW(player.id, player.pw);
-	}
 }
 
 void MurmurDBus::getTexture(int id, const QDBusMessage &msg, QByteArray &texture) {
-	if (! server->isPlayerId(id)) {
+	if (! server->isUserId(id)) {
 		qdbc.send(msg.createErrorReply("net.sourceforge.mumble.Error.playerid", "Invalid player id"));
 		return;
 	}
@@ -759,7 +626,7 @@ void MurmurDBus::getTexture(int id, const QDBusMessage &msg, QByteArray &texture
 }
 
 void MurmurDBus::setTexture(int id, const QByteArray &texture, const QDBusMessage &msg) {
-	if (! server->isPlayerId(id)) {
+	if (! server->isUserId(id)) {
 		qdbc.send(msg.createErrorReply("net.sourceforge.mumble.Error.playerid", "Invalid player id"));
 		return;
 	}
@@ -769,16 +636,15 @@ void MurmurDBus::setTexture(int id, const QByteArray &texture, const QDBusMessag
 	}
 }
 
-void MurmurDBus::getRegisteredPlayers(const QString &filter, QList<RegisteredPlayer> &players) {
-	players.clear();
-	QMap<int, QPair<QString, QString> > l = server->getRegisteredPlayers(filter);
-	QMap<int, QPair<QString, QString> >::const_iterator i;
+void MurmurDBus::getRegisteredPlayers(const QString &filter, QList<RegisteredPlayer> &users) {
+	users.clear();
+	QMap<int, QString > l = server->getRegisteredUsers(filter);
+	QMap<int, QString >::const_iterator i;
 	for (i = l.constBegin(); i != l.constEnd(); ++i) {
 		RegisteredPlayer r;
 		r.id = i.key();
-		r.name = i.value().first;
-		r.email = i.value().second;
-		players << r;
+		r.name = i.value();
+		users << r;
 	}
 }
 
@@ -808,30 +674,30 @@ void MurmurDBus::setAuthenticator(const QDBusObjectPath &path, bool reentrant, c
 	qsAuthPath = path.path();
 	qsAuthService = msg.service();
 	bReentrant = reentrant;
-	server->log(QString("DBus Authenticator set to %1 %2 (%3)").arg(qsAuthService).arg(qsAuthPath).arg(reentrant));
+	server->log(QString("DBus Authenticator set to %1 %2 (%3)").arg(qsAuthService, qsAuthPath, QString::number(reentrant)));
 }
 
-void MurmurDBus::setTemporaryGroups(int channel, int playerid, const QStringList &groups, const QDBusMessage &msg) {
+void MurmurDBus::setTemporaryGroups(int channel, int userid, const QStringList &groups, const QDBusMessage &msg) {
 	CHANNEL_SETUP_VAR(channel);
 
-	server->setTempGroups(playerid, cChannel, groups);
+	server->setTempGroups(userid, cChannel, groups);
 }
 
-PlayerInfo::PlayerInfo(const Player *p) {
+PlayerInfo::PlayerInfo(const User *p) {
 	session = p->uiSession;
 	mute = p->bMute;
 	deaf = p->bDeaf;
-	suppressed = p->bSuppressed;
+	suppressed = p->bSuppress;
 	selfMute = p->bSelfMute;
 	selfDeaf = p->bSelfDeaf;
 	channel = p->cChannel->iId;
 }
 
-PlayerInfoExtended::PlayerInfoExtended(const Player *p) : PlayerInfo(p) {
+PlayerInfoExtended::PlayerInfoExtended(const User *p) : PlayerInfo(p) {
 	id = p->iId;
 	name = p->qsName;
 
-	const User *u = static_cast<const User *>(p);
+	const ServerUser *u = static_cast<const ServerUser *>(p);
 	onlinesecs = u->bwr.onlineSeconds();
 	bytespersec = u->bwr.bandwidth();
 }
@@ -848,7 +714,7 @@ ACLInfo::ACLInfo(const ChanACL *acl) {
 	applyHere = acl->bApplyHere;
 	applySubs = acl->bApplySubs;
 	inherited = false;
-	playerid = acl->iPlayerId;
+	playerid = acl->iUserId;
 	group = acl->qsGroup;
 	allow = acl->pAllow;
 	deny = acl->pDeny;
@@ -861,9 +727,9 @@ GroupInfo::GroupInfo(const Group *g) {
 	members.clear();
 }
 
-BanInfo::BanInfo(QPair<quint32,int> b) {
-	address = b.first;
-	bits = b.second;
+BanInfo::BanInfo(const Ban &b) {
+	address = ntohl(b.haAddress.hash[3]);
+	bits = b.iMask;
 }
 
 RegisteredPlayer::RegisteredPlayer() {
@@ -879,15 +745,15 @@ LogEntry::LogEntry(const ServerDB::LogRecord &r) {
 	txt = r.second;
 }
 
-void MurmurDBus::playerStateChanged(const Player *p) {
+void MurmurDBus::userStateChanged(const User *p) {
 	emit playerStateChanged(PlayerInfo(p));
 }
 
-void MurmurDBus::playerConnected(const Player *p) {
+void MurmurDBus::userConnected(const User *p) {
 	emit playerConnected(PlayerInfo(p));
 }
 
-void MurmurDBus::playerDisconnected(const Player *p) {
+void MurmurDBus::userDisconnected(const User *p) {
 	emit playerDisconnected(PlayerInfo(p));
 }
 
