@@ -34,7 +34,7 @@
 
 #include "murmur_pch.h"
 
-#include "Player.h"
+#include "User.h"
 #include "Channel.h"
 #include "Group.h"
 #include "ACL.h"
@@ -42,13 +42,15 @@
 #include "ServerDB.h"
 #include "Meta.h"
 
+struct Ban;
+
 struct PlayerInfo {
 	unsigned int session;
 	bool mute, deaf, suppressed;
 	bool selfMute, selfDeaf;
 	int channel;
 	PlayerInfo() { };
-	PlayerInfo(const Player *);
+	PlayerInfo(const User *);
 };
 Q_DECLARE_METATYPE(PlayerInfo);
 
@@ -58,7 +60,7 @@ struct PlayerInfoExtended : public PlayerInfo {
 	int onlinesecs;
 	int bytespersec;
 	PlayerInfoExtended() {};
-	PlayerInfoExtended(const Player *);
+	PlayerInfoExtended(const User *);
 };
 Q_DECLARE_METATYPE(PlayerInfoExtended);
 Q_DECLARE_METATYPE(QList<PlayerInfoExtended>);
@@ -99,7 +101,7 @@ struct BanInfo {
 	unsigned int address;
 	int bits;
 	BanInfo() { };
-	BanInfo(QPair<quint32,int>);
+	BanInfo(const Ban &);
 };
 Q_DECLARE_METATYPE(BanInfo);
 Q_DECLARE_METATYPE(QList<BanInfo>);
@@ -142,22 +144,20 @@ class MurmurDBus : public QDBusAbstractAdaptor {
 	public slots:
 		// These have the result ref as the first parameter, so won't be converted to DBus
 		void authenticateSlot(int &res, QString &uname, const QString &pw);
-		void registerPlayerSlot(int &res, const QString &name);
-		void unregisterPlayerSlot(int &res, int id);
-		void getRegisteredPlayersSlot(const QString &filter, QMap<int, QPair<QString, QString> > &res);
-		void getRegistrationSlot(int &res, int id, QString &name, QString &email);
-		void setPwSlot(int &res, int id, const QString &pw);
-		void setEmailSlot(int &res, int id, const QString &email);
-		void setNameSlot(int &res, int id, const QString &name);
+		void registerUserSlot(int &res, const QMap<QString, QString> &);
+		void unregisterUserSlot(int &res, int id);
+		void getRegisteredPlayersSlot(const QString &filter, QMap<int, QString> &res);
+		void getRegistrationSlot(int &, int, QMap<QString, QString> &);
+		void setInfoSlot(int &, int, const QMap<QString, QString> &);
 		void setTextureSlot(int &res, int id, const QByteArray &texture);
 		void nameToIdSlot(int &res, const QString &name);
 		void idToNameSlot(QString &res, int id);
 		void idToTextureSlot(QByteArray &res, int id);
 
 		// These use private types, so won't be converted to DBus
-		void playerStateChanged(const Player *p);
-		void playerConnected(const Player *p);
-		void playerDisconnected(const Player *p);
+		void userStateChanged(const User *p);
+		void userConnected(const User *p);
+		void userDisconnected(const User *p);
 
 		void channelStateChanged(const Channel *c);
 		void channelCreated(const Channel *c);
