@@ -44,8 +44,10 @@ MetaParams::MetaParams() {
 	qsPassword = QString();
 	usPort = 64738;
 	iTimeout = 30;
-	iMaxBandwidth = 10000;
+	iMaxBandwidth = 72000;
 	iMaxUsers = 1000;
+	iMaxTextMessageLength = 5000;
+	bAllowHTML = true;
 	iDefaultChan = 0;
 	qsWelcomeText = QString("Welcome to this server");
 	qsDatabase = QString();
@@ -53,11 +55,14 @@ MetaParams::MetaParams() {
 	qsDBusService = "net.sourceforge.mumble.murmur";
 	qsDBDriver = "QSQLITE";
 	qsLogfile = "murmur.log";
-	qhaBind = QHostAddress(QHostAddress::Any);
 
 	iLogDays = 31;
 
 	iObfuscate = 0;
+	bSendVersion = true;
+	bBonjour = true;
+	bAllowPing = true;
+	bCertRequired = false;
 
 	iBanTries = 10;
 	iBanTimeframe = 120;
@@ -65,8 +70,15 @@ MetaParams::MetaParams() {
 
 	uiUid = uiGid = 0;
 
-	qrPlayerName = QRegExp(QLatin1String("[-=\\w\\[\\]\\{\\}\\(\\)\\@\\|\\.]+"));
+	qrUserName = QRegExp(QLatin1String("[-=\\w\\[\\]\\{\\}\\(\\)\\@\\|\\.]+"));
 	qrChannelName = QRegExp(QLatin1String("[ \\-=\\w\\#\\[\\]\\{\\}\\(\\)\\@\\|]+"));
+
+	qsSettings = NULL;
+}
+
+MetaParams::~MetaParams() {
+	if (qsSettings)
+		delete qsSettings;
 }
 
 void MetaParams::read(QString fname) {
@@ -74,6 +86,8 @@ void MetaParams::read(QString fname) {
 		QStringList datapaths;
 
 #if defined(Q_OS_WIN)
+		datapaths << QDesktopServices::storageLocation(QDesktopServices::DataLocation);
+
 		size_t reqSize;
 		_wgetenv_s(&reqSize, NULL, 0, L"APPDATA");
 		if (reqSize > 0) {
@@ -87,6 +101,7 @@ void MetaParams::read(QString fname) {
 #elif defined(Q_OS_MAC)
 		datapaths << QDir::homePath() + QLatin1String("/Library/Preferences/Mumble/");
 #else
+		datapaths << QDir::homePath() + QLatin1String("/.murmurd");
 		datapaths << QDir::homePath() + QLatin1String("/.config/Mumble");
 #endif
 		datapaths << QDir::homePath();
@@ -94,11 +109,13 @@ void MetaParams::read(QString fname) {
 		datapaths << QCoreApplication::instance()->applicationDirPath();
 
 		foreach(const QString &p, datapaths) {
-			QFileInfo fi(p, "murmur.ini");
-			if (fi.exists() && fi.isReadable()) {
-				qdBasePath = QDir(p);
-				fname = fi.absoluteFilePath();
-				break;
+			if (! p.isEmpty()) {
+				QFileInfo fi(p, "murmur.ini");
+				if (fi.exists() && fi.isReadable()) {
+					qdBasePath = QDir(p);
+					fname = fi.absoluteFilePath();
+					break;
+				}
 			}
 		}
 		if (fname.isEmpty()) {
@@ -115,66 +132,114 @@ void MetaParams::read(QString fname) {
 		f.close();
 	}
 	QDir::setCurrent(qdBasePath.absolutePath());
-	QSettings qs(fname, QSettings::IniFormat);
+	qsSettings = new QSettings(fname, QSettings::IniFormat);
+#if QT_VERSION >= 0x040500
+	qsSettings->setIniCodec("UTF-8");
+#endif
+	qWarning("Initializing settings from %s (basepath %s)", qPrintable(qsSettings->fileName()), qPrintable(qdBasePath.absolutePath()));
 
-	qWarning("Initializing settings from %s (basepath %s)", qPrintable(qs.fileName()), qPrintable(qdBasePath.absolutePath()));
-
-	QString qsHost = qs.value("host", QString()).toString();
+	QString qsHost = qsSettings->value("host", QString()).toString();
 	if (! qsHost.isEmpty()) {
-		if (! qhaBind.setAddress(qsHost)) {
-			QHostInfo hi = QHostInfo::fromName(qsHost);
-			foreach(QHostAddress qha, hi.addresses()) {
-				if (qha.protocol() == QAbstractSocket::IPv4Protocol) {
-					qhaBind = qha;
-					break;
+		foreach(const QString &host, qsHost.split(QRegExp(QLatin1String("\\s+")), QString::SkipEmptyParts)) {
+			QHostAddress qhaddr;
+			if (qhaddr.setAddress(host)) {
+				qlBind << qhaddr;
+			} else {
+				bool found = false;
+				QHostInfo hi = QHostInfo::fromName(host);
+				foreach(QHostAddress qha, hi.addresses()) {
+					if ((qha.protocol() == QAbstractSocket::IPv4Protocol) || (qha.protocol() == QAbstractSocket::IPv6Protocol)) {
+						qlBind << qha;
+						found = true;
+					}
+				}
+				if (! found) {
+					qFatal("Lookup of bind hostname %s failed", qPrintable(host));
 				}
 			}
-			if ((qhaBind == QHostAddress::Any) || (qhaBind.isNull())) {
-				qFatal("Lookup of bind hostname %s failed", qPrintable(qsHost));
-			}
-
 		}
-		qWarning("Binding to address %s", qPrintable(qhaBind.toString()));
+		foreach(const QHostAddress &qha, qlBind)
+			qWarning("Binding to address %s", qPrintable(qha.toString()));
 	}
 
-	qsPassword = qs.value("serverpassword", qsPassword).toString();
-	usPort = static_cast<unsigned short>(qs.value("port", usPort).toUInt());
-	iTimeout = qs.value("timeout", iTimeout).toInt();
-	iMaxBandwidth = qs.value("bandwidth", iMaxBandwidth).toInt();
-	iDefaultChan = qs.value("defaultchannel", iDefaultChan).toInt();
-	iMaxUsers = qs.value("users", iMaxUsers).toInt();
-	qsWelcomeText = qs.value("welcometext", qsWelcomeText).toString();
+	if (qlBind.isEmpty()) {
+		bool hasipv6 = false;
+		bool hasipv4 = false;
 
-	qsDatabase = qs.value("database", qsDatabase).toString();
+		foreach(const QNetworkInterface &qni, QNetworkInterface::allInterfaces()) {
+			if (!(qni.flags() & QNetworkInterface::IsUp))
+				continue;
+			if (!(qni.flags() & QNetworkInterface::IsRunning))
+				continue;
+			if (qni.flags() & QNetworkInterface::IsLoopBack)
+				continue;
 
-	qsDBDriver = qs.value("dbDriver", qsDBDriver).toString();
-	qsDBUserName = qs.value("dbUsername", qsDBUserName).toString();
-	qsDBPassword = qs.value("dbPassword", qsDBPassword).toString();
-	qsDBHostName = qs.value("dbHost", qsDBHostName).toString();
-	qsDBPrefix = qs.value("dbPrefix", qsDBPrefix).toString();
-	qsDBOpts = qs.value("dbOpts", qsDBOpts).toString();
-	iDBPort = qs.value("dbPort", iDBPort).toInt();
+			foreach(const QNetworkAddressEntry &qna, qni.addressEntries()) {
+				const QHostAddress &qha = qna.ip();
+				switch (qha.protocol()) {
+					case QAbstractSocket::IPv4Protocol:
+						hasipv4 = true;
+						break;
+					case QAbstractSocket::IPv6Protocol:
+						hasipv6 = true;
+						break;
+					default:
+						break;
+				}
+			}
+		}
 
-	qsIceEndpoint = qs.value("ice", qsIceEndpoint).toString();
+		if (hasipv6)
+			qlBind << QHostAddress(QHostAddress::AnyIPv6);
+#ifdef Q_OS_UNIX
+		else
+#endif
+			if (hasipv4)
+				qlBind << QHostAddress(QHostAddress::Any);
+	}
 
-	iLogDays = qs.value("logdays", iLogDays).toInt();
+	qsPassword = qsSettings->value("serverpassword", qsPassword).toString();
+	usPort = static_cast<unsigned short>(qsSettings->value("port", usPort).toUInt());
+	iTimeout = qsSettings->value("timeout", iTimeout).toInt();
+	iMaxTextMessageLength = qsSettings->value("textmessagelength", iMaxTextMessageLength).toInt();
+	bAllowHTML = qsSettings->value("allowhtml", bAllowHTML).toBool();
+	iMaxBandwidth = qsSettings->value("bandwidth", iMaxBandwidth).toInt();
+	iDefaultChan = qsSettings->value("defaultchannel", iDefaultChan).toInt();
+	iMaxUsers = qsSettings->value("users", iMaxUsers).toInt();
+	qsWelcomeText = qsSettings->value("welcometext", qsWelcomeText).toString();
+	bCertRequired = qsSettings->value("certrequired", bCertRequired).toBool();
 
-	qsDBus = qs.value("dbus", qsDBus).toString();
-	qsDBusService = qs.value("dbusservice", qsDBusService).toString();
-	qsLogfile = qs.value("logfile", qsLogfile).toString();
-	qsPid = qs.value("pidfile", qsPid).toString();
+	qsDatabase = qsSettings->value("database", qsDatabase).toString();
 
-	qsRegName = qs.value("registerName", qsRegName).toString();
-	qsRegPassword = qs.value("registerPassword", qsRegPassword).toString();
-	qsRegHost = qs.value("registerHostname", qsRegHost).toString();
-	qurlRegWeb = QUrl(qs.value("registerUrl", qurlRegWeb.toString()).toString());
+	qsDBDriver = qsSettings->value("dbDriver", qsDBDriver).toString();
+	qsDBUserName = qsSettings->value("dbUsername", qsDBUserName).toString();
+	qsDBPassword = qsSettings->value("dbPassword", qsDBPassword).toString();
+	qsDBHostName = qsSettings->value("dbHost", qsDBHostName).toString();
+	qsDBPrefix = qsSettings->value("dbPrefix", qsDBPrefix).toString();
+	qsDBOpts = qsSettings->value("dbOpts", qsDBOpts).toString();
+	iDBPort = qsSettings->value("dbPort", iDBPort).toInt();
 
-	iBanTries = qs.value("autobanAttempts", iBanTries).toInt();
-	iBanTimeframe = qs.value("autobanTimeframe", iBanTimeframe).toInt();
-	iBanTime = qs.value("autobanTime", iBanTime).toInt();
+	qsIceEndpoint = qsSettings->value("ice", qsIceEndpoint).toString();
+
+	iLogDays = qsSettings->value("logdays", iLogDays).toInt();
+
+	qsDBus = qsSettings->value("dbus", qsDBus).toString();
+	qsDBusService = qsSettings->value("dbusservice", qsDBusService).toString();
+	qsLogfile = qsSettings->value("logfile", qsLogfile).toString();
+	qsPid = qsSettings->value("pidfile", qsPid).toString();
+
+	qsRegName = qsSettings->value("registerName", qsRegName).toString();
+	qsRegPassword = qsSettings->value("registerPassword", qsRegPassword).toString();
+	qsRegHost = qsSettings->value("registerHostname", qsRegHost).toString();
+	qurlRegWeb = QUrl(qsSettings->value("registerUrl", qurlRegWeb.toString()).toString());
+	bBonjour = qsSettings->value("bonjour", bBonjour).toBool();
+
+	iBanTries = qsSettings->value("autobanAttempts", iBanTries).toInt();
+	iBanTimeframe = qsSettings->value("autobanTimeframe", iBanTimeframe).toInt();
+	iBanTime = qsSettings->value("autobanTime", iBanTime).toInt();
 
 #ifdef Q_OS_UNIX
-	const QString uname = qs.value("uname").toString();
+	const QString uname = qsSettings->value("uname").toString();
 	if (! uname.isEmpty() && (geteuid() == 0)) {
 		struct passwd *pw = getpwnam(qPrintable(uname));
 		if (pw) {
@@ -188,19 +253,38 @@ void MetaParams::read(QString fname) {
 	}
 #endif
 
-	qrPlayerName = QRegExp(qs.value("playername", qrPlayerName.pattern()).toString());
-	qrChannelName = QRegExp(qs.value("channelname", qrChannelName.pattern()).toString());
+	qrUserName = QRegExp(qsSettings->value("username", qrUserName.pattern()).toString());
+	qrChannelName = QRegExp(qsSettings->value("channelname", qrChannelName.pattern()).toString());
 
-	bool bObfuscate = qs.value("obfuscate", false).toBool();
+	bool bObfuscate = qsSettings->value("obfuscate", false).toBool();
 	if (bObfuscate) {
 		qWarning("IP address obfuscation enabled.");
 		iObfuscate = qrand();
 	}
+	bSendVersion = qsSettings->value("sendversion", bSendVersion).toBool();
+	bAllowPing = qsSettings->value("allowping", bAllowPing).toBool();
 
-	QString qsSSLCert = qs.value("sslCert").toString();
-	QString qsSSLKey = qs.value("sslKey").toString();
+	QString qsSSLCert = qsSettings->value("sslCert").toString();
+	QString qsSSLKey = qsSettings->value("sslKey").toString();
+	QString qsSSLCA = qsSettings->value("sslCA").toString();
 
-	qbaPassPhrase = qs.value("sslPassPhrase").toByteArray();
+	qbaPassPhrase = qsSettings->value("sslPassPhrase").toByteArray();
+
+	if (! qsSSLCA.isEmpty()) {
+		QFile pem(qsSSLCA);
+		if (pem.open(QIODevice::ReadOnly)) {
+			QByteArray qba = pem.readAll();
+			pem.close();
+			QList<QSslCertificate> ql = QSslCertificate::fromData(qba);
+			if (ql.isEmpty()) {
+				qCritical("Failed to parse any CA certificates from %s", qPrintable(qsSSLCA));
+			} else {
+				QSslSocket::addDefaultCaCertificates(ql);
+			}
+		} else {
+			qCritical("Failed to read %s", qPrintable(qsSSLCert));
+		}
+	}
 
 	QByteArray crt, key;
 
@@ -223,51 +307,66 @@ void MetaParams::read(QString fname) {
 		}
 	}
 
-	if (! crt.isEmpty()) {
-		qscCert = QSslCertificate(crt);
-		if (qscCert.isNull()) {
-			qCritical("Failed to parse certificate.");
-		}
-	}
-
-	if (! key.isEmpty() && qscCert.isNull()) {
-		qscCert = QSslCertificate(key);
-		if (! qscCert.isNull()) {
-			qWarning("Using certificate from key file.");
-		}
-	}
-
-	if (! qscCert.isNull()) {
-		QSsl::KeyAlgorithm alg = qscCert.publicKey().algorithm();
-
+	if (! key.isEmpty() || ! crt.isEmpty()) {
 		if (! key.isEmpty()) {
-			qskKey = QSslKey(key, alg, QSsl::Pem, QSsl::PrivateKey, qbaPassPhrase);
-			if (qskKey.isNull()) {
-				qCritical("Failed to parse key file.");
-			}
+			qskKey = QSslKey(key, QSsl::Rsa, QSsl::Pem, QSsl::PrivateKey, qbaPassPhrase);
+			if (qskKey.isNull())
+				qskKey = QSslKey(key, QSsl::Dsa, QSsl::Pem, QSsl::PrivateKey, qbaPassPhrase);
 		}
-
-		if (! crt.isEmpty() && qskKey.isNull()) {
-			qskKey = QSslKey(crt, alg, QSsl::Pem, QSsl::PrivateKey, qbaPassPhrase);
-			if (! qskKey.isNull()) {
-				qWarning("Using key from certificate file.");
-			}
+		if (qskKey.isNull() && ! crt.isEmpty()) {
+			qskKey = QSslKey(crt, QSsl::Rsa, QSsl::Pem, QSsl::PrivateKey, qbaPassPhrase);
+			if (qskKey.isNull())
+				qskKey = QSslKey(crt, QSsl::Dsa, QSsl::Pem, QSsl::PrivateKey, qbaPassPhrase);
+			if (! qskKey.isNull())
+				qCritical("Using private key found in certificate file.");
 		}
 		if (qskKey.isNull())
-			qscCert = QSslCertificate();
-	}
+			qFatal("No private key found in certificate or key file.");
 
-	if (qscCert.isNull() || qskKey.isNull()) {
-		if (! key.isEmpty() || ! crt.isEmpty()) {
-			qFatal("Certificate specified, but failed to load.");
+		QList<QSslCertificate> ql = QSslCertificate::fromData(crt);
+		ql << QSslCertificate::fromData(key);
+		for (int i=0;i<ql.size(); ++i) {
+			const QSslCertificate &c = ql.at(i);
+			if (Server::isKeyForCert(qskKey, c)) {
+				qscCert = c;
+				ql.removeAt(i);
+				break;
+			}
+		}
+		if (qscCert.isNull()) {
+			qFatal("Failed to find certificate matching private key.");
+		}
+		if (ql.size() > 0) {
+			QSslSocket::addDefaultCaCertificates(ql);
+			qCritical("Adding %d CA certificates from certificate file.", ql.size());
 		}
 	}
 
+	if (! QSslSocket::supportsSsl()) {
+		qFatal("Qt without SSL Support");
+	}
+
+	QList<QSslCipher> pref;
+	foreach(QSslCipher c, QSslSocket::defaultCiphers()) {
+		if (c.usedBits() < 128)
+			continue;
+		pref << c;
+	}
+	if (pref.isEmpty())
+		qFatal("No SSL ciphers of at least 128 bit found");
+	QSslSocket::setDefaultCiphers(pref);
+
 	qmConfig.clear();
-	qmConfig.insert(QLatin1String("host"),qhaBind.toString());
+	QStringList hosts;
+	foreach(const QHostAddress &qha, qlBind) {
+		hosts << qha.toString();
+	}
+	qmConfig.insert(QLatin1String("host"),hosts.join(" "));
 	qmConfig.insert(QLatin1String("password"),qsPassword);
 	qmConfig.insert(QLatin1String("port"),QString::number(usPort));
 	qmConfig.insert(QLatin1String("timeout"),QString::number(iTimeout));
+	qmConfig.insert(QLatin1String("textmessagelength"), QString::number(iMaxTextMessageLength));
+	qmConfig.insert(QLatin1String("allowhtml"), bAllowHTML ? QLatin1String("true") : QLatin1String("false"));
 	qmConfig.insert(QLatin1String("bandwidth"),QString::number(iMaxBandwidth));
 	qmConfig.insert(QLatin1String("users"),QString::number(iMaxUsers));
 	qmConfig.insert(QLatin1String("defaultchannel"),QString::number(iDefaultChan));
@@ -276,11 +375,13 @@ void MetaParams::read(QString fname) {
 	qmConfig.insert(QLatin1String("registerpassword"),qsRegPassword);
 	qmConfig.insert(QLatin1String("registerhostname"),qsRegHost);
 	qmConfig.insert(QLatin1String("registerurl"),qurlRegWeb.toString());
+	qmConfig.insert(QLatin1String("bonjour"), bBonjour ? QLatin1String("true") : QLatin1String("false"));
 	qmConfig.insert(QLatin1String("certificate"),qscCert.toPem());
 	qmConfig.insert(QLatin1String("key"),qskKey.toPem());
 	qmConfig.insert(QLatin1String("obfuscate"),bObfuscate ? QLatin1String("true") : QLatin1String("false"));
-	qmConfig.insert(QLatin1String("playername"),qrPlayerName.pattern());
+	qmConfig.insert(QLatin1String("username"),qrUserName.pattern());
 	qmConfig.insert(QLatin1String("channelname"),qrChannelName.pattern());
+	qmConfig.insert(QLatin1String("certrequired"), bCertRequired ? QLatin1String("true") : QLatin1String("false"));
 }
 
 Meta::Meta() {
@@ -330,6 +431,10 @@ bool Meta::boot(int srvnum) {
 	if (! ServerDB::serverExists(srvnum))
 		return false;
 	Server *s = new Server(srvnum, this);
+	if (! s->bValid) {
+		delete s;
+		return false;
+	}
 	qhServers.insert(srvnum, s);
 	emit started(s);
 	return true;
