@@ -39,12 +39,14 @@
 struct MetaParams {
 	QDir qdBasePath;
 
-	QHostAddress qhaBind;
+	QList<QHostAddress> qlBind;
 	unsigned short usPort;
 	int iTimeout;
 	int iMaxBandwidth;
 	int iMaxUsers;
 	int iDefaultChan;
+	int iMaxTextMessageLength;
+	bool bAllowHTML;
 	QString qsPassword;
 	QString qsWelcomeText;
 
@@ -64,6 +66,8 @@ struct MetaParams {
 	int iLogDays;
 
 	int iObfuscate;
+	bool bSendVersion;
+	bool bAllowPing;
 
 	QString qsDBus;
 	QString qsDBusService;
@@ -75,8 +79,9 @@ struct MetaParams {
 	QString qsRegPassword;
 	QString qsRegHost;
 	QUrl qurlRegWeb;
+	bool bBonjour;
 
-	QRegExp qrPlayerName;
+	QRegExp qrUserName;
 	QRegExp qrChannelName;
 
 	QSslCertificate qscCert;
@@ -87,7 +92,10 @@ struct MetaParams {
 
 	unsigned int uiUid, uiGid;
 
+	QSettings *qsSettings;
+
 	MetaParams();
+	~MetaParams();
 	void read(QString fname = QString("murmur.ini"));
 };
 
