@@ -49,8 +49,8 @@ class ServerI : virtual public Server {
 		                          ::Ice::Int,
 		                          const Ice::Current&);
 
-		virtual void getPlayers_async(const ::Murmur::AMD_Server_getPlayersPtr&,
-		                              const Ice::Current&);
+		virtual void getUsers_async(const ::Murmur::AMD_Server_getUsersPtr&,
+		                            const Ice::Current&);
 
 		virtual void getChannels_async(const ::Murmur::AMD_Server_getChannelsPtr&,
 		                               const Ice::Current&);
@@ -65,10 +65,10 @@ class ServerI : virtual public Server {
 		                           const ::Murmur::BanList&,
 		                           const Ice::Current&);
 
-		virtual void kickPlayer_async(const ::Murmur::AMD_Server_kickPlayerPtr&,
-		                              ::Ice::Int,
-		                              const ::std::string&,
-		                              const Ice::Current&);
+		virtual void kickUser_async(const ::Murmur::AMD_Server_kickUserPtr&,
+		                            ::Ice::Int,
+		                            const ::std::string&,
+		                            const Ice::Current&);
 
 		virtual void sendMessage_async(const ::Murmur::AMD_Server_sendMessagePtr&,
 		                               ::Ice::Int,
@@ -85,7 +85,7 @@ class ServerI : virtual public Server {
 		                            const Ice::Current&);
 
 		virtual void setState_async(const ::Murmur::AMD_Server_setStatePtr&,
-		                            const ::Murmur::Player&,
+		                            const ::Murmur::User&,
 		                            const Ice::Current&);
 
 		virtual void getChannelState_async(const ::Murmur::AMD_Server_getChannelStatePtr&,
@@ -122,33 +122,34 @@ class ServerI : virtual public Server {
 		                          bool,
 		                          const Ice::Current&);
 
-		virtual void getPlayerNames_async(const ::Murmur::AMD_Server_getPlayerNamesPtr&,
-		                                  const ::Murmur::IdList&,
-		                                  const Ice::Current&);
-
-		virtual void getPlayerIds_async(const ::Murmur::AMD_Server_getPlayerIdsPtr&,
-		                                const ::Murmur::NameList&,
+		virtual void getUserNames_async(const ::Murmur::AMD_Server_getUserNamesPtr&,
+		                                const ::Murmur::IdList&,
 		                                const Ice::Current&);
 
-		virtual void registerPlayer_async(const ::Murmur::AMD_Server_registerPlayerPtr&,
-		                                  const ::std::string&,
+		virtual void getUserIds_async(const ::Murmur::AMD_Server_getUserIdsPtr&,
+		                              const ::Murmur::NameList&,
+		                              const Ice::Current&);
+
+		virtual void registerUser_async(const ::Murmur::AMD_Server_registerUserPtr&,
+		                                const Murmur::InfoMap&,
+		                                const Ice::Current&);
+
+		virtual void unregisterUser_async(const ::Murmur::AMD_Server_unregisterUserPtr&,
+		                                  ::Ice::Int,
 		                                  const Ice::Current&);
 
-		virtual void unregisterPlayer_async(const ::Murmur::AMD_Server_unregisterPlayerPtr&,
-		                                    ::Ice::Int,
-		                                    const Ice::Current&);
-
-		virtual void updateregistration_async(const ::Murmur::AMD_Server_updateregistrationPtr&,
-		                                      const ::Murmur::RegisteredPlayer&,
+		virtual void updateRegistration_async(const ::Murmur::AMD_Server_updateRegistrationPtr&,
+		                                      Ice::Int,
+		                                      const Murmur::InfoMap&,
 		                                      const Ice::Current&);
 
 		virtual void getRegistration_async(const ::Murmur::AMD_Server_getRegistrationPtr&,
 		                                   ::Ice::Int,
 		                                   const Ice::Current&);
 
-		virtual void getRegisteredPlayers_async(const ::Murmur::AMD_Server_getRegisteredPlayersPtr&,
-		                                        const ::std::string&,
-		                                        const Ice::Current&);
+		virtual void getRegisteredUsers_async(const ::Murmur::AMD_Server_getRegisteredUsersPtr&,
+		                                      const ::std::string&,
+		                                      const Ice::Current&);
 
 		virtual void verifyPassword_async(const ::Murmur::AMD_Server_verifyPasswordPtr&,
 		                                  const ::std::string&,
