@@ -34,7 +34,7 @@
 #include "Meta.h"
 #include "Server.h"
 #include "ServerDB.h"
-#include "Player.h"
+#include "User.h"
 #include "Channel.h"
 #include "Group.h"
 #include "MurmurIce.h"
@@ -46,26 +46,6 @@ using namespace Murmur;
 static MurmurIce *mi = NULL;
 static Ice::ObjectPtr iopServer;
 
-class IceEvent : public QEvent {
-		Q_DISABLE_COPY(IceEvent);
-	protected:
-		boost::function<void ()> func;
-	public:
-		IceEvent(boost::function<void ()> f) : QEvent(static_cast<QEvent::Type>(ICE_QEVENT)), func(f) {};
-		void execute() {
-			func();
-		};
-};
-
-static inline QString fromStdUtf8String(const ::std::string &str) {
-	return QString::fromUtf8(str.data(), str.length());
-}
-
-static inline ::std::string toStdUtf8String(const QString &str) {
-	const QByteArray &qba = str.toUtf8();
-	return ::std::string(qba.constData(), qba.length());
-}
-
 void IceStart() {
 	mi = new MurmurIce();
 }
@@ -78,46 +58,67 @@ void IceStop() {
 
 static void logToLog(const ServerDB::LogRecord &r, Murmur::LogEntry &le) {
 	le.timestamp = r.first;
-	le.txt = toStdUtf8String(r.second);
+	le.txt = u8(r.second);
 }
 
-static void playerToPlayer(const ::Player *p, Murmur::Player &mp) {
+static void userToUser(const ::User *p, Murmur::User &mp) {
 	mp.session = p->uiSession;
-	mp.playerid = p->iId;
-	mp.name = toStdUtf8String(p->qsName);
+	mp.userid = p->iId;
+	mp.name = u8(p->qsName);
 	mp.mute = p->bMute;
 	mp.deaf = p->bDeaf;
-	mp.suppressed = p->bSuppressed;
+	mp.suppress = p->bSuppress;
 	mp.selfMute = p->bSelfMute;
 	mp.selfDeaf = p->bSelfDeaf;
 	mp.channel = p->cChannel->iId;
+	mp.comment = u8(p->qsComment);
 
-	const User *u=static_cast<const User *>(p);
+	const ServerUser *u=static_cast<const ServerUser *>(p);
 	mp.onlinesecs = u->bwr.onlineSeconds();
 	mp.bytespersec = u->bwr.bandwidth();
+	mp.version = u->uiVersion;
+	mp.release = u8(u->qsRelease);
+	mp.os = u8(u->qsOS);
+	mp.osversion = u8(u->qsOSVersion);
+	mp.identity = u8(u->qsIdentity);
+	mp.context = u->ssContext;
+	mp.idlesecs = u->bwr.idleSeconds();
+
+	mp.tcponly = ! u->bUdp;
+
+	::Murmur::NetAddress addr(16, 0);
+	const Q_IPV6ADDR &a = u->haAddress.qip6;
+	for (int i=0;i<16;++i)
+		addr[i] = a[i];
+
+	mp.address = addr;
 }
 
 static void channelToChannel(const ::Channel *c, Murmur::Channel &mc) {
 	mc.id = c->iId;
-	mc.name = toStdUtf8String(c->qsName);
+	mc.name = u8(c->qsName);
 	mc.parent = c->cParent ? c->cParent->iId : -1;
 	mc.links.clear();
+	if (! c->qsDesc.isEmpty())
+		mc.description = u8(c->qsDesc);
+	mc.position = c->iPosition;
 	foreach(::Channel *chn, c->qsPermLinks)
 		mc.links.push_back(chn->iId);
+	mc.temporary = c->bTemporary;
 }
 
 static void ACLtoACL(const ::ChanACL *acl, Murmur::ACL &ma) {
 	ma.applyHere = acl->bApplyHere;
 	ma.applySubs = acl->bApplySubs;
 	ma.inherited = false;
-	ma.playerid = acl->iPlayerId;
-	ma.group = toStdUtf8String(acl->qsGroup);
+	ma.userid = acl->iUserId;
+	ma.group = u8(acl->qsGroup);
 	ma.allow = acl->pAllow;
 	ma.deny = acl->pDeny;
 }
 
 static void groupToGroup(const ::Group *g, Murmur::Group &mg) {
-	mg.name = toStdUtf8String(g->qsName);
+	mg.name = u8(g->qsName);
 	mg.inherit = g->bInherit;
 	mg.inheritable = g->bInheritable;
 	mg.add.clear();
@@ -125,11 +126,47 @@ static void groupToGroup(const ::Group *g, Murmur::Group &mg) {
 	mg.members.clear();
 }
 
-static void banToBan(const QPair<quint32,int> b, Murmur::Ban &mb) {
-	mb.address = b.first;
-	mb.bits = b.second;
+static void banToBan(const ::Ban &b, Murmur::Ban &mb) {
+	::Murmur::NetAddress addr(16, 0);
+	const Q_IPV6ADDR &a = b.haAddress.qip6;
+	for (int i=0;i<16;++i)
+		addr[i] = a[i];
+
+	mb.address = addr;
+	mb.bits = b.iMask;
+	mb.name = u8(b.qsUsername);
+	mb.hash = u8(b.qsHash);
+	mb.reason = u8(b.qsReason);
+	mb.start = b.qdtStart.toTime_t();
+	mb.duration = b.iDuration;
 }
 
+static void banToBan(const ::Murmur::Ban &mb, ::Ban &b) {
+	if (mb.address.size() != 16)
+		for (int i=0;i<16;++i)
+			b.haAddress.qip6[i] = 0;
+	else
+		for (int i=0;i<16;++i)
+			b.haAddress.qip6[i] = mb.address[i];
+	b.iMask = mb.bits;
+	b.qsUsername = u8(mb.name);
+	b.qsHash = u8(mb.hash);
+	b.qsReason = u8(mb.reason);
+	b.qdtStart = QDateTime::fromTime_t(static_cast<quint32>(mb.start));
+	b.iDuration = mb.duration;
+}
+
+static void infoToInfo(const QMap<int, QString> &info, Murmur::UserInfoMap &im) {
+	QMap<int, QString>::const_iterator i;
+	for (i = info.constBegin(); i != info.constEnd(); ++i)
+		im[static_cast<Murmur::UserInfo>(i.key())] = u8(i.value());
+}
+
+static void infoToInfo(const Murmur::UserInfoMap &im, QMap<int, QString> &info) {
+	Murmur::UserInfoMap::const_iterator i;
+	for (i = im.begin(); i != im.end(); ++i)
+		info.insert((*i).first, u8((*i).second));
+}
 
 class ServerLocator : public virtual Ice::ServantLocator {
 	public:
@@ -155,12 +192,12 @@ MurmurIce::MurmurIce() {
 
 		adapter->activate();
 		foreach(const Ice::EndpointPtr ep, mprx->ice_getEndpoints()) {
-			qWarning("MurmurIce: Endpoint \"%s\" running", qPrintable(fromStdUtf8String(ep->toString())));
+			qWarning("MurmurIce: Endpoint \"%s\" running", qPrintable(u8(ep->toString())));
 		}
 
 		meta->connectListener(this);
 	} catch (Ice::Exception &e) {
-		qCritical("MurmurIce: Initialization failed: %s", qPrintable(fromStdUtf8String(e.ice_name())));
+		qCritical("MurmurIce: Initialization failed: %s", qPrintable(u8(e.ice_name())));
 	}
 }
 
@@ -176,10 +213,8 @@ MurmurIce::~MurmurIce() {
 }
 
 void MurmurIce::customEvent(QEvent *evt) {
-	if (evt->type() == ICE_QEVENT) {
-		IceEvent *ie = static_cast<IceEvent *>(evt);
-		ie->execute();
-	}
+	if (evt->type() == EXEC_QEVENT)
+		static_cast<ExecEvent *>(evt)->execute();
 }
 
 void MurmurIce::badMetaProxy(const ::Murmur::MetaCallbackPrx &prx) {
@@ -203,14 +238,14 @@ void MurmurIce::badAuthenticator(::Server *server) {
 static ServerPrx idToProxy(int id, const Ice::ObjectAdapterPtr &adapter) {
 	Ice::Identity ident;
 	ident.category = "s";
-	ident.name = toStdUtf8String(QString::number(id));
+	ident.name = u8(QString::number(id));
 
 	return ServerPrx::uncheckedCast(adapter->createProxy(ident));
 }
 
 void MurmurIce::started(::Server *s) {
 	s->connectListener(mi);
-	connect(s, SIGNAL(contextAction(const Player *, const QString &, unsigned int, int)), this, SLOT(contextAction(const Player *, const QString &, unsigned int, int)));
+	connect(s, SIGNAL(contextAction(const User *, const QString &, unsigned int, int)), this, SLOT(contextAction(const User *, const QString &, unsigned int, int)));
 
 	const QList< ::Murmur::MetaCallbackPrx> &qmList = qmMetaCallbacks;
 
@@ -245,7 +280,7 @@ void MurmurIce::stopped(::Server *s) {
 	}
 }
 
-void MurmurIce::playerConnected(const ::Player *p) {
+void MurmurIce::userConnected(const ::User *p) {
 	::Server *s = qobject_cast< ::Server *> (sender());
 
 	const QList< ::Murmur::ServerCallbackPrx> &qmList = qmServerCallbacks[s->iServerNum];
@@ -253,19 +288,19 @@ void MurmurIce::playerConnected(const ::Player *p) {
 	if (qmList.isEmpty())
 		return;
 
-	::Murmur::Player mp;
-	playerToPlayer(p, mp);
+	::Murmur::User mp;
+	userToUser(p, mp);
 
 	foreach(const ::Murmur::ServerCallbackPrx &prx, qmList) {
 		try {
-			prx->playerConnected(mp);
+			prx->userConnected(mp);
 		} catch (...) {
 			badServerProxy(prx, s->iServerNum);
 		}
 	}
 }
 
-void MurmurIce::playerDisconnected(const ::Player *p) {
+void MurmurIce::userDisconnected(const ::User *p) {
 	::Server *s = qobject_cast< ::Server *> (sender());
 
 	mi->qmServerContextCallbacks[s->iServerNum].remove(p->uiSession);
@@ -275,19 +310,19 @@ void MurmurIce::playerDisconnected(const ::Player *p) {
 	if (qmList.isEmpty())
 		return;
 
-	::Murmur::Player mp;
-	playerToPlayer(p, mp);
+	::Murmur::User mp;
+	userToUser(p, mp);
 
 	foreach(const ::Murmur::ServerCallbackPrx &prx, qmList) {
 		try {
-			prx->playerDisconnected(mp);
+			prx->userDisconnected(mp);
 		} catch (...) {
 			badServerProxy(prx, s->iServerNum);
 		}
 	}
 }
 
-void MurmurIce::playerStateChanged(const ::Player *p) {
+void MurmurIce::userStateChanged(const ::User *p) {
 	::Server *s = qobject_cast< ::Server *> (sender());
 
 	const QList< ::Murmur::ServerCallbackPrx> &qmList = qmServerCallbacks[s->iServerNum];
@@ -295,12 +330,12 @@ void MurmurIce::playerStateChanged(const ::Player *p) {
 	if (qmList.isEmpty())
 		return;
 
-	::Murmur::Player mp;
-	playerToPlayer(p, mp);
+	::Murmur::User mp;
+	userToUser(p, mp);
 
 	foreach(const ::Murmur::ServerCallbackPrx &prx, qmList) {
 		try {
-			prx->playerStateChanged(mp);
+			prx->userStateChanged(mp);
 		} catch (...) {
 			badServerProxy(prx, s->iServerNum);
 		}
@@ -367,7 +402,7 @@ void MurmurIce::channelStateChanged(const ::Channel *c) {
 	}
 }
 
-void MurmurIce::contextAction(const ::Player *pSrc, const QString &action, unsigned int session, int iChannel) {
+void MurmurIce::contextAction(const ::User *pSrc, const QString &action, unsigned int session, int iChannel) {
 	::Server *s = qobject_cast< ::Server *> (sender());
 
 	QMap<int, QMap<int, QMap<QString, ::Murmur::ServerContextCallbackPrx> > > &qmAll = mi->qmServerContextCallbacks;
@@ -378,20 +413,20 @@ void MurmurIce::contextAction(const ::Player *pSrc, const QString &action, unsig
 	if (! qmServer.contains(pSrc->uiSession))
 		return;
 
-	QMap<QString, ::Murmur::ServerContextCallbackPrx> &qmPlayer = qmServer[pSrc->uiSession];
-	if (! qmPlayer.contains(action))
+	QMap<QString, ::Murmur::ServerContextCallbackPrx> &qmUser = qmServer[pSrc->uiSession];
+	if (! qmUser.contains(action))
 		return;
 
-	const ::Murmur::ServerContextCallbackPrx &prx = qmPlayer[action];
+	const ::Murmur::ServerContextCallbackPrx &prx = qmUser[action];
 
-	::Murmur::Player mp;
-	playerToPlayer(pSrc, mp);
+	::Murmur::User mp;
+	userToUser(pSrc, mp);
 
 	try {
-		prx->contextAction(toStdUtf8String(action), mp, session, iChannel);
+		prx->contextAction(u8(action), mp, session, iChannel);
 	} catch (...) {
 		qCritical("Registered Ice ServerContextCallback %s on server %d, session %d, action %s failed", qPrintable(QString::fromStdString(communicator->proxyToString(prx))), s->iServerNum, pSrc->uiSession, qPrintable(action));
-		qmPlayer.remove(action);
+		qmUser.remove(action);
 	}
 }
 
@@ -400,7 +435,7 @@ void MurmurIce::idToNameSlot(QString &name, int id) {
 
 	ServerAuthenticatorPrx prx = mi->qmServerAuthenticator.value(server->iServerNum);
 	try {
-		name = fromStdUtf8String(prx->idToName(id));
+		name = u8(prx->idToName(id));
 	} catch (...) {
 		badAuthenticator(server);
 	}
@@ -412,7 +447,7 @@ void MurmurIce::idToTextureSlot(QByteArray &qba, int id) {
 	try {
 		const ::Murmur::Texture &tex = prx->idToTexture(id);
 
-		qba.resize(tex.size());
+		qba.resize(static_cast<int>(tex.size()));
 		char *ptr = qba.data();
 		for (unsigned int i=0;i<tex.size();++i)
 			ptr[i] = tex[i];
@@ -426,7 +461,7 @@ void MurmurIce::nameToIdSlot(int &id, const QString &name) {
 
 	ServerAuthenticatorPrx prx = mi->qmServerAuthenticator.value(server->iServerNum);
 	try {
-		id = prx->nameToId(toStdUtf8String(name));
+		id = prx->nameToId(u8(name));
 	} catch (...) {
 		badAuthenticator(server);
 	}
@@ -439,123 +474,103 @@ void MurmurIce::authenticateSlot(int &res, QString &uname, const QString &pw) {
 	::std::string newname;
 	::Murmur::GroupNameList groups;
 	try {
-		res = prx->authenticate(toStdUtf8String(uname), toStdUtf8String(pw), newname, groups);
+		res = prx->authenticate(u8(uname), u8(pw), newname, groups);
 	} catch (...) {
 		badAuthenticator(server);
 	}
 	if (res >= 0) {
 		if (newname.length() > 0)
-			uname = fromStdUtf8String(newname);
+			uname = u8(newname);
 		QStringList qsl;
 		foreach(const ::std::string &str, groups) {
-			qsl << fromStdUtf8String(str);
+			qsl << u8(str);
 		}
 		if (! qsl.isEmpty())
 			server->setTempGroups(res, NULL, qsl);
 	}
 }
 
-void MurmurIce::registerPlayerSlot(int &res, const QString &name) {
+void MurmurIce::registerUserSlot(int &res, const QMap<int, QString> &info) {
 	::Server *server = qobject_cast< ::Server *> (sender());
 
 	ServerUpdatingAuthenticatorPrx prx = mi->qmServerUpdatingAuthenticator.value(server->iServerNum);
 	if (! prx)
 		return;
+
+	::Murmur::UserInfoMap im;
+
+	infoToInfo(info, im);
 	try {
-		res = prx->registerPlayer(toStdUtf8String(name));
+		res = prx->registerUser(im);
 	} catch (...) {
 		badAuthenticator(server);
 	}
 }
 
-void MurmurIce::unregisterPlayerSlot(int &res, int id) {
+void MurmurIce::unregisterUserSlot(int &res, int id) {
 	::Server *server = qobject_cast< ::Server *> (sender());
 
 	ServerUpdatingAuthenticatorPrx prx = mi->qmServerUpdatingAuthenticator.value(server->iServerNum);
 	if (! prx)
 		return;
 	try {
-		res = prx->unregisterPlayer(id);
+		res = prx->unregisterUser(id);
 	} catch (...) {
 		badAuthenticator(server);
 	}
 }
 
-void MurmurIce::getRegistrationSlot(int &res, int id, QString &name, QString &email) {
+void MurmurIce::getRegistrationSlot(int &res, int id, QMap<int, QString> &info) {
 	::Server *server = qobject_cast< ::Server *> (sender());
 
 	ServerUpdatingAuthenticatorPrx prx = mi->qmServerUpdatingAuthenticator.value(server->iServerNum);
 	if (! prx)
 		return;
 
-	::std::string rname;
-	::std::string remail;
+	Murmur::UserInfoMap im;
 	try {
-		res = prx->getRegistration(id, rname, remail);
+		if (prx->getInfo(id, im)) {
+			res = 1;
+			infoToInfo(im, info);
+		}
 	} catch (...) {
 		badAuthenticator(server);
 		return;
 	}
-
-	if (res >= 0) {
-		name = fromStdUtf8String(rname);
-		email = fromStdUtf8String(remail);
-	}
 }
 
-void  MurmurIce::getRegisteredPlayersSlot(const QString &filter, QMap<int, QPair<QString, QString> > &m) {
+void  MurmurIce::getRegisteredUsersSlot(const QString &filter, QMap<int, QString> &m) {
 	::Server *server = qobject_cast< ::Server *> (sender());
 
 	ServerUpdatingAuthenticatorPrx prx = mi->qmServerUpdatingAuthenticator.value(server->iServerNum);
 	if (! prx)
 		return;
 
-	::Murmur::RegisteredPlayerList lst;
+	::Murmur::NameMap lst;
 
 	try {
-		lst = prx->getRegisteredPlayers(toStdUtf8String(filter));
+		lst = prx->getRegisteredUsers(u8(filter));
 	} catch (...) {
 		badAuthenticator(server);
 		return;
 	}
-	foreach(const ::Murmur::RegisteredPlayer &p, lst)
-		m.insert(p.playerid, QPair<QString, QString>(fromStdUtf8String(p.name),fromStdUtf8String(p.email)));
+	::Murmur::NameMap::const_iterator i;
+	for (i=lst.begin(); i != lst.end(); ++i)
+		m.insert((*i).first, u8((*i).second));
 }
 
-void MurmurIce::setNameSlot(int &res, int id, const QString &name) {
+void MurmurIce::setInfoSlot(int &res, int id, const QMap<int, QString> &info) {
 	::Server *server = qobject_cast< ::Server *> (sender());
 
 	ServerUpdatingAuthenticatorPrx prx = mi->qmServerUpdatingAuthenticator.value(server->iServerNum);
 	if (! prx)
 		return;
+
+	Murmur::UserInfoMap im;
+	infoToInfo(info, im);
+
 	try {
-		res = prx->setName(id, toStdUtf8String(name));
-	} catch (...) {
-		badAuthenticator(server);
-	}
-}
-
-void MurmurIce::setEmailSlot(int &res, int id, const QString &email) {
-	::Server *server = qobject_cast< ::Server *> (sender());
-
-	ServerUpdatingAuthenticatorPrx prx = mi->qmServerUpdatingAuthenticator.value(server->iServerNum);
-	if (! prx)
-		return;
-	try {
-		res = prx->setEmail(id, toStdUtf8String(email));
-	} catch (...) {
-		badAuthenticator(server);
-	}
-}
-
-void MurmurIce::setPwSlot(int &res, int id, const QString &pw) {
-	::Server *server = qobject_cast< ::Server *> (sender());
-
-	ServerUpdatingAuthenticatorPrx prx = mi->qmServerUpdatingAuthenticator.value(server->iServerNum);
-	if (! prx)
-		return;
-	try {
-		res = prx->setPassword(id, toStdUtf8String(pw));
+		res = prx->setInfo(id, im);
 	} catch (...) {
 		badAuthenticator(server);
 	}
@@ -603,7 +618,7 @@ Ice::ObjectPtr ServerLocator::locate(const Ice::Current &, Ice::LocalObjectPtr &
 	}
 
 #define NEED_PLAYER \
-	User *user = server->qhUsers.value(session); \
+	ServerUser *user = server->qhUsers.value(session); \
 	if (!user) { \
 		cb->ice_exception(::Murmur::InvalidSessionException()); \
 		return; \
@@ -622,7 +637,7 @@ Ice::ObjectPtr ServerLocator::locate(const Ice::Current &, Ice::LocalObjectPtr &
 
 void ServerI::ice_ping(const Ice::Current &current) const {
 	// This is executed in the ice thread.
-	int server_id = fromStdUtf8String(current.id.name).toInt();
+	int server_id = u8(current.id.name).toInt();
 	if (! ServerDB::serverExists(server_id))
 		throw ::Ice::ObjectNotExistException(__FILE__, __LINE__);
 }
@@ -720,7 +735,7 @@ static void impl_Server_id(const ::Murmur::AMD_Server_idPtr cb, int server_id) {
 
 static void impl_Server_getConf(const ::Murmur::AMD_Server_getConfPtr cb, int server_id,  const ::std::string& key) {
 	NEED_SERVER_EXISTS;
-	cb->ice_response(toStdUtf8String(ServerDB::getConf(server_id, fromStdUtf8String(key)).toString()));
+	cb->ice_response(u8(ServerDB::getConf(server_id, u8(key)).toString()));
 }
 
 static void impl_Server_getAllConf(const ::Murmur::AMD_Server_getAllConfPtr cb, int server_id) {
@@ -731,15 +746,15 @@ static void impl_Server_getAllConf(const ::Murmur::AMD_Server_getAllConfPtr cb,
 	QMap<QString, QString> values = ServerDB::getAllConf(server_id);
 	QMap<QString, QString>::const_iterator i;
 	for (i=values.constBegin();i != values.constEnd(); ++i) {
-		cm[toStdUtf8String(i.key())] = toStdUtf8String(i.value());
+		cm[u8(i.key())] = u8(i.value());
 	}
 	cb->ice_response(cm);
 }
 
 static void impl_Server_setConf(const ::Murmur::AMD_Server_setConfPtr cb, int server_id,  const ::std::string& key,  const ::std::string& value) {
 	NEED_SERVER_EXISTS;
-	QString k=fromStdUtf8String(key);
-	QString v=fromStdUtf8String(value);
+	QString k=u8(key);
+	QString v=u8(value);
 	ServerDB::setConf(server_id, k, v);
 	if (server)
 		server->setLiveConf(k, v);
@@ -748,7 +763,7 @@ static void impl_Server_setConf(const ::Murmur::AMD_Server_setConfPtr cb, int se
 
 static void impl_Server_setSuperuserPassword(const ::Murmur::AMD_Server_setSuperuserPasswordPtr cb, int server_id,  const ::std::string& pw) {
 	NEED_SERVER_EXISTS;
-	ServerDB::setSUPW(server_id, fromStdUtf8String(pw));
+	ServerDB::setSUPW(server_id, u8(pw));
 	cb->ice_response();
 }
 
@@ -766,13 +781,13 @@ static void impl_Server_getLog(const ::Murmur::AMD_Server_getLogPtr cb, int serv
 	cb->ice_response(ll);
 }
 
-static void impl_Server_getPlayers(const ::Murmur::AMD_Server_getPlayersPtr cb, int server_id) {
+static void impl_Server_getUsers(const ::Murmur::AMD_Server_getUsersPtr cb, int server_id) {
 	NEED_SERVER;
-	::Murmur::PlayerMap pm;
-	::Murmur::Player mp;
-	foreach(const ::Player *p, server->qhUsers) {
-		if (p->sState == ::Player::Authenticated) {
-			playerToPlayer(p, mp);
+	::Murmur::UserMap pm;
+	::Murmur::User mp;
+	foreach(const ::User *p, server->qhUsers) {
+		if (static_cast<const ServerUser *>(p)->sState == ::ServerUser::Authenticated) {
+			userToUser(p, mp);
 			pm[p->uiSession] = mp;
 		}
 	}
@@ -790,24 +805,24 @@ static void impl_Server_getChannels(const ::Murmur::AMD_Server_getChannelsPtr cb
 	cb->ice_response(cm);
 }
 
-static bool playerSort(const ::Player *a, const ::Player *b) {
+static bool userSort(const ::User *a, const ::User *b) {
 	return a->qsName < b->qsName;
 }
 
 static bool channelSort(const ::Channel *a, const ::Channel *b) {
-	return a->qsName < b->qsName;
+	return ::Channel::lessThan(a, b);
 }
 
 TreePtr recurseTree(const ::Channel *c) {
 	TreePtr t = new Tree();
 	channelToChannel(c, t->c);
-	QList< ::Player *> players = c->qlPlayers;
-	qSort(players.begin(), players.end(), playerSort);
+	QList< ::User *> users = c->qlUsers;
+	qSort(users.begin(), users.end(), userSort);
 
-	foreach(const ::Player *p, players) {
-		::Murmur::Player mp;
-		playerToPlayer(p, mp);
-		t->players.push_back(mp);
+	foreach(const ::User *p, users) {
+		::Murmur::User mp;
+		userToUser(p, mp);
+		t->users.push_back(mp);
 	}
 
 	QList< ::Channel *> channels = c->qlChannels;
@@ -829,8 +844,7 @@ static void impl_Server_getBans(const ::Murmur::AMD_Server_getBansPtr cb, int se
 	NEED_SERVER;
 	::Murmur::BanList bl;
 	::Murmur::Ban mb;
-	QPair<quint32, int> ban;
-	foreach(ban, server->qlBans) {
+	foreach(const ::Ban &ban, server->qlBans) {
 		banToBan(ban, mb);
 		bl.push_back(mb);
 	}
@@ -841,21 +855,22 @@ static void impl_Server_setBans(const ::Murmur::AMD_Server_setBansPtr cb, int se
 	NEED_SERVER;
 	server->qlBans.clear();
 	foreach(const ::Murmur::Ban &mb, bans) {
-		server->qlBans << QPair<quint32,int>(mb.address,mb.bits);
+		::Ban ban;
+		banToBan(mb, ban);
+		server->qlBans << ban;
 	}
 	server->saveBans();
 	cb->ice_response();
 }
 
-static void impl_Server_kickPlayer(const ::Murmur::AMD_Server_kickPlayerPtr cb, int server_id,  ::Ice::Int session,  const ::std::string& reason) {
+static void impl_Server_kickUser(const ::Murmur::AMD_Server_kickUserPtr cb, int server_id,  ::Ice::Int session,  const ::std::string& reason) {
 	NEED_SERVER;
 	NEED_PLAYER;
 
-	MessagePlayerKick mpk;
-	mpk.uiSession = 0;
-	mpk.uiVictim = session;
-	mpk.qsReason = fromStdUtf8String(reason);
-	server->sendAll(&mpk);
+	MumbleProto::UserRemove mpur;
+	mpur.set_session(session);
+	mpur.set_reason(reason);
+	server->sendAll(mpur);
 	user->disconnectSocket();
 	cb->ice_response();
 }
@@ -864,7 +879,7 @@ static void impl_Server_sendMessage(const ::Murmur::AMD_Server_sendMessagePtr cb
 	NEED_SERVER;
 	NEED_PLAYER;
 
-	server->sendTextMessage(NULL, user, false, fromStdUtf8String(text));
+	server->sendTextMessage(NULL, user, false, u8(text));
 	cb->ice_response();
 }
 
@@ -881,26 +896,25 @@ static void impl_Server_addContextCallback(const Murmur::AMD_Server_addContextCa
 
 	QMap<QString, ::Murmur::ServerContextCallbackPrx> & qmPrx = mi->qmServerContextCallbacks[server_id][session];
 
-	if (!(ctx & (MessageContextAddAction::CtxServer | MessageContextAddAction::CtxChannel | MessageContextAddAction::CtxPlayer))) {
+	if (!(ctx & (MumbleProto::ContextActionAdd_Context_Server | MumbleProto::ContextActionAdd_Context_Channel | MumbleProto::ContextActionAdd_Context_User))) {
 		cb->ice_exception(InvalidCallbackException());
 		return;
 	}
 
 	try {
 		const Murmur::ServerContextCallbackPrx &oneway = Murmur::ServerContextCallbackPrx::checkedCast(cbptr->ice_oneway()->ice_connectionCached(false));
-		qmPrx.insert(fromStdUtf8String(action), oneway);
+		qmPrx.insert(u8(action), oneway);
 		cb->ice_response();
 	} catch (...) {
 		cb->ice_exception(InvalidCallbackException());
 		return;
 	}
 
-	MessageContextAddAction mcaa;
-	mcaa.uiSession = 0;
-	mcaa.qsAction = fromStdUtf8String(action);
-	mcaa.qsText = fromStdUtf8String(text);
-	mcaa.ctx = static_cast<MessageContextAddAction::Context>(ctx);
-	server->sendMessage(user, &mcaa);
+	MumbleProto::ContextActionAdd mpcaa;
+	mpcaa.set_action(action);
+	mpcaa.set_text(text);
+	mpcaa.set_context(ctx);
+	server->sendMessage(user, mpcaa);
 }
 
 static void impl_Server_removeContextCallback(const Murmur::AMD_Server_removeContextCallbackPtr cb, int server_id, const Murmur::ServerContextCallbackPrx& cbptr) {
@@ -927,19 +941,19 @@ static void impl_Server_getState(const ::Murmur::AMD_Server_getStatePtr cb, int
 	NEED_SERVER;
 	NEED_PLAYER;
 
-	::Murmur::Player mp;
-	playerToPlayer(user, mp);
+	::Murmur::User mp;
+	userToUser(user, mp);
 	cb->ice_response(mp);
 }
 
-static void impl_Server_setState(const ::Murmur::AMD_Server_setStatePtr cb, int server_id,  const ::Murmur::Player& state) {
+static void impl_Server_setState(const ::Murmur::AMD_Server_setStatePtr cb, int server_id,  const ::Murmur::User& state) {
 	int session = state.session;
 	::Channel *channel;
 	NEED_SERVER;
 	NEED_PLAYER;
 	NEED_CHANNEL_VAR(channel, state.channel);
 
-	server->setPlayerState(user, channel, state.mute, state.deaf, state.suppressed);
+	server->setUserState(user, channel, state.mute, state.deaf, state.suppress, u8(state.comment));
 	cb->ice_response();
 }
 
@@ -947,7 +961,7 @@ static void impl_Server_sendMessageChannel(const ::Murmur::AMD_Server_sendMessag
 	NEED_SERVER;
 	NEED_CHANNEL;
 
-	server->sendTextMessage(channel, NULL, tree, fromStdUtf8String(text));
+	server->sendTextMessage(channel, NULL, tree, u8(text));
 	cb->ice_response();
 }
 
@@ -967,7 +981,7 @@ static void impl_Server_setChannelState(const ::Murmur::AMD_Server_setChannelSta
 	::Channel *np;
 	NEED_CHANNEL_VAR(np, state.parent);
 
-	QString qsName = fromStdUtf8String(state.name);
+	QString qsName = u8(state.name);
 
 	QSet< ::Channel *> newset;
 	foreach(int linkid, state.links) {
@@ -976,7 +990,7 @@ static void impl_Server_setChannelState(const ::Murmur::AMD_Server_setChannelSta
 		newset << cLink;
 	}
 
-	if (! server->setChannelState(channel, np, qsName, newset))
+	if (! server->setChannelState(channel, np, qsName, newset, u8(state.description), state.position))
 		cb->ice_exception(::Murmur::InvalidChannelException());
 	else
 		cb->ice_response();
@@ -985,8 +999,13 @@ static void impl_Server_setChannelState(const ::Murmur::AMD_Server_setChannelSta
 static void impl_Server_removeChannel(const ::Murmur::AMD_Server_removeChannelPtr cb, int server_id,  ::Ice::Int channelid) {
 	NEED_SERVER;
 	NEED_CHANNEL;
-	server->removeChannel(channel, NULL);
-	cb->ice_response();
+
+	if (!channel->cParent) {
+		cb->ice_exception(::Murmur::InvalidChannelException());
+	} else {
+		server->removeChannel(channel);
+		cb->ice_response();
+	}
 }
 
 static void impl_Server_addChannel(const ::Murmur::AMD_Server_addChannelPtr cb, int server_id,  const ::std::string& name,  ::Ice::Int parent) {
@@ -994,18 +1013,17 @@ static void impl_Server_addChannel(const ::Murmur::AMD_Server_addChannelPtr cb,
 	::Channel *p, *nc;
 	NEED_CHANNEL_VAR(p, parent);
 
-	QString qsName = fromStdUtf8String(name);
+	QString qsName = u8(name);
 
 	nc = server->addChannel(p, qsName);
 	server->updateChannel(nc);
 	int newid = nc->iId;
 
-	MessageChannelAdd mca;
-	mca.uiSession = 0;
-	mca.qsName = qsName;
-	mca.iParent = parent;
-	mca.iId = newid;
-	server->sendAll(&mca);
+	MumbleProto::ChannelState mpcs;
+	mpcs.set_channel_id(newid);
+	mpcs.set_parent(parent);
+	mpcs.set_name(name);
+	server->sendAll(mpcs);
 
 	cb->ice_response(newid);
 }
@@ -1091,7 +1109,7 @@ static void impl_Server_setACL(const ::Murmur::AMD_Server_setACLPtr cb, int serv
 
 	channel->bInheritACL = inherit;
 	foreach(const ::Murmur::Group &gi, groups) {
-		QString name = fromStdUtf8String(gi.name);
+		QString name = u8(gi.name);
 		g = new ::Group(channel, name);
 		g->bInherit = gi.inherit;
 		g->bInheritable = gi.inheritable;
@@ -1103,118 +1121,98 @@ static void impl_Server_setACL(const ::Murmur::AMD_Server_setACLPtr cb, int serv
 		acl = new ChanACL(channel);
 		acl->bApplyHere = ai.applyHere;
 		acl->bApplySubs = ai.applySubs;
-		acl->iPlayerId = ai.playerid;
-		acl->qsGroup = fromStdUtf8String(ai.group);
-		acl->pDeny = static_cast<ChanACL::Permissions>(ai.deny);
-		acl->pAllow = static_cast<ChanACL::Permissions>(ai.allow);
+		acl->iUserId = ai.userid;
+		acl->qsGroup = u8(ai.group);
+		acl->pDeny = static_cast<ChanACL::Permissions>(ai.deny) & ChanACL::All;
+		acl->pAllow = static_cast<ChanACL::Permissions>(ai.allow) & ChanACL::All;
 	}
+
 	server->clearACLCache();
 	server->updateChannel(channel);
 	cb->ice_response();
 }
 
-static void impl_Server_getPlayerNames(const ::Murmur::AMD_Server_getPlayerNamesPtr cb, int server_id,  const ::Murmur::IdList& ids) {
+static void impl_Server_getUserNames(const ::Murmur::AMD_Server_getUserNamesPtr cb, int server_id,  const ::Murmur::IdList& ids) {
 	NEED_SERVER;
 	::Murmur::NameMap nm;
-	foreach(int playerid, ids) {
-		if (! server->qhUserNameCache.contains(playerid)) {
-			QString name=server->getUserName(playerid);
-			if (! name.isEmpty())
-				server->qhUserNameCache.insert(playerid, name);
-		}
-		nm[playerid] = toStdUtf8String(server->qhUserNameCache.value(playerid));
+	foreach(int userid, ids) {
+		nm[userid] = u8(server->getUserName(userid));
 	}
 	cb->ice_response(nm);
 }
 
-static void impl_Server_getPlayerIds(const ::Murmur::AMD_Server_getPlayerIdsPtr cb, int server_id,  const ::Murmur::NameList& names) {
+static void impl_Server_getUserIds(const ::Murmur::AMD_Server_getUserIdsPtr cb, int server_id,  const ::Murmur::NameList& names) {
 	NEED_SERVER;
 	::Murmur::IdMap im;
 	foreach(const string &n, names) {
-		QString name = fromStdUtf8String(n);
-		if (! server->qhUserIDCache.contains(name)) {
-			int playerid = server->getUserID(name);
-			if (playerid != -1)
-				server->qhUserIDCache.insert(name, playerid);
-		}
-		im[n] = server->qhUserIDCache.value(name);
+		QString name = u8(n);
+		im[n] = server->getUserID(name);
 	}
 	cb->ice_response(im);
 }
 
-static void impl_Server_registerPlayer(const ::Murmur::AMD_Server_registerPlayerPtr cb, int server_id,  const ::std::string& name) {
+static void impl_Server_registerUser(const ::Murmur::AMD_Server_registerUserPtr cb, int server_id, const ::Murmur::UserInfoMap &im) {
 	NEED_SERVER;
-	int playerid = server->registerPlayer(fromStdUtf8String(name));
-	if (playerid < 0)
-		cb->ice_exception(InvalidPlayerException());
+
+	QMap<int, QString> info;
+	infoToInfo(im, info);
+
+	int userid = server->registerUser(info);
+	if (userid < 0)
+		cb->ice_exception(InvalidUserException());
 	else
-		cb->ice_response(playerid);
+		cb->ice_response(userid);
 }
 
-static void impl_Server_unregisterPlayer(const ::Murmur::AMD_Server_unregisterPlayerPtr cb, int server_id,  ::Ice::Int playerid) {
+static void impl_Server_unregisterUser(const ::Murmur::AMD_Server_unregisterUserPtr cb, int server_id,  ::Ice::Int userid) {
 	NEED_SERVER;
-	if (! server->unregisterPlayer(playerid))
-		cb->ice_exception(InvalidPlayerException());
+	if (! server->unregisterUser(userid))
+		cb->ice_exception(InvalidUserException());
 	else
 		cb->ice_response();
 }
 
-static void impl_Server_updateregistration(const ::Murmur::AMD_Server_updateregistrationPtr cb, int server_id,  const ::Murmur::RegisteredPlayer& registration) {
+static void impl_Server_updateRegistration(const ::Murmur::AMD_Server_updateRegistrationPtr cb, int server_id,  int id, const ::Murmur::UserInfoMap &im) {
 	NEED_SERVER;
 
-	QString name, email;
-	if (! server->getRegistration(registration.playerid, name, email)) {
-		cb->ice_exception(InvalidPlayerException());
+	if (! server->isUserId(id)) {
+		cb->ice_exception(InvalidUserException());
 		return;
 	}
 
-	QString newname = fromStdUtf8String(registration.name);
-	QString newemail = fromStdUtf8String(registration.email);
-	QString newpw = fromStdUtf8String(registration.pw);
-
-	if ((! newname.isEmpty()) && (newname != name))
-		if (! server->setName(registration.playerid, newname)) {
-			cb->ice_exception(InvalidPlayerException());
-			return;
-		}
-
-	if ((! newemail.isEmpty()) && (newemail != email))
-		server->setEmail(registration.playerid, newemail);
-
-	if ((! newpw.isEmpty()))
-		server->setPW(registration.playerid, newpw);
+	QMap<int, QString> info;
+	infoToInfo(im, info);
 
+	if (! server->setInfo(id, info)) {
+		cb->ice_exception(InvalidUserException());
+		return;
+	}
 	cb->ice_response();
 }
 
-static void impl_Server_getRegistration(const ::Murmur::AMD_Server_getRegistrationPtr cb, int server_id,  ::Ice::Int playerid) {
+static void impl_Server_getRegistration(const ::Murmur::AMD_Server_getRegistrationPtr cb, int server_id,  ::Ice::Int userid) {
 	NEED_SERVER;
 
-	QString name, email;
-	if (! server->getRegistration(playerid, name, email)) {
-		cb->ice_exception(InvalidPlayerException());
+	QMap<int, QString> info = server->getRegistration(userid);
+
+	if (info.isEmpty()) {
+		cb->ice_exception(InvalidUserException());
 		return;
 	}
 
-	::Murmur::RegisteredPlayer reg;
-	reg.playerid = playerid;
-	reg.name = toStdUtf8String(name);
-	reg.email = toStdUtf8String(email);
-	cb->ice_response(reg);
+	Murmur::UserInfoMap im;
+	infoToInfo(info, im);
+	cb->ice_response(im);
 }
 
-static void impl_Server_getRegisteredPlayers(const ::Murmur::AMD_Server_getRegisteredPlayersPtr cb, int server_id,  const ::std::string& filter) {
+static void impl_Server_getRegisteredUsers(const ::Murmur::AMD_Server_getRegisteredUsersPtr cb, int server_id,  const ::std::string& filter) {
 	NEED_SERVER;
-	Murmur::RegisteredPlayerList rpl;
-	Murmur::RegisteredPlayer reg;
+	Murmur::NameMap rpl;
 
-	const QMap<int, QPair<QString, QString> > l = server->getRegisteredPlayers(fromStdUtf8String(filter));
-	QMap<int, QPair<QString, QString > >::const_iterator i;
+	const QMap<int, QString> l = server->getRegisteredUsers(u8(filter));
+	QMap<int, QString>::const_iterator i;
 	for (i = l.constBegin(); i != l.constEnd(); ++i) {
-		reg.playerid = i.key();
-		reg.name = toStdUtf8String(i.value().first);
-		reg.email = toStdUtf8String(i.value().second);
-		rpl.push_back(reg);
+		rpl[i.key()] = u8(i.value());
 	}
 
 	cb->ice_response(rpl);
@@ -1222,19 +1220,19 @@ static void impl_Server_getRegisteredPlayers(const ::Murmur::AMD_Server_getRegis
 
 static void impl_Server_verifyPassword(const ::Murmur::AMD_Server_verifyPasswordPtr cb, int server_id,  const ::std::string& name,  const ::std::string& pw) {
 	NEED_SERVER;
-	QString uname = fromStdUtf8String(name);
-	cb->ice_response(server->authenticate(uname, fromStdUtf8String(pw)));
+	QString uname = u8(name);
+	cb->ice_response(server->authenticate(uname, u8(pw)));
 }
 
-static void impl_Server_getTexture(const ::Murmur::AMD_Server_getTexturePtr cb, int server_id,  ::Ice::Int playerid) {
+static void impl_Server_getTexture(const ::Murmur::AMD_Server_getTexturePtr cb, int server_id,  ::Ice::Int userid) {
 	NEED_SERVER;
 
-	if (! server->isPlayerId(playerid)) {
-		cb->ice_exception(InvalidPlayerException());
+	if (! server->isUserId(userid)) {
+		cb->ice_exception(InvalidUserException());
 		return;
 	}
 
-	const QByteArray &qba = server->getUserTexture(playerid);
+	const QByteArray &qba = server->getUserTexture(userid);
 
 	::Murmur::Texture tex;
 	tex.resize(qba.size());
@@ -1245,19 +1243,19 @@ static void impl_Server_getTexture(const ::Murmur::AMD_Server_getTexturePtr cb,
 	cb->ice_response(tex);
 }
 
-static void impl_Server_setTexture(const ::Murmur::AMD_Server_setTexturePtr cb, int server_id,  ::Ice::Int playerid,  const ::Murmur::Texture& tex) {
+static void impl_Server_setTexture(const ::Murmur::AMD_Server_setTexturePtr cb, int server_id,  ::Ice::Int userid,  const ::Murmur::Texture& tex) {
 	NEED_SERVER;
 
-	if (! server->isPlayerId(playerid)) {
-		cb->ice_exception(InvalidPlayerException());
+	if (! server->isUserId(userid)) {
+		cb->ice_exception(InvalidUserException());
 		return;
 	}
 
-	QByteArray qba(tex.size(), 0);
+	QByteArray qba(static_cast<int>(tex.size()), 0);
 	char *ptr = qba.data();
 	for (unsigned int i=0;i<tex.size();++i)
 		ptr[i] = tex[i];
-	if (! server->setTexture(playerid, qba))
+	if (! server->setTexture(userid, qba))
 		cb->ice_exception(InvalidTextureException());
 	else
 		cb->ice_response();
@@ -1287,7 +1285,7 @@ static void impl_Meta_getDefaultConf(const ::Murmur::AMD_Meta_getDefaultConfPtr
 	::Murmur::ConfigMap cm;
 	QMap<QString, QString>::const_iterator i;
 	for (i=meta->mp.qmConfig.constBegin();i != meta->mp.qmConfig.constEnd(); ++i) {
-		cm[toStdUtf8String(i.key())] = toStdUtf8String(i.value());
+		cm[u8(i.key())] = u8(i.value());
 	}
 	cb->ice_response(cm);
 }
@@ -1304,7 +1302,7 @@ static void impl_Meta_getVersion(const ::Murmur::AMD_Meta_getVersionPtr cb, cons
 	int major, minor, patch;
 	QString txt;
 	::Meta::getVersion(major, minor, patch, txt);
-	cb->ice_response(major, minor, patch, toStdUtf8String(txt));
+	cb->ice_response(major, minor, patch, u8(txt));
 }
 
 static void impl_Meta_addCallback(const Murmur::AMD_Meta_addCallbackPtr cb, const Ice::ObjectAdapterPtr, const Murmur::MetaCallbackPrx& cbptr) {
