@@ -1,4 +1,5 @@
 /* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+   Copyright (C) 2009, Stefan Hacker <dd0t@users.sourceforge.net>
 
    All rights reserved.
 
@@ -34,10 +35,16 @@
 #include "murmur_pch.h"
 #include "Message.h"
 #include "Timer.h"
-#include "Player.h"
+#include "User.h"
 #include "Connection.h"
+#include "Net.h"
+#include "ACL.h"
 #include "DBus.h"
 
+#ifdef USE_BONJOUR
+#include "BonjourServer.h"
+#endif
+
 class Channel;
 class PacketDataStream;
 
@@ -60,6 +67,7 @@ struct BandwidthRecord {
 	void addFrame(int size);
 	int bytesPerSec() const;
 	int onlineSeconds() const;
+	int idleSeconds() const;
 	int bandwidth() const;
 };
 
@@ -88,55 +96,133 @@ class SslServer : public QTcpServer {
 
 class Server;
 
-class User : public Connection, public Player {
+struct WhisperTarget {
+	struct Channel {
+		int iId;
+		bool bChildren;
+		bool bLinks;
+		QString qsGroup;
+	};
+	QList<unsigned int> qlSessions;
+	QList<WhisperTarget::Channel> qlChannels;
+};
+
+#define EXEC_QEVENT (QEvent::User + 959)
+
+class ExecEvent : public QEvent {
+		Q_DISABLE_COPY(ExecEvent);
+	protected:
+		boost::function<void ()> func;
+	public:
+		ExecEvent(boost::function<void ()>);
+		void execute();
+};
+
+class ServerUser : public Connection, public User {
 	private:
 		Q_OBJECT
-		Q_DISABLE_COPY(User)
+		Q_DISABLE_COPY(ServerUser)
 	protected:
 		Server *s;
 	public:
+		enum State { Connected, Authenticated };
+		State sState;
+		operator const QString() const;
+
+		double dUDPPingAvg, dUDPPingVar;
+		double dTCPPingAvg, dTCPPingVar;
+		quint64 uiUDPPackets, uiTCPPackets;
+
+		unsigned int uiVersion;
+		QString qsRelease;
+		QString qsOS;
+		QString qsOSVersion;
+
+		std::string ssContext;
+		QString qsIdentity;
+
+		bool bVerified;
+		QStringList qslEmail;
+
+		HostAddress haAddress;
+		bool bUdp;
+
+		QList<int> qlCodecs;
+
+		QStringList qslAccessTokens;
+
+		QMap<int, WhisperTarget> qmTargets;
+		typedef QPair<QSet<ServerUser *>, QSet<ServerUser *> > TargetCache;
+		QMap<int, TargetCache> qmTargetCache;
+		QMap<QString, QString> qmWhisperRedirect;
+
+		int iLastPermissionCheck;
+		QMap<int, unsigned int> qmPermissionSent;
+#ifdef Q_OS_UNIX
+		int sUdpSocket;
+#else
+		SOCKET sUdpSocket;
+#endif
 		BandwidthRecord bwr;
-		struct sockaddr_in saiUdpAddress;
-		User(Server *parent, QSslSocket *socket);
+		struct sockaddr_storage saiUdpAddress;
+		ServerUser(Server *parent, QSslSocket *socket);
 };
 
-class Server : public QThread, public MessageHandler {
+class Server : public QThread {
 	private:
 		Q_OBJECT;
 		Q_DISABLE_COPY(Server);
 	protected:
 		bool bRunning;
 
+#ifdef USE_BONJOUR
+		BonjourServer *bsRegistration;
+#endif
 		void startThread();
 		void stopThread();
 
+		void customEvent(QEvent *evt);
 		// Former ServerParams
 	public:
-		QHostAddress qhaBind;
+		QList<QHostAddress> qlBind;
 		unsigned short usPort;
 		int iTimeout;
 		int iMaxBandwidth;
 		int iMaxUsers;
 		int iDefaultChan;
+		int iMaxTextMessageLength;
+		bool bAllowHTML;
 		QString qsPassword;
 		QString qsWelcomeText;
+		bool bCertRequired;
 
 		QString qsRegName;
 		QString qsRegPassword;
 		QString qsRegHost;
 		QUrl qurlRegWeb;
+		bool bBonjour;
+		bool bAllowPing;
 
-		QRegExp qrPlayerName;
+		QRegExp qrUserName;
 		QRegExp qrChannelName;
 
+		QList<QSslCertificate> qlCA;
 		QSslCertificate qscCert;
 		QSslKey qskKey;
-		QByteArray qbaPassPhrase;
 
 		bool bValid;
 
 		void readParams();
 
+		int iCodecAlpha;
+		int iCodecBeta;
+		bool bPreferAlpha;
+		void recheckCodecVersions();
+
+#ifdef USE_BONJOUR
+		void initBonjour();
+		void removeBonjour();
+#endif
 		// Registration, implementation in Register.cpp
 		QTimer qtTick;
 		QHttp *http;
@@ -150,73 +236,88 @@ class Server : public QThread, public MessageHandler {
 
 		// Certificate stuff, implemented partially in Cert.cpp
 	public:
+		static bool isKeyForCert(const QSslKey &key, const QSslCertificate &cert);
 		void initializeCert();
 		const QString getDigest() const;
 
 	public slots:
 		void newClient();
-		void connectionClosed(QString);
+		void connectionClosed(QAbstractSocket::SocketError, const QString &);
 		void sslError(const QList<QSslError> &);
-		void message(QByteArray &, Connection *cCon = NULL);
+		void message(unsigned int, const QByteArray &, ServerUser *cCon = NULL);
 		void checkTimeout();
 		void tcpTransmitData(QByteArray, unsigned int);
 		void doSync(unsigned int);
+		void encrypted();
+		void udpActivated(int);
 	signals:
 		void reqSync(unsigned int);
 		void tcpTransmit(QByteArray, unsigned int id);
 	public:
 		int iServerNum;
 		QQueue<int> qqIds;
-		SslServer *qtsServer;
+		QList<SslServer *> qlServer;
 		QTimer *qtTimeout;
 
 #ifdef Q_OS_UNIX
-		int sUdpSocket;
+		int aiNotify[2];
+		QList<int> qlUdpSocket;
 #else
-		SOCKET sUdpSocket;
+		HANDLE hNotify;
+		QList<SOCKET> qlUdpSocket;
 #endif
+		quint32 uiVersionBlob;
+		QList<QSocketNotifier *> qlUdpNotifier;
 
-		QHash<unsigned int, User *> qhUsers;
-		QHash<quint64, User *> qhPeerUsers;
-		QHash<unsigned int, QSet<User *> > qhHostUsers;
+		QHash<unsigned int, ServerUser *> qhUsers;
+		QHash<QPair<HostAddress, quint16>, ServerUser *> qhPeerUsers;
+		QHash<HostAddress, QSet<ServerUser *> > qhHostUsers;
 		QHash<unsigned int, Channel *> qhChannels;
 		QReadWriteLock qrwlUsers;
 		ChanACL::ACLCache acCache;
 		QMutex qmCache;
-		QHash<int, QByteArray> qhUserTextureCache;
 		QHash<int, QString> qhUserNameCache;
 		QHash<QString, int> qhUserIDCache;
 
-		QList<QPair<quint32, int> > qlBans;
+		QList<Ban> qlBans;
 
-		void processMsg(PacketDataStream &pds, Connection *cCon);
-		void sendMessage(User *u, const char *data, int len, QByteArray &cache);
-		void fakeUdpPacket(Message *msg, Connection *source);
+		void processMsg(ServerUser *u, const char *data, int len);
+		void sendMessage(ServerUser *u, const char *data, int len, QByteArray &cache, bool force = false);
 		void run();
 
 		bool validateChannelName(const QString &name);
-		bool validatePlayerName(const QString &name);
+		bool validateUserName(const QString &name);
 
-		bool checkDecrypt(User *u, const char *encrypted, char *plain, unsigned int cryptlen);
+		bool checkDecrypt(ServerUser *u, const char *encrypted, char *plain, unsigned int cryptlen);
 
-		bool hasPermission(Player *p, Channel *c, ChanACL::Perm perm);
-		void clearACLCache(Player *p = NULL);
+		bool hasPermission(ServerUser *p, Channel *c, QFlags<ChanACL::Perm> perm);
+		void sendClientPermission(ServerUser *u, Channel *c, bool updatelast = false);
+		void flushClientPermissionCache(ServerUser *u, MumbleProto::PermissionQuery &mpqq);
+		void clearACLCache(User *p = NULL);
 
-		void sendAll(Message *);
-		void sendExcept(Message *, Connection *);
-		void sendMessage(Connection *, Message *);
-		void sendChannelDescription(Player *, const Channel *);
-		void sendChannelDescriptionUpdate(const Channel *changed, const Channel *current=NULL);
+		void sendProtoAll(const ::google::protobuf::Message &msg, unsigned int msgType);
+		void sendProtoExcept(ServerUser *, const ::google::protobuf::Message &msg, unsigned int msgType);
+		void sendProtoMessage(ServerUser *, const ::google::protobuf::Message &msg, unsigned int msgType);
+
+#define MUMBLE_MH_MSG(x) \
+		void sendAll(const MumbleProto:: x &msg) { sendProtoAll(msg, MessageHandler:: x); } \
+		void sendExcept(ServerUser *u, const MumbleProto:: x &msg) { sendProtoExcept(u, msg, MessageHandler:: x); } \
+		void sendMessage(ServerUser *u, const MumbleProto:: x &msg) { sendProtoMessage(u, msg, MessageHandler:: x); }
+
+		MUMBLE_MH_ALL
+#undef MUMBLE_MH_MSG
 
 		void setLiveConf(const QString &key, const QString &value);
 
-		QString addressToString(const QHostAddress &);
+		QString addressToString(const QHostAddress &, unsigned short port);
 
 		void log(const QString &);
-		void log(User *u, const QString &);
+		void log(ServerUser *u, const QString &);
 
-		void removeChannel(Channel *c, Player *src, Channel *dest = NULL);
-		void playerEnterChannel(Player *u, Channel *c, bool quiet = false);
+		void removeChannel(int id);
+		void removeChannel(Channel *c, Channel *dest = NULL);
+		void userEnterChannel(User *u, Channel *c, bool quiet = false, bool ignoretemp = false);
+		bool unregisterUser(int id);
 
 		Server(int snum, QObject *parent = NULL);
 		~Server();
@@ -226,59 +327,54 @@ class Server : public QThread, public MessageHandler {
 		void disconnectAuthenticator(QObject *p);
 		void connectListener(QObject *p);
 		void disconnectListener(QObject *p);
-		void setTempGroups(const int playerid, Channel *cChannel, const QStringList &groups);
+		void setTempGroups(const int userid, Channel *cChannel, const QStringList &groups);
 	signals:
-		void registerPlayerSig(int &, const QString &);
-		void unregisterPlayerSig(int &, int);
-		void getRegisteredPlayersSig(const QString &, QMap<int, QPair<QString, QString> > &);
-		void getRegistrationSig(int &, int, QString &, QString &);
-		void authenticateSig(int &, QString &, const QString &);
-		void setPwSig(int &, int, const QString &);
-		void setEmailSig(int &, int, const QString &);
-		void setNameSig(int &, int, const QString &);
+		void registerUserSig(int &, const QMap<int, QString> &);
+		void unregisterUserSig(int &, int);
+		void getRegisteredUsersSig(const QString &, QMap<int, QString > &);
+		void getRegistrationSig(int &, int, QMap<int, QString> &);
+		void authenticateSig(int &, QString &, const QList<QSslCertificate> &, const QString &, bool, const QString &);
+		void setInfoSig(int &, int, const QMap<int, QString> &);
 		void setTextureSig(int &, int, const QByteArray &);
 		void idToNameSig(QString &, int);
 		void nameToIdSig(int &, const QString &);
 		void idToTextureSig(QByteArray &, int);
 
-		void playerStateChanged(const Player *);
-		void playerConnected(const Player *);
-		void playerDisconnected(const Player *);
+		void userStateChanged(const User *);
+		void userConnected(const User *);
+		void userDisconnected(const User *);
 		void channelStateChanged(const Channel *);
 		void channelCreated(const Channel *);
 		void channelRemoved(const Channel *);
 
-		void contextAction(const Player *, const QString &, unsigned int, int);
+		void contextAction(const User *, const QString &, unsigned int, int);
 	public:
-		void setPlayerState(Player *p, Channel *parent, bool mute, bool deaf, bool suppressed);
-		bool setChannelState(Channel *c, Channel *parent, const QString &qsName, const QSet<Channel *> &links);
-		void sendTextMessage(Channel *cChannel, User *pPlayer, bool tree, const QString &text);
+		void setUserState(User *p, Channel *parent, bool mute, bool deaf, bool suppressed, const QString &comment = QString());
+		bool setChannelState(Channel *c, Channel *parent, const QString &qsName, const QSet<Channel *> &links, const QString &desc = QString(), const int position = 0);
+		void sendTextMessage(Channel *cChannel, ServerUser *pUser, bool tree, const QString &text);
 
 		// Database / DBus functions. Implementation in ServerDB.cpp
 		void initialize();
-		typedef QPair<quint32, int> qpBan;
-		int authenticate(QString &name, const QString &pw);
-		Channel *addChannel(Channel *c, const QString &name, const QString &desc = QString());
-		void removeChannel(const Channel *c);
+		int authenticate(QString &name, const QString &pw, const QStringList &emails = QStringList(), const QString &certhash = QString(), bool bStrongCert = false, const QList<QSslCertificate> & = QList<QSslCertificate>());
+		Channel *addChannel(Channel *c, const QString &name, bool temporary = false, int position = 0);
+		void removeChannelDB(const Channel *c);
 		void readChannels(Channel *p = NULL);
 		void readLinks();
 		void updateChannel(const Channel *c);
 		void readChannelPrivs(Channel *c);
-		void setLastChannel(const Player *u);
+		void setLastChannel(const User *u);
 		int readLastChannel(int id);
 		void dumpChannel(const Channel *c);
 		int getUserID(const QString &name);
 		QString getUserName(int id);
 		QByteArray getUserTexture(int id);
-		int registerPlayer(const QString &name);
-		bool unregisterPlayer(int id);
-		QMap<int, QPair<QString, QString> > getRegisteredPlayers(const QString &filter);
-		bool setPW(int id, const QString &pw);
-		bool setName(int id, const QString &name);
-		bool setEmail(int id, const QString &email);
+		QMap<int, QString> getRegistration(int id);
+		int registerUser(const QMap<int, QString> &info);
+		bool unregisterUserDB(int id);
+		QMap<int, QString > getRegisteredUsers(const QString &filter = QString());
+		bool setInfo(int id, const QMap<int, QString> &info);
 		bool setTexture(int id, const QByteArray &texture);
-		bool isPlayerId(int id);
-		bool getRegistration(int id, QString &name, QString &email);
+		bool isUserId(int id);
 		void addLink(Channel *c, Channel *l);
 		void removeLink(Channel *c, Channel *l);
 		void getBans();
@@ -288,37 +384,9 @@ class Server : public QThread, public MessageHandler {
 		void dblog(const QString &str);
 
 		// From msgHandler. Implementation in Messages.cpp
-		virtual void msgSpeex(Connection *, MessageSpeex *);
-		virtual void msgServerAuthenticate(Connection *, MessageServerAuthenticate *);
-		virtual void msgPing(Connection *, MessagePing *);
-		virtual void msgPingStats(Connection *, MessagePingStats *);
-		virtual void msgServerReject(Connection *, MessageServerReject *);
-		virtual void msgServerSync(Connection *, MessageServerSync *);
-		virtual void msgServerJoin(Connection *, MessageServerJoin *);
-		virtual void msgServerLeave(Connection *, MessageServerLeave *);
-		virtual void msgPlayerMute(Connection *, MessagePlayerMute *);
-		virtual void msgPlayerDeaf(Connection *, MessagePlayerDeaf *);
-		virtual void msgPlayerSelfMuteDeaf(Connection *, MessagePlayerSelfMuteDeaf *);
-		virtual void msgPlayerKick(Connection *, MessagePlayerKick *);
-		virtual void msgPlayerBan(Connection *, MessagePlayerBan *);
-		virtual void msgPlayerMove(Connection *, MessagePlayerMove *);
-		virtual void msgPlayerRename(Connection *, MessagePlayerRename *);
-		virtual void msgChannelAdd(Connection *, MessageChannelAdd *);
-		virtual void msgChannelRemove(Connection *, MessageChannelRemove *);
-		virtual void msgChannelMove(Connection *, MessageChannelMove *);
-		virtual void msgChannelLink(Connection *, MessageChannelLink *);
-		virtual void msgChannelRename(Connection *, MessageChannelRename *);
-		virtual void msgChannelDescUpdate(Connection *, MessageChannelDescUpdate *);
-		virtual void msgServerBanList(Connection *, MessageServerBanList *);
-		virtual void msgTextMessage(Connection *, MessageTextMessage *);
-		virtual void msgPermissionDenied(Connection *, MessagePermissionDenied *);
-		virtual void msgEditACL(Connection *, MessageEditACL *);
-		virtual void msgQueryUsers(Connection *, MessageQueryUsers *);
-		virtual void msgTexture(Connection *, MessageTexture *);
-		virtual void msgCryptSetup(Connection *, MessageCryptSetup *);
-		virtual void msgCryptSync(Connection *, MessageCryptSync *);
-		virtual void msgContextAddAction(Connection *, MessageContextAddAction *);
-		virtual void msgContextAction(Connection *, MessageContextAction *);
+#define MUMBLE_MH_MSG(x) void msg##x(ServerUser *, MumbleProto:: x &);
+		MUMBLE_MH_ALL
+#undef MUMBLE_MH_MSG
 };
 
 
