@@ -152,6 +152,9 @@ int main(int argc, char **argv) {
 	icon.addFile(QLatin1String(":/murmur.64x64.png"));
 	a.setWindowIcon(icon);
 #else
+#ifndef Q_OS_MAC
+	setenv("AVAHI_COMPAT_NOWARN", "1", 1);
+#endif
 	QCoreApplication a(argc, argv);
 	UnixMurmur unixhandler;
 	unixhandler.initialcap();
@@ -243,6 +246,7 @@ int main(int argc, char **argv) {
 			       "default locations.", argv[0]);
 #ifdef Q_OS_UNIX
 		} else if (arg == "-limits") {
+			Meta::mp.read(inifile);
 			unixhandler.setuid();
 			unixhandler.finalcap();
 			LimitTest::testLimits(a);
@@ -398,8 +402,15 @@ int main(int argc, char **argv) {
 #ifdef USE_ICE
 	IceStart();
 #endif
-
 	meta->getOSInfo();
+
+	int major, minor, patch;
+	QString strver;
+
+	meta->getVersion(major, minor, patch, strver);
+
+	qWarning("Murmur %d.%d.%d (%s) running on %s: %s: Booting servers", major, minor, patch, qPrintable(strver), qPrintable(meta->qsOS), qPrintable(meta->qsOSVersion));
+
 	meta->bootAll();
 
 	res=a.exec();
